@@ -16,22 +16,24 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifndef _BaseContentHandler_h_
+#define _BaseContentHandler_h_
+
 #include "nsIProtocolHandler.h"
-#include "nsIContentHandler.h"
 
 #include "nsString.h"
 
-class GBaseProtocolHandler : public nsIProtocolHandler,
-			     public nsIContentHandler
+class GBaseProtocolHandler : public nsIProtocolHandler
 {
   public:
 	NS_DECL_ISUPPORTS
 	NS_DECL_NSIPROTOCOLHANDLER
-	NS_DECL_NSICONTENTHANDLER
 
 	GBaseProtocolHandler (const char *aScheme);
 	virtual ~GBaseProtocolHandler();
 	/* additional members */
+  protected:
 	nsCString mScheme;
-	nsCString mMimeType;
 };
+
+#endif //_BaseContentHandler_h_
\ No newline at end of file
