@@ -147,6 +147,7 @@
 #include "mime.h"
 #include "window.h"
 #include "mozilla.h"
+#include "prefs.h"
 #include "misc.h"
 #include "glade.h"
 #include "downloader.h"
@@ -397,7 +398,7 @@ NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
 		delete mMimeItem;
 		break;
 	 case MIME_SAVE_TO_DISK:
-	 	if (gnome_config_get_int (CONF_DOWNLOADING_DOWNLOADER)
+	 	if (eel_gconf_get_integer (CONF_DOWNLOADING_DOWNLOADER)
 	 	    == DOWNLOADER_BUILTIN)
 	 	{
 			mLauncher->SaveToDisk (nsnull,PR_FALSE);
@@ -443,7 +444,7 @@ NS_IMETHODIMP GContentHandler::
 
 	if (NS_FAILED(rv))
 	{
-		char *dirName = gnome_config_get_string (CONF_DOWNLOADING_DIR);
+		char *dirName = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
 
 		if (!dirName)
 		{
@@ -467,7 +468,7 @@ NS_IMETHODIMP GContentHandler::
 	nsCOMPtr <nsILocalFile> aFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
 
 	PRInt16 okToSave;
-	if (gnome_config_get_bool (CONF_DOWNLOADING_ASK_DIR))
+	if (eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR))
 	{
 		aFilePicker->Show (&okToSave);
 
@@ -543,6 +544,8 @@ NS_IMETHODIMP GContentHandler::FindHelperApp (void)
 
 NS_IMETHODIMP GContentHandler::LaunchHelperApp (void)
 {
+	gchar *tmp_dir = g_get_tmp_dir ();
+
 	if (mMimeItem)
 	{
 #if MOZILLA_VERSION > VERSION3(0,9,2)
@@ -576,14 +579,14 @@ NS_IMETHODIMP GContentHandler::LaunchHelperApp (void)
 			char **argv = g_strsplit (full_command, " ", 100);
 			int i;
 			for (i = 0; argv[i] != NULL; i++);
-			gnome_execute_async ("/tmp", i, argv);
+			gnome_execute_async (tmp_dir, i, argv);
 			g_strfreev (argv);
 		}
 		else
 		{
 			char *argv[3] = { "gnome-terminal", "-e",
 					  full_command };
-			gnome_execute_async ("/tmp", 3, argv);
+			gnome_execute_async (tmp_dir, 3, argv);
 		}
 		g_free (full_command);
 		g_free (aFileName);
@@ -878,7 +881,7 @@ GDownloadProgressListener::
 
 	PRInt64 now = PR_Now ();
 	mLastUpdate = now;
-#if MOZILLA_VERSION > VERSION4(0,9,2,1)
+#if MOZILLA_VERSION > VERSION4(0,9,3,1)
 	mStartTime = mTimeDownloadStarted;
 #else
 	mStartTime = now;
@@ -902,11 +905,14 @@ GDownloadProgressListener::
 	dialog_set_parent (mProgressDialog, aParentWidget);
 
 	aLauncher->SetWebProgressListener (this);
+
+	// Increment window count while progress is under way.
+	window_count++;
 }
 
 GDownloadProgressListener::~GDownloadProgressListener()
 {
-  /* destructor code */
+	/* destructor code */
 }
 
 /* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long aStateFlags, in unsigned long aStatus); */
@@ -922,6 +928,16 @@ NS_IMETHODIMP GDownloadProgressListener::
 		mLauncher->CloseProgressWindow ();
 		if (mUsedByHelper)
 			mHandler->LaunchHelperApp ();
+
+		// Decrement window count as this progess is finished.
+		window_count--;
+
+		// If there are no more windows left, exit galeon.
+		if (g_list_length (all_windows) == 0 && !galeon_server_mode
+		    && window_count ==0)
+		{
+			galeon_exit ();
+		}
 	}
 
 	return NS_OK;
@@ -978,9 +994,16 @@ NS_IMETHODIMP GDownloadProgressListener::
 	{
 		gfloat progress = (gfloat)aCurTotalProgress/
 				  (gfloat)aMaxTotalProgress;
+		gchar *strper = g_strdup_printf (_("%.2f%% - Downloading"), 
+						 progress*100);
+
+		if (progress > 0 && progress < 1)
+			gtk_window_set_title (GTK_WINDOW(mProgressDialog), 
+					      strper);
 		gtk_progress_set_percentage (GTK_PROGRESS(mProgressBar),
 					     progress);
 		totalStr = g_strdup_printf ("%d",totalKBytes);
+		g_free (strper);
 	}
 	else
 	{
@@ -1163,7 +1186,7 @@ mime_ask_dialog_ok_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 	switch (action)
 	{
 	 case MIME_SAVE_TO_DISK:
-	 	if (gnome_config_get_int (CONF_DOWNLOADING_DOWNLOADER)
+	 	if (eel_gconf_get_integer (CONF_DOWNLOADING_DOWNLOADER)
 	 	    == DOWNLOADER_BUILTIN)
 	 	{
 			aLauncher->SaveToDisk (nsnull,PR_FALSE);
@@ -1229,6 +1252,11 @@ helper_chooser_ok_clicked_cb (GtkButton *button,
 	{
 		GtkWidget *menu = GTK_OPTION_MENU (dialog->mOptionMenu)->menu;
 		GList *menuItems = GTK_MENU_SHELL (menu)->children;
+		if (!menuItems)
+		{
+			helper_chooser_cancel_clicked_cb (button, dialog);
+			return;
+		}
 		gpointer item = gtk_menu_get_active (GTK_MENU(menu));
 		gint index = g_list_index (menuItems,item);
 		GList *helperApps = g_list_nth(dialog->mHelperApps,index);
@@ -1239,6 +1267,11 @@ helper_chooser_ok_clicked_cb (GtkButton *button,
 		helperApp = new GnomeVFSMimeApplication;
 		helperApp->command = g_strdup (gtk_entry_get_text(
 						GTK_ENTRY(dialog->mHelperEntry)));
+		if (!helperApp->command)
+		{
+			helper_chooser_cancel_clicked_cb (button, dialog);
+			return;
+		}
 		helperApp->id = g_strdup ("OtHeR");
 		helperApp->requires_terminal = gtk_toggle_button_get_active (
 				GTK_TOGGLE_BUTTON(dialog->mCheckUseTerminal));
@@ -1246,7 +1279,7 @@ helper_chooser_ok_clicked_cb (GtkButton *button,
 					GTK_TOGGLE_BUTTON(dialog->mCheckUseUrl)) ?
 					GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS :
 					GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_PATHS;
-	};
+	}
 
 	PRBool alwaysUse = gtk_toggle_button_get_active (
 				GTK_TOGGLE_BUTTON(dialog->mCheckAlwaysUse)) ?
