@@ -22,6 +22,14 @@
 #include "nsIHelperAppLauncherDialog.h"
 #include "nsError.h"
 
+#define G_CONTENTHANDLER_CID			     \
+{ /* 16072c4a-23a6-4996-9beb-9335c06bbeae */         \
+    0x16072c4a,                                      \
+    0x23a6,                                          \
+    0x4996,                                          \
+    {0x9b, 0xeb, 0x93, 0x35, 0xc0, 0x6b, 0xbe, 0xae} \
+}
+
 class nsIFactory;
 
 extern nsresult NS_NewContentHandlerFactory(nsIFactory** aFactory);
