@@ -25,7 +25,7 @@
 //
 // GContentOverrideHandler
 // 
-#define GALEON_CONTENTOVERRIDEHANDLER_CID \
+#define G_CONTENTOVERRIDEHANDLER_CID \
    {0x091140b8, 0x6266, 0x438a, {0x8c, 0xf0, 0x4b, 0xd7, 0xb9, 0xe, 0x05, 0xd5}}
 
 class nsIFactory;
