@@ -21,11 +21,13 @@
 #include "misc.h"
 #include "window.h"
 #include "mozilla.h"
+#include "prefs.h"
 #include "dialog.h"
+#include "eel-gconf-extensions.h"
 
 #include <gtk/gtkmain.h>
 #include <gtk/gtkfilesel.h>
-#include <libgnome/gnome-config.h>
+#include <gtk/gtkhbbox.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-messagebox.h>
 #include <libgnomeui/gnome-dialog-util.h>
@@ -54,27 +56,33 @@
 class GFilePicker;
 
 extern "C" {
-void filePicker_save_as_ok_cb(GtkButton *button, GFilePicker *aHandler);
-void filePicker_save_as_cancel_cb(GtkButton *button, GFilePicker *aHandler);
+void filePicker_save_as_ok_cb (GtkButton *button, GFilePicker *aHandler);
+void filePicker_save_as_cancel_cb (GtkButton *button, GFilePicker *aHandler);
 gboolean filePicker_save_as_close_cb (GtkWidget *dialog, GdkEvent *event,
 				      GFilePicker *aHandler);
+void filePicker_save_content_cb (GtkToggleButton *button,
+				 GFilePicker *aHandler);
 }
 
 /* Header file */
 class GFilePicker : public nsIFilePicker
 {
-	friend void filePicker_save_as_ok_cb(GtkButton *button,
-					     GFilePicker *aHandler);
-	friend void filePicker_save_as_cancel_cb(GtkButton *button,
-						 GFilePicker *aHandler);
+	friend void filePicker_save_as_ok_cb (GtkButton *button,
+					      GFilePicker *aHandler);
+	friend void filePicker_save_as_cancel_cb (GtkButton *button,
+						  GFilePicker *aHandler);
 	friend gboolean filePicker_save_as_close_cb (GtkWidget *dialog,
 						     GdkEvent *event,
 						     GFilePicker *aHandler);
+	friend void filePicker_save_content_cb (GtkToggleButton *button,
+						GFilePicker *aHandler);
   public:
 	NS_DECL_ISUPPORTS
 	NS_DECL_NSIFILEPICKER
 
-	GFilePicker();
+	enum { returnOKSaveContent = 256 };
+
+	GFilePicker(PRBool showContentCheck = PR_FALSE);
 	virtual ~GFilePicker();
 	/* additional members */
 	NS_IMETHODIMP InitWithGtkWidget (GtkWidget *parentWidget, 
@@ -94,6 +102,8 @@ class GFilePicker : public nsIFilePicker
 
 	PRBool mFileSelectorClosed;
 	PRBool mFileSelectorOk;
+	PRBool mShowContentCheck;
+	PRBool mSaveContent;
 
 	GtkWidget *mParentWidget;	
 	GtkWidget *mFileSelector;
@@ -102,7 +112,7 @@ class GFilePicker : public nsIFilePicker
 /* Implementation file */
 NS_IMPL_ISUPPORTS1(GFilePicker, nsIFilePicker)
 
-GFilePicker::GFilePicker()
+GFilePicker::GFilePicker (PRBool showContentCheck) : mSaveContent(PR_FALSE)
 {
 	NS_INIT_ISUPPORTS();
 	/* member initializers and constructor code */
@@ -115,7 +125,7 @@ GFilePicker::GFilePicker()
 
 	if (NS_FAILED(rv))
 	{
-		char *dirName = gnome_config_get_string (CONF_DOWNLOADING_DIR);
+		char *dirName = eel_gconf_get_string (CONF_DOWNLOADING_DIR);
 
 		if (!dirName)
 		{
@@ -128,7 +138,7 @@ GFilePicker::GFilePicker()
 					   mDisplayDirectory);
 	}
 
-
+	mShowContentCheck = showContentCheck;
 }
 
 GFilePicker::~GFilePicker()
@@ -273,6 +283,29 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 			    GTK_SIGNAL_FUNC(filePicker_save_as_close_cb),
 			    (gpointer)this);
 
+	if (mShowContentCheck)
+	{
+		GtkWidget *bbox = gtk_hbutton_box_new ();
+		gtk_button_box_set_layout (GTK_BUTTON_BOX(bbox),
+					   GTK_BUTTONBOX_END);
+		gtk_button_box_set_spacing (GTK_BUTTON_BOX(bbox), 0);
+        	gtk_box_pack_end (GTK_BOX(
+        		GTK_FILE_SELECTION(mFileSelector)->action_area),
+				  bbox, TRUE, TRUE, 0);
+
+		GtkWidget *saveContent = 
+			gtk_check_button_new_with_label(_("Save with content"));
+		gtk_signal_connect(GTK_OBJECT(saveContent),
+				   "clicked",
+				   GTK_SIGNAL_FUNC(filePicker_save_content_cb),
+				   (gpointer)this);
+
+		gtk_box_pack_start(GTK_BOX(bbox), saveContent,
+				   FALSE, FALSE, 0);
+
+		gtk_widget_show_all(bbox);
+	}
+
 	if (mMode == nsIFilePicker::modeGetFolder)
 	{
 		gtk_widget_set_sensitive (GTK_FILE_SELECTION(mFileSelector)
@@ -293,6 +326,8 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 	if (mFileSelectorOk == PR_TRUE)
 	{
 		*_retval = nsIFilePicker::returnOK;
+		if (mSaveContent == PR_TRUE)
+			*_retval = returnOKSaveContent;
 	}
 	else
 	{
@@ -307,7 +342,7 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 ////////////////////////////////////////////////////////////////////////////////
 
 NS_IMETHODIMP GFilePicker::InitWithGtkWidget (GtkWidget *parentWidget, 
-					      const char *title,PRInt16 mode)
+					      const char *title, PRInt16 mode)
 {
 	mParentWidget = parentWidget;
 	
@@ -318,7 +353,7 @@ NS_IMETHODIMP GFilePicker::InitWithGtkWidget (GtkWidget *parentWidget,
 	mMode = mode;
 
 	mFile = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-	
+
 	return NS_OK;
 }
 
@@ -481,6 +516,13 @@ filePicker_save_as_close_cb (GtkWidget *dialog, GdkEvent *event,
 	return FALSE;
 }
 
+void filePicker_save_content_cb (GtkToggleButton *button,
+				 GFilePicker *aFilePicker)
+{
+	aFilePicker->mSaveContent = gtk_toggle_button_get_active (button) ?
+				    PR_TRUE : PR_FALSE;
+}	
+
 ////////////////////////////////////////////////////////////////////////////////
 /**
  * show_file_picker: Shows a file picker. Can be configured to select a
@@ -499,9 +541,16 @@ filePicker_save_as_close_cb (GtkWidget *dialog, GdkEvent *event,
 extern "C" gboolean
 show_file_picker (GtkWidget *parentWidget, const char* title,
 		  const char* directory, const char* file, FilePickerMode mode,
-		  char * *ret_fullpath)
+		  char * *ret_fullpath, gboolean *ret_save_content)
 {
-	GFilePicker *aFilePicker = new GFilePicker;
+	PRBool showContentCheck;
+	if (ret_save_content == NULL)
+		showContentCheck = PR_FALSE;
+	else
+		showContentCheck = PR_TRUE;
+
+	GFilePicker *aFilePicker = new GFilePicker (showContentCheck);
+
 	PRUnichar *uDefaultString = mozilla_locale_to_unicode (file);
 
 	if (!directory)
@@ -516,6 +565,14 @@ show_file_picker (GtkWidget *parentWidget, const char* title,
 	
 	PRInt16 retval;
 	aFilePicker->Show (&retval);
+
+	if (ret_save_content != NULL)
+	{
+		if (retval == GFilePicker::returnOKSaveContent)
+			*ret_save_content = TRUE;
+		else
+			*ret_save_content = FALSE;
+	}
 	
 	if (retval == nsIFilePicker::returnCancel)
 	{
