@@ -19,6 +19,8 @@
 #include "galeon.h"
 #include "mozilla.h"
 
+#include <unistd.h>
+#include <time.h>
 #include <gtkmozembed.h>
 #include <gtkmozembed_internal.h>
 
@@ -66,6 +68,7 @@
 #include "nsIClipboardCommands.h"
 #include "GaleonWrapper.h"
 #include "PersistProgress.h"
+#include "PrintListener.h"
 #include "GlobalHistory.h"
 #include "nsIGlobalHistory.h"
 #include "nsIDOMHTMLImageElement.h"
@@ -124,8 +127,9 @@ nsresult GaleonWrapper::Init (GaleonEmbed *galeon_embed)
 						     GHFactory,
 						     PR_TRUE);
 
-	NS_WITH_SERVICE(nsIGlobalHistory, inst, NS_GLOBALHISTORY_CONTRACTID, &result);
- 	dsHistory->SetGlobalHistory(inst);
+	nsCOMPtr<nsIGlobalHistory> inst =  
+		do_GetService(NS_GLOBALHISTORY_CONTRACTID, &result);
+	dsHistory->SetGlobalHistory(inst);
 
 	nsCOMPtr<nsPIDOMWindow> piWin;
 	GetPIDOMWindow(getter_AddRefs(piWin));
@@ -139,10 +143,7 @@ nsresult GaleonWrapper::Init (GaleonEmbed *galeon_embed)
         mEventListener = new GaleonEventListener();
         mEventListener->Init(this);
 
-	if (attachListener)
-	{
-		return AddListener ();
-	}
+	return AddListener ();
 
 	return NS_OK;
 }
@@ -205,8 +206,46 @@ nsresult GaleonWrapper::SetOffline(PRBool aOffline)
 	return io->SetOffline(aOffline);
 }
 
-nsresult GaleonWrapper::Print (nsIPrintOptions *options)
+nsresult GaleonWrapper::Print (nsIPrintOptions *options, PRBool preview)
 {
+#if MOZILLA_VERSION >= VERSION3(0,9,3)
+	nsresult result;
+
+	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser, &result));
+	if (NS_FAILED(result) || !print) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	result = GetMainDOMWindow (getter_AddRefs(DOMWindow));
+	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
+
+	if (!preview)
+	{
+		result = print->Print (DOMWindow, options, NULL);
+	}
+	else
+	{
+		char *filename;
+
+		filename = g_strconcat(g_get_tmp_dir(),"/galeon-print-XXXXXX", NULL);
+		/* get a name for the temporary file */
+		int res;
+		res = mkstemp (filename);
+		if (res == -1)
+			return NS_ERROR_FAILURE;
+		close (res);
+		mozilla_delete_temp_file_on_exit (filename);
+
+		nsCString fileString(filename);
+		options->SetToFileName (fileString.ToNewUnicode());
+		options->SetPrintToFile (TRUE);
+
+		PrintListener *listener = 
+			new PrintListener(filename);
+		result = print->Print (DOMWindow, options, listener); 
+	}
+
+	return result;
+#else
 	nsresult rv = NS_OK;
 
 	nsCOMPtr<nsIDocShell> DocShell;
@@ -223,6 +262,7 @@ nsresult GaleonWrapper::Print (nsIPrintOptions *options)
 	}
 
 	return rv;
+#endif
 }
 
 nsresult GaleonWrapper::GetSHistory (nsISHistory **aSHistory)
@@ -277,8 +317,7 @@ nsresult GaleonWrapper::GetDocShell (nsIDocShell **aDocShell)
 
 nsresult GaleonWrapper::Destroy ()
 {
- 	if (attachListener)
-  		return RemoveListener ();
+  	RemoveListener ();
       	mWebBrowser = nsnull;
 	mChromeNav = nsnull;
 	
@@ -371,15 +410,34 @@ nsresult GaleonWrapper::GetZoom (float *aZoom)
 	return mdv->GetTextZoom (aZoom);
 }
 
+nsresult GaleonWrapper::GetDOMWindow (nsIDOMWindow **aDOMWindow)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIWebBrowserFocus> focus = do_GetInterface(mWebBrowser, &rv);
+	if (NS_FAILED(rv) || !focus) return NS_ERROR_FAILURE;
+
+	rv = focus->GetFocusedWindow (aDOMWindow);
+	if (NS_FAILED(rv))
+		rv = GetMainDOMWindow (aDOMWindow);
+	return rv;
+}
+
+nsresult GaleonWrapper::GetMainDOMWindow (nsIDOMWindow **aDOMWindow)
+{
+	return mWebBrowser->GetContentDOMWindow (aDOMWindow);
+}
+
 nsresult GaleonWrapper::GetDOMDocument (nsIDOMDocument **aDOMDocument)
 {
 	nsresult result;
 
-	nsCOMPtr<nsIWebBrowserFocus> focus(do_GetInterface(mWebBrowser));
-
 	nsCOMPtr<nsIDOMWindow> DOMWindow;
-	result = focus->GetFocusedWindow (getter_AddRefs(DOMWindow));
-	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
+	result = GetDOMWindow (getter_AddRefs(DOMWindow));
+	if (NS_FAILED(result) || !DOMWindow)
+	{
+		return GetMainDOMDocument (aDOMDocument);
+	}
 
 	return DOMWindow->GetDocument (aDOMDocument);
 }
@@ -446,6 +504,31 @@ nsresult GaleonWrapper::SaveLink (const char *url)
 #endif
 }
 
+nsresult GaleonWrapper::SaveDocument (char *filename, const char *datapath,
+				      int action, PRBool mainDoc)
+{
+	nsresult rv;
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+	if (mainDoc)
+		rv = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
+	else
+		rv = GetDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(rv) || !aDOMDocument) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIWebBrowserPersist> persist = do_GetInterface (mWebBrowser,
+								  &rv);
+	if (NS_FAILED(rv) || !persist) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIWebBrowserPersistProgress> progress;
+	if (action)
+	{
+		progress = new PersistProgress(filename, action);
+		persist->SetProgressListener (progress);
+	}
+
+	return persist->SaveDocument (aDOMDocument, filename, datapath);
+}
+
 nsresult GaleonWrapper::GetSHInfo (PRInt32 *count, PRInt32 *index)
 {
 	nsresult result;
@@ -793,6 +876,14 @@ nsresult GaleonWrapper::GetEventContext (nsIDOMEventTarget *EventTarget,
 			/* Link */
 			if (tag.EqualsWithConversion("a", PR_TRUE))
 			{
+				PRBool isHref;
+				rv = element->HasAttribute (NS_LITERAL_STRING("href"),
+							    &isHref);
+				if (NS_FAILED(rv))
+					return NS_ERROR_FAILURE;
+				if (isHref == PR_FALSE)
+					return NS_OK;
+
 				info->context |= CONTEXT_LINK;
 
 				nsCOMPtr <nsIDOMHTMLAnchorElement> anchor =
@@ -1215,11 +1306,30 @@ NS_IMETHODIMP GaleonEventListener::DragGesture(nsIDOMEvent* aMouseEvent)
 		if (old_dragging_link) g_free (old_dragging_link);
 		gtk_object_set_data (GTK_OBJECT(wrapper->mGtkMozEmbed),
 				     "dragging_link", g_strdup (url));
+
+
+		GdkEvent gdk_event;
+		gdk_event.type = GDK_BUTTON_PRESS;
+		gdk_event.button.window = NULL;
+		gdk_event.button.send_event = 0;
+		gdk_event.button.time = time (NULL);
+		gdk_event.button.x = 0;
+		gdk_event.button.y = 0;
+		gdk_event.button.pressure = 0;
+		gdk_event.button.xtilt = 0;
+		gdk_event.button.ytilt = 0;
+		gdk_event.button.state = 0;
+		gdk_event.button.button = 0;
+		gdk_event.button.source = (GdkInputSource)0;
+		gdk_event.button.deviceid = 0;
+		gdk_event.button.x_root = 0;
+		gdk_event.button.y_root = 0;
+
 		gtk_drag_begin (GTK_WIDGET(wrapper->mGtkMozEmbed),
 				link_drag_types_tl, 
 				(GdkDragAction)(GDK_ACTION_COPY | 
 						GDK_ACTION_LINK),
-				1, NULL);
+				1, &gdk_event);
 	}
 
 	mozilla_free_context_info_sub (&info->ctx);
