@@ -19,7 +19,7 @@
 void auto_completion_add_url (gchar *url);
 void auto_completion_reset (void);
 gchar *auto_completion_complete_url (gchar *prefix);
-gchar *auto_completion_complete_url_extended (gchar *prefix);
+gchar *auto_completion_complete_url_extended (gchar *prefix, gboolean suggest);
 void auto_completion_display (GaleonWindow *window, GtkWidget *entry);
 void auto_completion_display_alternatives (GaleonWindow *window,
 					   GtkWidget *entry);
