@@ -23,6 +23,7 @@
 #include "autocompletion.h"
 #include "bookmarks.h"
 #include "window.h"
+#include "prefs.h"
 #include "favicon.h"
 
 /* GNOME includes */
@@ -42,6 +43,7 @@ PixmapData *folder_open_pixmap_data = NULL;
 PixmapData *default_folder_pixmap_data = NULL;
 PixmapData *default_folder_open_pixmap_data = NULL;
 PixmapData *site_pixmap_data = NULL;
+PixmapData *smart_bm_tray_pixmap_data = NULL;
 
 /** The root of the bookmark tree */
 BookmarkItem *bookmarks_root = NULL;
@@ -55,7 +57,9 @@ gboolean bookmarks_dirty = FALSE;
 /** The next id to assign (for alias saving) */
 static long maxid = 0;
 
-/** Aliases that have been loaded, but whose real bookmark has not been loaded */
+/** 
+ * Aliases that have been loaded, but whose real bookmark has not been loaded 
+ */
 GList *unresolved_aliases = NULL;
 
 /** The favicon pixmap data cache */
@@ -88,6 +92,7 @@ static BookmarkItem *bookmarks_find_by_id (long id,
 static BookmarkItem *bookmarks_merge_trees_look_for_equal 
 	(BookmarkItem *folder, BookmarkItem *bookmark);
 static gint bookmarks_compare_name (gconstpointer a, gconstpointer b);
+static gchar *bookmarks_get_bookmark_path (BookmarkItem *b);
 
 
 /**
@@ -124,12 +129,14 @@ bookmarks_exit (void)
 	g_free (default_folder_open_pixmap_data);
 	g_free (default_folder_pixmap_data);
 	g_free (site_pixmap_data);
+	g_free (smart_bm_tray_pixmap_data);
 
 	folder_open_pixmap_data = NULL;
 	folder_pixmap_data = NULL;
 	default_folder_open_pixmap_data = NULL;
 	default_folder_pixmap_data = NULL;
 	site_pixmap_data = NULL;
+	smart_bm_tray_pixmap_data = NULL;
 
 	bookmarks_remove_recursively (bookmarks_root);
 }
@@ -218,6 +225,9 @@ bookmarks_load_icons (void)
 		pixmap_data_new_from_file (SHARE_DIR "/default_open.xpm");
 	site_pixmap_data = 
 		pixmap_data_new_from_file (SHARE_DIR "/i-bookmark.xpm");
+	smart_bm_tray_pixmap_data = 
+		pixmap_data_new_from_file (SHARE_DIR "/smart-bm-tray.xpm");
+
 	/* TODO: add icons for alias */
 
 	/* if we didn't manage to load the icons, make an empty structure
@@ -243,6 +253,10 @@ bookmarks_load_icons (void)
 	{
 		site_pixmap_data = g_new0 (PixmapData, 1);
 	}
+	if (smart_bm_tray_pixmap_data == NULL)
+	{
+		smart_bm_tray_pixmap_data = g_new0 (PixmapData,1);
+	}
 }
 
 /**
@@ -276,13 +290,13 @@ bookmarks_load_from (char *file)
 }
 
 /**
- * bookmarks_xml_read: read all items from xml file
+ * bookmarks_xml_read: read all items from xml file, doing alias resolution
  */
 static BookmarkItem * 
 bookmarks_xml_read (xmlNodePtr item) 
 {
 	static BookmarkItem *error_bookmark = NULL;
-	BookmarkItem *b, *result;
+	BookmarkItem *result;
 	item = item->childs;
 	unresolved_aliases = NULL;
 	
@@ -291,38 +305,42 @@ bookmarks_xml_read (xmlNodePtr item)
 			(BM_SITE, TRUE, _("Error"), "about:blank", NULL,
 			 _("An error ocurred loading your bookmarks."), NULL);
 	
+	/* old format, no longer supported */
+	/*
 	result = bookmarks_new_bookmark (BM_FOLDER, TRUE, _("Bookmarks"),
 					 NULL, NULL, NULL, NULL);
-	
-	if (item->next) /* old format */
-		while (item != NULL) {
-			xmlNodePtr item2 = item;
-			b = bookmarks_xml_read_item(item2);
-			result->list = g_list_append(result->list, b);
-			b->parent = result;
-			item = item->next;
-		}
-	else {
-		bookmarks_free_bookmark (result);
-		result = bookmarks_xml_read_item (item);
-	}
+	if (item->next) 
+		while (item != NULL) { 
+			xmlNodePtr item2 = item; 
+			BookmarkItem *b = bookmarks_xml_read_item (item2); 
+			result->list = g_list_append (result->list, b);
+			b->parent = result; 
+			item = item->next; 
+		} 
+	*/
 
+	result = bookmarks_xml_read_item (item);
+	
 	/* Finish creating the aliases */
 	while (unresolved_aliases) {
 		BookmarkItem *alias = unresolved_aliases->data;
 		BookmarkItem *real = bookmarks_find_by_id (alias->id, result);
 		unresolved_aliases = g_list_remove (unresolved_aliases, alias);
-		if (real)
+		if (real) {
 			bookmarks_new_alias_do (real, alias);
-		else
+		} else {
+			g_warning 
+				("Aliased bookmark not found: %ld", alias->id);
 			bookmarks_new_alias_do (error_bookmark, alias);
+		}
 	}
 
 	return result;
 }
 
 /**
- * bookmarks_xml_read_item: read an item from xml file
+ * bookmarks_xml_read_item: read an item from xml file. Does not resolve 
+ * aliases. Use with care.
  */
 static BookmarkItem *
 bookmarks_xml_read_item (xmlNodePtr item)
@@ -395,11 +413,14 @@ bookmarks_xml_read_item (xmlNodePtr item)
 		tbs = xmlGetProp (item, "toolbar_style");
 		ccm = xmlGetProp (item, "create_context_menu");
 		expanded = xmlGetProp (item, "expanded");
-		b->create_toolbar = (ct && !strcmp (ct, "TRUE")) ? TRUE : FALSE;
+		b->create_toolbar = (ct && !strcmp (ct, "TRUE")) ? TRUE 
+			: FALSE;
 		if (tbs)
 			b->toolbar_style = atoi(tbs);
-		b->create_context_menu = (ccm && !strcmp (ccm, "TRUE")) ? TRUE : FALSE;
-		b->expanded = (expanded && !strcmp (expanded, "TRUE")) ? TRUE : FALSE;
+		b->create_context_menu = (ccm && !strcmp (ccm, "TRUE")) ? TRUE
+			: FALSE;
+		b->expanded = (expanded && !strcmp (expanded, "TRUE")) ? TRUE
+			: FALSE;
 		xmlFree (ct);
 		xmlFree (tbs);
 		xmlFree (ccm);
@@ -495,7 +516,7 @@ bookmarks_backup (void)
 	int count = 0;
 
 	/* check if it's not enabled */
-	if (gnome_config_get_bool (CONF_BOOKMARKS_BACKUP_ENABLED) == FALSE)
+	if (eel_gconf_get_boolean (CONF_BOOKMARKS_BACKUP_ENABLED) == FALSE)
 	{
 		return;
 	}
@@ -505,7 +526,7 @@ bookmarks_backup (void)
 		return;
 	}
 
-	for (count = gnome_config_get_int (CONF_BOOKMARKS_BACKUP_COUNT); 
+	for (count = eel_gconf_get_integer (CONF_BOOKMARKS_BACKUP_COUNT); 
 	     count >= 1 ; count--) {
 		gchar *str;
 		gchar *str2;
@@ -744,15 +765,18 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
  */
 void 
 bookmarks_remove_recursively (BookmarkItem *b) {
-	GList *position, *li;
+	GList *position;
 	
 	if (b == NULL) return;
-	
+
 	if (b->parent) {
 		position = g_list_find (b->parent->list, b);
-		b->parent->list = 
-			g_list_remove_link (b->parent->list, position);
-		bookmarks_update_alias (b->parent);
+		if (position) {
+			b->parent->list = 
+				g_list_remove_link (b->parent->list, position);
+			bookmarks_update_alias (b->parent);
+		}
+		b->parent = NULL;
 	}
 	
 	if (!b->alias_of) 
@@ -765,10 +789,14 @@ bookmarks_remove_recursively (BookmarkItem *b) {
 		   it's next, if any */
 		b->alias_of->alias = b->alias;
 	
-	if ((b->type == BM_FOLDER || b->type == BM_AUTOBOOKMARKS) && 
-	    ! b->alias_of)
-		for (li = b->list; li != NULL; li = li->next) 
-			bookmarks_remove_recursively (li->data);
+	if (((b->type == BM_FOLDER) || (b->type == BM_AUTOBOOKMARKS)) && 
+	    !b->alias_of)
+	{
+		while (b->list)
+		{
+			bookmarks_remove_recursively (b->list->data);
+		}
+	}
 	
 	if (b == default_bookmarks_root) 
 		default_bookmarks_root = bookmarks_root;
@@ -1004,7 +1032,7 @@ void
 bookmarks_free_bookmark (BookmarkItem *b)
 {
 	if (! b) return;
-
+	
 	/* aliases fields point to the real bookmark fields */
 	if (!b->alias_of) {
 		if (b->name) g_free (b->name);
@@ -1200,7 +1228,7 @@ bookmarks_string_request_callback (gchar *string, BookmarkItem *b)
 		}
 
 		if (b->url &&
-		    gnome_config_get_bool (CONF_GENERAL_FAVICONS_ENABLED))
+		    eel_gconf_get_boolean (CONF_BOOKMARKS_FAVICONS_ENABLED))
 		{
 			favicon_get_siteicon (NULL, b->url);
 		}
@@ -1252,8 +1280,7 @@ add_temp_link_bookmark (const gchar *name, const gchar *url, BookmarkItem *paren
 {
 	BookmarkItem *bookmark;
 	gchar *newname;
-	gint shorttext = gnome_config_get_int 
-		("/galeon/TempBookmarks/shorttext=16");
+	gint shorttext = eel_gconf_get_integer (CONF_BOOKMARKS_SHORTTEXT);
 
 	/**
 	 * if shorttext is < 0, use old url only behavior,
@@ -1509,7 +1536,7 @@ bookmarks_item_to_string (BookmarkItem *bookmark)
 }
 
 /**
- * bookmarks_load_from_string: Read a bookmark from a string
+ * bookmarks_item_from_string: Read a bookmark from a string
  * @string: the string to parse.
  * Return value: The new allocated bookmark or NULL if error
  **/
@@ -1524,8 +1551,8 @@ bookmarks_item_from_string (char *string)
 	
 	if (doc)
 	{
-		item = doc->root->childs;
-		b = bookmarks_xml_read_item (item);
+		item = doc->root;
+		b = bookmarks_xml_read (item);
 		xmlFreeDoc (doc);
 	} 
 	else
@@ -1654,6 +1681,7 @@ bookmarks_remove_all_alias (BookmarkItem *b)
 	if (alias) {
 		bookmarks_remove_all_alias (alias);
 		bookmarks_remove_recursively (alias);
+		b->alias = NULL;
 	}
 }
 
@@ -1667,7 +1695,8 @@ static BookmarkItem *bookmarks_find_by_id (long id, BookmarkItem *r)
 	if (!r) r = bookmarks_root;
 	
 	/* Don't return bookmarks that may be not yet created aliases */
-	if ((r->id == id) && r->name != NULL) return bookmarks_find_real_bookmark (r);
+	if ((r->id == id) && (r->name != NULL)) 
+		return bookmarks_find_real_bookmark (r);
 	if ((r->type == BM_FOLDER) && !r->alias_of)
 		for (l = r->list; l != NULL; l = g_list_next (l)) {
 			b = bookmarks_find_by_id (id, l->data);
@@ -1686,7 +1715,7 @@ bookmarks_get_siteicon (const gchar *url)
 	gchar *favicon_url, *favicon_path;
 	PixmapData *result = NULL;
 
-	if (!gnome_config_get_bool (CONF_GENERAL_FAVICONS_ENABLED))
+	if (!eel_gconf_get_boolean (CONF_BOOKMARKS_FAVICONS_ENABLED))
 	{
 		return site_pixmap_data;
 	}
@@ -1932,3 +1961,61 @@ bookmarks_compare_name (gconstpointer a, gconstpointer b)
 
 	return g_strcasecmp (b1->name, b2->name);
 }
+
+/**
+ * bookmarks_folder_open_in_myportal: Open the MyPortal page for a category
+ **/
+void
+bookmarks_folder_open_in_myportal (GaleonEmbed *embed, BookmarkItem *bi,
+				   gboolean new_window, gboolean reuse)
+{
+	gchar *url;
+	gchar *path;
+
+	g_return_if_fail (bi != NULL);
+
+	if ((bi->type != BM_FOLDER) && (bi->type != BM_AUTOBOOKMARKS))
+	{
+		bi = bi->parent;
+	}
+
+	path = bookmarks_get_bookmark_path (bi);
+	url = g_strconcat ("myportal:", path, NULL);
+	g_free (path);
+
+	if (reuse)
+	{
+		embed_load_url (embed, url);
+	}
+	else 
+	{
+		embed_create_from_url (embed, url, TRUE,
+				new_window);
+	}
+	g_free (url);
+}
+
+/**
+ * Returns the path of the bookmark (for MyPortal)
+ */
+static gchar *
+bookmarks_get_bookmark_path (BookmarkItem *b)
+{
+	gchar *stripped = strip_uline_accel (b->name);
+	gchar *escaped = escape_path (stripped);
+	gchar *path;
+	if ((b->parent == NULL) || (b->parent == bookmarks_root)) 
+	{
+		path = g_strconcat ("/", escaped, NULL);
+	}
+	else
+	{
+		gchar *parent = bookmarks_get_bookmark_path (b->parent);
+		path = g_strconcat (parent, "/", escaped, NULL);
+		g_free (parent);
+	}
+	g_free (escaped);
+	g_free (stripped);
+	return path;
+}
+
