@@ -97,6 +97,8 @@ void bookmarks_set_root (BookmarkItem *new_root);
 void bookmarks_sort_norec (BookmarkItem *b, gboolean update_edit);
 void bookmarks_sort_rec (BookmarkItem *b, gboolean update_edit);
 void bookmarks_editor_popup_menu (BookmarksEditorControls *controls);
+void bookmarks_folder_open_in_myportal (GaleonEmbed *embed, BookmarkItem *bi,
+		gboolean new_window, gboolean reuse);
 
 /* bookmarks_menus.c */
 void bookmarks_menu_create (GaleonWindow *window);
@@ -159,6 +161,7 @@ void bookmarks_editor_bookmark_copy_cb (GtkMenuItem *menuitem,
 void bookmarks_nick_entry_activate (GtkEntry *entry, BookmarkItem *b);
 void bookmarks_editor_show_attach_cb (GtkWidget *toggle, 
 				      BookmarksEditorControls *controls);
+void bookmarks_open_in_myportal_cb (GtkWidget *widget, BookmarkItem *bi);
 
 /* bookmarks_toolbar.c */
 void bookmarks_toolbars_create (GaleonWindow *window);
