@@ -23,6 +23,7 @@
 #include "glade.h"
 #include "window.h"
 #include "state.h"
+#include "prefs.h"
 #include "context.h"
 #include "dialog.h"
 
@@ -273,8 +274,7 @@ bookmarks_editor_show_dialog (GaleonWindow *window)
 	
 	/* 630 is chosen since thats wide enough for the toolbar in
 	 * with all the standard GNOME defaults */
-	state_load_window_state (controls->dialog, "bookmarks_editor",
-				 630, 450);
+	state_load_window_state (controls->dialog, "bookmarks_editor");
 	state_load_column_widths (controls->ctree, "bookmarks_editor");
 	state_load_pane_pos (controls->vpane,
 			     "bookmarks_editor_vpane=-1");
@@ -284,8 +284,8 @@ bookmarks_editor_show_dialog (GaleonWindow *window)
 	/* setup toolbar*/
 	gtk_toolbar_set_style 
 		(GTK_TOOLBAR (controls->toolbar),
-		gnome_config_get_int 
-			(CONF_APPEARANCE_BOOKMARKS_EDITOR_TOOLBAR_STYLE));
+		eel_gconf_get_integer
+			(CONF_TOOLBAR_BOOKMARKS_EDITOR_STYLE));
 	gnome_dock_item_set_shadow_type
 		(GNOME_DOCK_ITEM (controls->toolbar_dock), 
 		 toolbar_relief ? GTK_SHADOW_OUT : GTK_SHADOW_NONE);
@@ -386,8 +386,8 @@ bookmarks_editor_show_dock (GaleonWindow *window)
 	
 	gtk_toolbar_set_style 
 		(GTK_TOOLBAR (controls->toolbar), 
-		gnome_config_get_int 
-		       (CONF_APPEARANCE_BOOKMARKS_EDITOR_DOCKED_TOOLBAR_STYLE));
+		eel_gconf_get_integer 
+		       (CONF_TOOLBAR_BOOKMARKS_EDITOR_DOCKED_STYLE));
 
 	window_dock (window, controls->dialog, 
 		     CONF_STATE_BOOKMARKS_DOCK_WIDTH);
@@ -427,13 +427,17 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls,
 	text[2] = NULL;
 	switch (b->type) {
 	case BM_SEPARATOR:
-		text[0] = text[1] = "------";
+		text[0] = g_strdup ("------");
+		text[1] = g_strdup ("------");
 		isleaf = TRUE;
 		pixmap = mask = pixmap_open = mask_open = NULL;
 		break;
 	case BM_SITE:
-		text[0] = b->name;
-		text[1] = b->url;
+		/* b->name is nicely escaped, but we need to unescape
+		 * it here, as GtkCLists and GtkEntries don't know about 
+		 * accels */
+		text[0] = strip_uline_accel (b->name);
+		text[1] = g_strdup (b->url);
 		isleaf = TRUE;
 
 		bm_icon = bookmarks_get_siteicon (b->url);
@@ -445,8 +449,11 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls,
 		break;
 	case BM_FOLDER:
 	case BM_AUTOBOOKMARKS:
-		text[0] = b->name;
-		text[1] = "";
+		/* b->name is nicely escaped, but we need to unescape
+		 * it here, as GtkCLists and GtkEntries don't know about 
+		 * accels */
+		text[0] = strip_uline_accel (b->name);
+		text[1] = g_strdup ("");
 		isleaf = FALSE;
 		if (b == default_bookmarks_root) 
 		{
@@ -462,7 +469,8 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls,
 		}
 		break;
 	default:
-		text[0] = text[1] = "";
+		text[0] = g_strdup("");
+		text[1] =  g_strdup("");
 		isleaf = TRUE;
 		pixmap = mask = pixmap_open = mask_open = NULL;
 		break;
@@ -511,6 +519,8 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls,
 			bookmarks_editor_place_tree_item (controls, l->data);
 	}
 	gtk_clist_thaw (GTK_CLIST (controls->ctree));
+	g_free (text[0]);
+	g_free (text[1]);
 }
 
 void
@@ -633,13 +643,17 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 	text[2] = NULL;
 	switch (b->type) {
 	case BM_SEPARATOR:
-		text[0] = text[1] = "------";
+		text[0] = g_strdup ("------"); 
+		text[1] = g_strdup ("------");
 		isleaf = TRUE;
 		pixmap = mask = NULL;
 		break;
 	case BM_SITE:
-		text[0] = b->name;
-		text[1] = b->url;
+		/* b->name is nicely escaped, but we need to unescape
+		 * it here, as GtkCLists and GtkEntries don't know about 
+		 * accels */
+		text[0] = strip_uline_accel (b->name);
+		text[1] = g_strdup (b->url);
 		isleaf = TRUE;
 		bm_icon = bookmarks_get_siteicon (b->url);
 		pixmap = bm_icon->pixmap;
@@ -647,8 +661,11 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 		break;
 	case BM_FOLDER:
 	case BM_AUTOBOOKMARKS:
-		text[0] = b->name;
-		text[1] = "";
+		/* b->name is nicely escaped, but we need to unescape
+		 * it here, as GtkCLists and GtkEntries don't know about 
+		 * accels */
+		text[0] = strip_uline_accel (b->name);
+		text[1] = g_strdup ("");
 		isleaf = FALSE;
 		if (b == default_bookmarks_root)
 		{
@@ -661,7 +678,8 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 		}
 		break;
 	default:
-		text[0] = text[1] = "";
+		text[0] = g_strdup (""); 
+		text[1] = g_strdup ("");
 		isleaf = TRUE;
 		pixmap = mask = NULL;
 		break;
@@ -684,6 +702,8 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 		gtk_ctree_node_set_text
 			(GTK_CTREE (controls->ctree), bookmarks_get_tree_item (b, controls), 1, text[1]);
 	}
+	g_free (text[0]);
+	g_free (text[1]);
 }
 
 /**
@@ -784,15 +804,16 @@ bookmarks_editor_popup_menu (BookmarksEditorControls *controls)
  */
 #if 0
 static char *ugh[] = {
-	/* File */
+	N_("_File"),
 		N_("_Open..."),
 		N_("_Revert"),
 		N_("_Save"),
 		N_("_Close"),
-	/* Edit */
+	N_("_Edit"),
 		N_("C_ut"),
 		N_("_Copy"),
 		N_("_Paste"),
+	N_("_View")
 }
 #endif
 
