@@ -23,6 +23,7 @@
 #include "bookmarks.h"
 #include "favicon.h"
 #include "bookmarks_editor_callbacks.h"
+#include "prefs.h"
 #include "context.h"
 #include "dialog.h"
 #include "filepicker.h"
@@ -177,8 +178,13 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 		b_alias = controls->selection;
 		b_real = bookmarks_find_real_bookmark (controls->selection);
 		if (controls->name_entry)
-			name = gtk_editable_get_chars 
+		{
+			gchar *tmp;
+			tmp = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->name_entry), 0, -1);
+			name = escape_uline_accel (tmp);
+			g_free (tmp);
+		}
 		if (controls->url_entry)
 			url = gtk_editable_get_chars 
 				(GTK_EDITABLE (controls->url_entry), 0, -1);
@@ -370,7 +376,7 @@ bookmarks_editor_remove_clicked_cb (GtkButton *button,
 	if (autobookmarks_deleted)
 	{
 		autobookmarks_root = NULL;
-		gnome_config_set_bool (CONF_AUTOBOOKMARKS_ENABLED, FALSE);
+		eel_gconf_set_boolean (CONF_BOOKMARKS_AB_ENABLED, FALSE);
 	}
 
 	if (next_to_select)
@@ -932,9 +938,15 @@ static void
 bookmarks_editor_set_name (BookmarksEditorControls *controls, 
 			   BookmarkItem *bookmark, gboolean active)
 {
+	gchar *tmp = NULL;
+
+	if (bookmark && bookmark->name)
+		tmp = strip_uline_accel (bookmark->name);
+
 	bookmarks_editor_set_text (controls->name_entry, 
-				   bookmark ? bookmark->name : NULL, 
+				   bookmark ? tmp : NULL, 
 				   active);
+	if (tmp) g_free (tmp);
 }
 
 /**
@@ -1234,7 +1246,7 @@ bookmarks_dock_detach_button_clicked_cb (GtkButton *button,
 	gtk_clist_thaw (GTK_CLIST(controls->ctree));
 
 	bookmarks_editors = g_list_remove (bookmarks_editors, controls);
-	gnome_config_set_bool (CONF_STATE_BOOKMARKS_DOCK_ENABLE, FALSE);
+	eel_gconf_set_boolean (CONF_STATE_BOOKMARKS_DOCK_ENABLE, FALSE);
 	window_undock (window);
 	bookmarks_editor_show_dialog (window);
 }
@@ -1248,7 +1260,7 @@ bookmarks_editor_show_attach_cb (GtkWidget *toggle,
 	window = controls->window;
 	return_if_not_window (window);
 
-	gnome_config_set_bool (CONF_STATE_BOOKMARKS_DOCK_ENABLE, TRUE);
+	eel_gconf_set_boolean (CONF_STATE_BOOKMARKS_DOCK_ENABLE, TRUE);
 	if (controls->type != DOCKED_BOOKMARKS_EDITOR) {
 		bookmarks_editor_hide_dialog (controls);
 		bookmarks_editor_show_dock (window);
@@ -1303,17 +1315,16 @@ bookmarks_editor_toolbar_appearance_menu_cb (GtkRadioMenuItem *menuitem,
 	switch (controls->type)
 	{
 	case DOCKED_BOOKMARKS_EDITOR:
-		gnome_config_set_int 
-			(CONF_APPEARANCE_BOOKMARKS_EDITOR_DOCKED_TOOLBAR_STYLE,
+	        eel_gconf_set_integer 
+			(CONF_TOOLBAR_BOOKMARKS_EDITOR_DOCKED_STYLE,
 			 index);
 		break;
 	case STANDARD_BOOKMARKS_EDITOR:
-		gnome_config_set_int 
-			(CONF_APPEARANCE_BOOKMARKS_EDITOR_TOOLBAR_STYLE,
+		eel_gconf_set_integer 
+			(CONF_TOOLBAR_BOOKMARKS_EDITOR_STYLE,
 			 index);
 		break;
 	}
-	gnome_config_sync ();
 	
 	gtk_widget_queue_resize (controls->dialog);
 }
@@ -1341,12 +1352,12 @@ bookmarks_editor_toolbar_button_press_event_cb
 		switch (controls->type)
 		{
 		case DOCKED_BOOKMARKS_EDITOR:
-			style = gnome_config_get_int 
-				(CONF_APPEARANCE_BOOKMARKS_EDITOR_DOCKED_TOOLBAR_STYLE);
+			style = eel_gconf_get_integer 
+				 (CONF_TOOLBAR_BOOKMARKS_EDITOR_DOCKED_STYLE);
 			break;
 		case STANDARD_BOOKMARKS_EDITOR:
-			style = gnome_config_get_int 
-				(CONF_APPEARANCE_BOOKMARKS_EDITOR_TOOLBAR_STYLE);
+			style = eel_gconf_get_integer
+				 (CONF_TOOLBAR_BOOKMARKS_EDITOR_STYLE);
 			break;
 		}
 	
@@ -1381,7 +1392,7 @@ bookmarks_editor_save_as_clicked_cb (GtkWidget *widget,
 
 	if (show_file_picker (controls->dialog, _("Save as..."),
 			      g_get_home_dir(), NULL, modeSave,
-			      &file)) 
+			      &file, NULL)) 
 		if (dialog_check_overwrite_file (file, NULL)) {
 			bookmarks_save_as (file);
 		}
@@ -1396,7 +1407,7 @@ bookmarks_editor_open_clicked_cb (GtkWidget *widget,
 
 	if (show_file_picker (controls->dialog, _("Load from..."),
 			      g_get_home_dir(), NULL, modeOpen,
-			      &file)) {
+			      &file, NULL)) {
 		gchar *question = g_strdup_printf
 			(_("Your current set of bookmarks will be replaced "
 			   "with the contents of %s.\n"
