@@ -380,9 +380,6 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 	{
 	case DND_TARGET_GALEON_BOOKMARK:
 		b = bookmarks_item_from_string (selection_data->data);
-		/* how do we deal with freeing selection_data->data?
-		   I know its allocated in drag_data_get */
-		/* g_free (selection_data->data); <- does not work */
 		break;
 	case DND_TARGET_TEXT_URI_LIST:
 		b = bookmarks_new_bookmark 
@@ -443,10 +440,16 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 					   dest_info.cell.row));
 
 		if (!source_node || !dest_node)
+		{
+			bookmarks_free_bookmark (b);
 			return;
+		}
 		if (!check_drag(ctree, source_node, dest_node,
 				dest_info.insert_pos))
+		{
+			bookmarks_free_bookmark (b);
 			return;
+		}
 
 		switch (dest_info.insert_pos)
 		{
@@ -477,7 +480,7 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 
 			if (!source_node || !dest_node)
 			{
-				
+				bookmarks_free_bookmark (b);
 				return;
 			}
 
