@@ -20,6 +20,7 @@
 #include "misc.h"
 #include "embed.h"
 #include "bookmarks.h"
+#include "prefs.h"
 #include "context.h"
 #include "dialog.h"
 
@@ -155,8 +156,8 @@ bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 {
 	GList *l; 
 	GtkWidget *mi;
-	ExtraItemsLocation opts_location = gnome_config_get_int
-				("/galeon/General/bm_extra_items_location=1");
+	ExtraItemsLocation opts_location = eel_gconf_get_integer
+				(CONF_BOOKMARKS_XITEMS);
 
 	g_return_if_fail (bookmark->type == BM_FOLDER || 
 			  bookmark->type == BM_AUTOBOOKMARKS);
@@ -409,7 +410,7 @@ bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
 	context_menu_add_item (extra_menu, _("_Edit..."), 
 			bookmarks_edit_activate_cb, bookmark,
 			GNOME_STOCK_MENU_PROP, TRUE);
-	context_menu_add_item (extra_menu, _("Remove bookmark folder"),
+	context_menu_add_item (extra_menu, _("_Remove bookmark folder"),
 			bookmarks_remove_activate_cb, bookmark,
 			GNOME_STOCK_MENU_TRASH, 
 			!(bookmark == bookmarks_root));
@@ -443,6 +444,10 @@ bookmarks_menu_create_base_extra_items (GtkMenu *menu, BookmarkItem *bookmark)
 			_("Open whole folder in _windows"),
 			bookmarks_open_all_in_wins_activate_cb, bookmark, 
 			NULL, TRUE);
+	context_menu_add_item (menu, 
+			_("Open folder in My _Portal"),
+			bookmarks_open_in_myportal_cb, bookmark,
+			NULL, TRUE);
 	context_menu_add_check_item (menu,
 			_("_Create toolbar"),
 			bookmarks_create_toolbar_activate_cb, bookmark,
@@ -465,7 +470,7 @@ bookmarks_item_create_extra_items (GtkMenu *extra_menu, BookmarkItem *bookmark)
 	context_menu_add_item (extra_menu, _("_Edit..."), 
 			bookmarks_edit_activate_cb, bookmark,
 			GNOME_STOCK_MENU_PROP, TRUE);
-	context_menu_add_item (extra_menu, _("Remove bookmark"),
+	context_menu_add_item (extra_menu, _("_Remove bookmark"),
 			bookmarks_remove_activate_cb, bookmark,
 			GNOME_STOCK_MENU_TRASH, TRUE);
 }
