@@ -22,6 +22,7 @@
 #include "window.h"
 #include "misc.h"
 #include "find.h"
+#include "prefs.h"
 #include "context.h"
 
 #include <string.h>
@@ -62,20 +63,39 @@ static gboolean bookmarks_toolbar_button_press_event_cb (GtkWidget *item,
 static gboolean folder_button_press_event_cb (GtkWidget *item, 
 						GdkEventButton *event,
 						BookmarkItem *bi);
-static gboolean folder_button_release_event_cb (GtkWidget *item, 
-						  GdkEventButton *event,
-						  BookmarkItem *bi);
 static gboolean bookmark_button_press_event_cb (GtkWidget *item,
 						GdkEventButton *event,
 						BookmarkItem *bi);
-static gboolean bookmark_button_release_event_cb (GtkWidget *item, 
-						  GdkEventButton *event,
-						  BookmarkItem *bi);
-static gboolean smartbookmark_button_release_event_cb (GtkWidget *item, 
-						       GdkEventButton *event,
-						       BookmarkItem *bi);
+static gboolean bookmark_button_create_menu (GtkWidget *item, 
+					     GdkEventButton *event,
+					     BookmarkItem *bi);
+static gboolean smartbookmark_tray_button_press_event_cb (GtkWidget *item, 
+						          GdkEventButton *event,
+						          BookmarkItem *bi);
+static gboolean smartbookmark_go_button_press_event_cb (GtkWidget *item, 
+						          GdkEventButton *event,
+						          BookmarkItem *bi);
 static void bookmarks_toolbar_item_menu_hide_cb (GtkMenuItem *menuitem, 
 						 BookmarkItem *bm);
+
+static void smartbookmark_setup_tray (GtkWidget *tray,
+				      gboolean visiblity);
+static void smartbookmark_setup_history (GtkCombo *combo,
+				         GSList *history);
+static void smartbookmark_add_to_history (GtkWidget *widget,
+					  char *text);
+
+static void smartbookmark_visibility_changed (GConfClient *client,
+					      guint cnxn_id,
+					      GConfEntry *entry,
+					      gpointer user_data);
+static void smartbookmark_history_changed (GConfClient *client,
+					   guint cnxn_id,
+					   GConfEntry *entry,
+					   gpointer user_data);
+static void smartbookmark_remove_notifier (GtkWidget *widget,
+					   guint noid);
+
 						
 /* this magic number is used to hack around a problem with
  * gnome-libs (I think) which causes multiple toolbars with
@@ -127,7 +147,15 @@ void
 bookmarks_toolbars_recreate (GaleonWindow *window)
 {
 	GList *l;
+	gboolean show;
 
+	/* FIXME: I think this is wrong -- ricardo */
+	if (window->bookmarks_toolbars != NULL)
+		show = GTK_WIDGET_VISIBLE (GTK_WIDGET 
+			(window->bookmarks_toolbars->data)->parent);
+	else
+		show = TRUE;
+	
 	/* destroy all the toolbars */
 	for (l = window->bookmarks_toolbars; l != NULL; l = g_list_next (l))
 	{
@@ -141,6 +169,9 @@ bookmarks_toolbars_recreate (GaleonWindow *window)
 
 	/* now create toolbars for all of these */
 	bookmarks_toolbars_create (window);
+
+	/* update visibility */
+	bookmarks_toolbars_set_visibility (window, show);
 }
 
 /**
@@ -203,7 +234,7 @@ bookmarks_toolbar_create_from_folder (GaleonWindow *window, BookmarkItem *bi)
 	/* make the toolbar */
 	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, 
 				   GTK_TOOLBAR_TEXT);
-
+	
 	/* check whether it should be detachable or not */
 	props = (GNOME_DOCK_ITEM_BEH_EXCLUSIVE |
 		 (toolbar_detachable ? 0 : GNOME_DOCK_ITEM_BEH_LOCKED));
@@ -260,7 +291,7 @@ bookmarks_toolbar_create_from_folder (GaleonWindow *window, BookmarkItem *bi)
 				(bookmarks_toolbar_button_press_event_cb), bi);
 
 	/* show it */
-	gtk_widget_show_all (GTK_WIDGET (toolbar));
+	gtk_widget_show (GTK_WIDGET (toolbar));
 
 	/* add it to the per-window list of toolbars */
 	window->bookmarks_toolbars = 
@@ -278,7 +309,7 @@ bookmarks_toolbar_create_from_bookmark (GaleonWindow *window,
 					BookmarkItem *b, 
 					ToolbarStyle style)
 {
-        GtkWidget *button, *box, *pixmap, *entry;
+        GtkWidget *button, *box, *pixmap;
 	const PixmapData *bm_icon;
 	GtkWidget *dnd_dest;
 	gchar *name = NULL;
@@ -290,6 +321,7 @@ bookmarks_toolbar_create_from_bookmark (GaleonWindow *window,
 		box = gtk_vbox_new (FALSE, 2);
 	else
 		box = gtk_hbox_new (FALSE, 2);
+
 	if (style != TOOLBAR_STYLE_TEXT_ONLY)
 	{
 		if (b->pixmap_data == NULL)
@@ -339,8 +371,10 @@ bookmarks_toolbar_create_from_bookmark (GaleonWindow *window,
 		dnd_dest = button = gtk_button_new ();
 		gtk_container_add(GTK_CONTAINER(button), box);
 
-		if ((b->url != NULL) && strlen (b->url) > BOOKMARKS_TOOLTIP_MAX_LENGTH) {
-			gchar *s = g_strndup (b->url, BOOKMARKS_TOOLTIP_MAX_LENGTH);
+		if ((b->url != NULL)
+		    && strlen (b->url) > BOOKMARKS_TOOLTIP_MAX_LENGTH) {
+			gchar *s = g_strndup 
+				(b->url, BOOKMARKS_TOOLTIP_MAX_LENGTH);
 			short_url = g_strconcat (s, "...", NULL);
 			g_free (s);
 		} 
@@ -372,12 +406,12 @@ bookmarks_toolbar_create_from_bookmark (GaleonWindow *window,
 			gtk_signal_connect (GTK_OBJECT (dnd_dest), 
 					    "button-press-event",
 					    GTK_SIGNAL_FUNC
-					    (bookmark_button_press_event_cb),
+					    (bookmark_button_create_menu),
 					    b);
 			gtk_signal_connect (GTK_OBJECT (dnd_dest), 
-					    "button-release-event",
+					    "button-press-event",
 					    GTK_SIGNAL_FUNC
-					    (bookmark_button_release_event_cb),
+					    (bookmark_button_press_event_cb),
 					    b);
 		}
 		else
@@ -387,50 +421,177 @@ bookmarks_toolbar_create_from_bookmark (GaleonWindow *window,
 					    GTK_SIGNAL_FUNC
 					    (folder_button_press_event_cb),
 					    b);
-			gtk_signal_connect (GTK_OBJECT (dnd_dest), 
-					    "button-release-event",
-					    GTK_SIGNAL_FUNC
-					    (folder_button_release_event_cb),
-					    b);
 		}
+		
+		gtk_widget_show_all (button);
 	}
 	else
 	{
-		GtkWidget *button = gtk_button_new ();
+		GtkWidget *tray, *combo = NULL, *entry;
+		GSList *history, *item;
+		gboolean enable_history;
+
+		gchar *key, *str;
 		gint entry_width;
+		guint noid;
 
-		dnd_dest = button;
-		entry_width = gnome_config_get_int
-			("/galeon/Appearance/smart_bm_entry_width=100");
-		gtk_container_add (GTK_CONTAINER (button), 
-				   GTK_WIDGET (box));
-		gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-		GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+		GtkWidget *tray_button, *tray_state, *tray_box;
+
+		key = prefs_key_new (b->url);
+		
+		/* the tray */
+		tray = gtk_hbox_new (FALSE, 0);
+                tray_box = gtk_event_box_new ();
+                gtk_container_add (GTK_CONTAINER (tray_box), tray);
+                gtk_widget_show (tray_box);
+		
+		/* the tray button */
+		tray_state = gtk_pixmap_new 
+			(smart_bm_tray_pixmap_data->pixmap,
+			 smart_bm_tray_pixmap_data->mask);
+		tray_button = dnd_dest = gtk_button_new ();
+		GTK_WIDGET_UNSET_FLAGS (tray_button, GTK_CAN_FOCUS);
+		gtk_button_set_relief (GTK_BUTTON (tray_button),
+				       GTK_RELIEF_NONE);
+		
+		gtk_box_pack_end (GTK_BOX (box), tray_state, FALSE, TRUE, 0);
 		gtk_container_set_border_width (GTK_CONTAINER (box), 0);
-		/* otherwise build an entry field and pack it */
-		entry = gtk_entry_new ();
-		gtk_widget_set_usize(entry, entry_width, -2);
-		gtk_toolbar_append_widget (tb, button, 
-					   _("Search for entered phrase"),
-					   NULL);
-		gtk_toolbar_append_widget (tb, entry, b->url, NULL);
-		gtk_toolbar_append_space (tb);
-		gtk_object_set_data (GTK_OBJECT (button), "entry", entry);
-		gtk_object_set_data (GTK_OBJECT (entry), "GaleonWindow", 
-				     window);
-		gtk_signal_connect (GTK_OBJECT (entry), "key-press-event",
-		       (GtkSignalFunc) bookmarks_nick_entry_key_press_event_cb,
-		       b);
+		gtk_container_add (GTK_CONTAINER (tray_button), box);
+		gtk_widget_show_all (tray_button);
+		gtk_widget_show_all (tray_state);
 
-		/* the click handlers */
-		gtk_signal_connect (GTK_OBJECT (dnd_dest), 
-				    "button-press-event",
-				    GTK_SIGNAL_FUNC
-				    (bookmark_button_press_event_cb), b);
-		gtk_signal_connect (GTK_OBJECT (dnd_dest), 
-				    "button-release-event",
-				    GTK_SIGNAL_FUNC
-				    (smartbookmark_button_release_event_cb), b);
+		gtk_object_set_data (GTK_OBJECT (tray_button), "tray", tray);
+		gtk_object_set_data (GTK_OBJECT (tray), "state", tray_state);
+		gtk_object_set_data_full (GTK_OBJECT (tray), "key", key, 
+					  g_free);
+
+		gtk_toolbar_append_widget (tb, tray_button,
+			_("Show/hide the smartbookmark's combo box"), NULL);
+		gtk_toolbar_append_widget (tb, tray_box, b->url, NULL);
+
+		gtk_signal_connect (GTK_OBJECT (dnd_dest),
+				    "button-press-event", GTK_SIGNAL_FUNC
+				    (bookmark_button_create_menu), b);
+		gtk_signal_connect 
+			(GTK_OBJECT (dnd_dest),
+			 "button-press-event", GTK_SIGNAL_FUNC
+			 (smartbookmark_tray_button_press_event_cb), b);
+
+		/* the entry field */
+		enable_history = eel_gconf_get_boolean (CONF_BOOKMARKS_HISTORY);
+		
+		if (enable_history)
+		{
+			combo = gtk_combo_new ();
+			gtk_combo_disable_activate (GTK_COMBO (combo));
+			entry = GTK_COMBO (combo)->entry;
+			gtk_signal_connect (GTK_OBJECT
+					    (GTK_COMBO (combo)->button),
+					    "button-press-event", 
+					    GTK_SIGNAL_FUNC
+				    	    (bookmark_button_create_menu), 
+					    b);
+		}
+		else
+		{
+			entry = gtk_entry_new ();
+		}
+
+		gtk_object_set_data (GTK_OBJECT (tray_button), "entry", entry);
+
+		entry_width = eel_gconf_get_integer
+			(CONF_BOOKMARKS_ENTRY_WIDTH);
+		gtk_widget_set_usize(entry, entry_width, -2);
+		gtk_widget_show_all (enable_history ? combo : entry);
+
+		gtk_object_set_data (GTK_OBJECT (entry),
+				     "GaleonWindow", window);
+		gtk_box_pack_start (GTK_BOX (tray), 
+				enable_history ? combo : entry, TRUE, TRUE, 0);
+
+		gtk_signal_connect (GTK_OBJECT (entry),
+				    "key-press-event", GTK_SIGNAL_FUNC
+				    (bookmarks_nick_entry_key_press_event_cb),
+				    b);
+		gtk_signal_connect (GTK_OBJECT (entry),
+				    "button-press-event", GTK_SIGNAL_FUNC
+				    (bookmark_button_create_menu), b);
+		
+		/* the go button */
+                if (eel_gconf_get_boolean (CONF_BOOKMARKS_GOTO_BUTTON))
+                {
+		        GtkWidget *go_button, *go_icon;
+
+                        go_button = gtk_button_new ();
+                        GTK_WIDGET_UNSET_FLAGS (go_button, GTK_CAN_FOCUS);
+        		gtk_button_set_relief (GTK_BUTTON (go_button),
+        				       GTK_RELIEF_NONE);
+
+        		go_icon = gnome_stock_pixmap_widget_new
+        			(GTK_WIDGET (go_button),
+                                 GNOME_STOCK_PIXMAP_JUMP_TO);
+        		gtk_container_add (GTK_CONTAINER (go_button), go_icon);
+        		gtk_widget_show_all (go_button);
+			
+        		gtk_object_set_data (GTK_OBJECT (go_button),
+                                             "entry", entry);
+
+		        gtk_box_pack_start (GTK_BOX (tray), go_button,
+                                            TRUE, FALSE, 0);
+        		gtk_tooltips_set_tip 
+				(tb->tooltips, go_button,
+				 _("Search for entered phrase"), NULL);
+			
+        		gtk_signal_connect
+				(GTK_OBJECT (go_button),
+				 "button-press-event", GTK_SIGNAL_FUNC
+				 (bookmark_button_create_menu), b);
+        		gtk_signal_connect
+				(GTK_OBJECT (go_button),
+				 "button-press-event", GTK_SIGNAL_FUNC
+				 (smartbookmark_go_button_press_event_cb),
+				 b);
+	        }
+		
+		gtk_toolbar_append_space (tb);
+		
+		/* tray visibility notifier */
+		str = CONF_STATE_SMARTBOOKMARK_VISIBILITY (key);
+		
+		noid = eel_gconf_notification_add
+			(str, smartbookmark_visibility_changed, tray);
+		gtk_signal_connect (GTK_OBJECT (tray),
+				    "destroy", GTK_SIGNAL_FUNC
+				    (smartbookmark_remove_notifier),
+				    GINT_TO_POINTER (noid));
+		
+		smartbookmark_setup_tray (tray, !eel_gconf_get_boolean (str));
+		
+		g_free (str);
+		
+		if (enable_history)
+		{
+			g_assert (combo != entry);
+
+			/* history notifier */
+			str = CONF_STATE_SMARTBOOKMARK_HISTORY (key);
+		
+			noid = eel_gconf_notification_add
+				(str, smartbookmark_history_changed, combo);
+			gtk_signal_connect (GTK_OBJECT (combo),
+				    "destroy", GTK_SIGNAL_FUNC
+				    (smartbookmark_remove_notifier),
+				    GINT_TO_POINTER (noid));
+		
+			history = eel_gconf_get_string_list (str);
+			smartbookmark_setup_history (GTK_COMBO (combo),
+						     history);
+			for (item = history; item; item = item->next) 
+				g_free (item->data);
+			g_slist_free (history);
+				    
+			g_free (str);
+		}
 	}
 
 	gtk_signal_connect (GTK_OBJECT (dnd_dest), "drag_data_received",
@@ -457,8 +618,8 @@ bookmarks_toolbars_update_entries_global (void)
 	GList *toolbars;
 	GList *l;
 
-	entry_width = gnome_config_get_int
-		("/galeon/Appearance/smart_bm_entry_width=100");
+	entry_width = eel_gconf_get_integer
+		(CONF_BOOKMARKS_ENTRY_WIDTH);
 
 	for (l = all_windows; l != NULL; l = g_list_next (l))
 	{
@@ -520,6 +681,9 @@ folder_button_press_event_cb (GtkWidget *item, GdkEventButton *event,
 {
 	GaleonWindow *window = window_from_widget (item);
 
+	if (item->window != event->window)
+		return FALSE;
+
 	/* create the folder menu */
 	if (event->button == 1)
 	{
@@ -547,6 +711,19 @@ folder_button_press_event_cb (GtkWidget *item, GdkEventButton *event,
 			gtk_object_destroy (GTK_OBJECT (tooltips));
 		}
 	}
+	else if (event->button == 2)
+	{
+		/* Load the bookmark(s) */
+		gboolean tabbed_mode;
+
+		tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED);
+		if (event->state & GDK_SHIFT_MASK)
+			tabbed_mode = !tabbed_mode;
+
+		/* Load the bookmark in a new tab/window */
+		bookmarks_folder_open_all_items (window->active_embed,
+						 bi, !tabbed_mode, FALSE);
+	}
 	else if (event->button == 3)
 	{
 		GtkMenu *popup = GTK_MENU (gtk_menu_new ());
@@ -580,36 +757,8 @@ folder_button_press_event_cb (GtkWidget *item, GdkEventButton *event,
  * FIXME: comments
  */
 static gboolean
-folder_button_release_event_cb (GtkWidget *item, GdkEventButton *event,
-				  BookmarkItem *bi)
-{
-	GaleonWindow *window = window_from_widget (item);
-	
-	if (item->window != event->window)
-		return FALSE;
-
-	/* Load the bookmark(s) */
-	if (event->button == 2)
-	{
-		gboolean tabbed_mode;
-
-		tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
-		if (event->state & GDK_SHIFT_MASK)
-			tabbed_mode = !tabbed_mode;
-
-		/* Load the bookmark in a new tab/window */
-		bookmarks_folder_open_all_items (window->active_embed,
-						 bi, !tabbed_mode, FALSE);
-	}
-	return TRUE;
-}
-
-/**
- * FIXME: comments
- */
-static gboolean
-bookmark_button_press_event_cb (GtkWidget *item, GdkEventButton *event,
-				BookmarkItem *bi)
+bookmark_button_create_menu (GtkWidget *item, GdkEventButton *event,
+			     BookmarkItem *bi)
 {
 	GaleonWindow *window = window_from_widget (item);
 
@@ -645,8 +794,8 @@ bookmark_button_press_event_cb (GtkWidget *item, GdkEventButton *event,
  * FIXME: comments
  */
 static gboolean
-bookmark_button_release_event_cb (GtkWidget *item, GdkEventButton *event,
-				  BookmarkItem *bi)
+bookmark_button_press_event_cb (GtkWidget *item, GdkEventButton *event,
+				BookmarkItem *bi)
 {
 	GaleonWindow *window = window_from_widget (item);
 	
@@ -699,28 +848,103 @@ bookmarks_toolbar_button_press_event_cb (GtkWidget *item,
 }
 
 /**
- * FIXME: comments
+ * smartbookmark_tray_button_press_event_cb: The tray button of a smart
+ * bookmark was clicked. Open/close the tray, or open in new tab/window
+ * when middleclicked.
  */
 static gboolean
-smartbookmark_button_release_event_cb (GtkWidget *item, GdkEventButton *event,
-				       BookmarkItem *bi)
+smartbookmark_tray_button_press_event_cb (GtkWidget *item,
+				          GdkEventButton *event,
+				          BookmarkItem *bi)
+{
+	if (item->window != event->window)
+		return FALSE;
+
+	if (GTK_IS_EVENT_BOX (item))
+		return FALSE;
+
+	if (event->button == 2)
+	{
+		GtkWidget *entry =
+			gtk_object_get_data (GTK_OBJECT (item), "entry");
+		gchar *translated_text, *url;
+		gchar *text = gtk_entry_get_text (GTK_ENTRY (entry));
+		GaleonWindow *window = window_from_widget (item);
+
+		if (text != NULL && strlen (text) > 0)
+		{
+			/* translate non-alphanumeric characters+into %{} 
+			 * format */
+			translated_text = bookmarks_translate_string (text);
+
+			/* get a completed url */
+			url = bookmarks_substitute_argument (bi, 
+							     translated_text);
+
+			bookmarks_set_visited (bi);
+			if (eel_gconf_get_boolean (CONF_BOOKMARKS_HISTORY))
+				smartbookmark_add_to_history (entry, text);
+			embed_activate_link_mouse (window->active_embed, NULL,
+						   url, event);
+
+			g_free (url);
+			g_free (translated_text);
+		}
+		else
+		{
+			/* if search field is empty, go to a normal bookmark
+			 * with matching nick, if one exists. */
+			BookmarkItem *b = bookmarks_find_by_nick (bi->nick,
+						bookmarks_root, FALSE);
+
+			if (b != NULL && b->url != NULL)
+			{
+				bookmarks_set_visited (b);
+				embed_activate_link_mouse
+					(window->active_embed, NULL, b->url,
+					 event);
+			}
+		}
+	}
+	else
+	{
+		gchar *key, *str;
+		GtkWidget *tray;
+	
+		tray = gtk_object_get_data (GTK_OBJECT (item), "tray");
+		key = gtk_object_get_data (GTK_OBJECT (tray), "key");
+		str = CONF_STATE_SMARTBOOKMARK_VISIBILITY (key);
+		eel_gconf_set_boolean (str, !eel_gconf_get_boolean (str));
+		g_free (str);
+	}
+
+	return TRUE;
+}
+
+/**
+ * smartbookmark_go_button_press_event_cb: The go button of a smart bookmark
+ * was clicked. Build the url and jump to it.
+ */
+static gboolean
+smartbookmark_go_button_press_event_cb (GtkWidget *item,
+				        GdkEventButton *event,
+				        BookmarkItem *bi)
 {
 	GaleonWindow *window = window_from_widget (item);
 	GaleonEmbed *embed = window->active_embed;
 	gchar *text, *translated_text, *url;
 	GtkWidget *entry;
-	
+
 	if (item->window != event->window)
 		return FALSE;
 
 	if (GTK_IS_EVENT_BOX (item))
-	{
 		return FALSE;
-	}
-
+	
 	/* smart bookmark, do search for phrase */
 	entry = gtk_object_get_data (GTK_OBJECT (item), "entry");
 	text = gtk_entry_get_text (GTK_ENTRY (entry));
+
 	if (text != NULL && strlen (text) > 0)
 	{
 		/* translate non-alphanumeric characters+into %{} format */
@@ -740,10 +964,13 @@ smartbookmark_button_release_event_cb (GtkWidget *item, GdkEventButton *event,
 		{
 			/* otherwise activate normal behaviour */
 			bookmarks_set_visited (bi);
+			if (eel_gconf_get_boolean (CONF_BOOKMARKS_HISTORY))
+				smartbookmark_add_to_history (entry, text);
 			embed_activate_link_mouse (window->active_embed, NULL,
 						   url, event);
 		}
 		g_free (url);
+		g_free (translated_text);
 	}
 	else
 	{
@@ -764,7 +991,7 @@ smartbookmark_button_release_event_cb (GtkWidget *item, GdkEventButton *event,
 }
 
 /**
- * bookmarks_toolbar_menu_hide_cb: called by the hide context menu on the
+ * bookmarks_toolbar_item_menu_hide_cb: called by the hide context menu on the
  * toolbar items. Hide the toolbar.
  */
 static void
@@ -786,10 +1013,9 @@ bookmarks_nick_entry_key_press_event_cb (GtkEntry *entry, GdkEventKey *event,
 {
 	gchar *text, *url, *translated_text;
 	GaleonWindow *window;
-	if (event->keyval != GDK_Return)
-		return FALSE;
 
-	/* find the window */
+	if (event->keyval != GDK_Return) return FALSE;
+
 	window = gtk_object_get_data (GTK_OBJECT (entry),
 				      "GaleonWindow");
 
@@ -804,6 +1030,8 @@ bookmarks_nick_entry_key_press_event_cb (GtkEntry *entry, GdkEventKey *event,
 
 	/* load it in correct location*/
 	bookmarks_set_visited (bi);
+	if (eel_gconf_get_boolean (CONF_BOOKMARKS_HISTORY))
+		smartbookmark_add_to_history (GTK_WIDGET (entry), text);
 	embed_activate_link_keyboard (window->active_embed, NULL, url,
 				      event->state);
 
@@ -852,3 +1080,139 @@ bookmarks_toolbar_drag_data_received_cb (GtkWidget *widget,
 	 }
 }
 
+/**
+ * smartbookmark_setup_tray: setup the tray according to the @visiblity flag.
+ */
+static void
+smartbookmark_setup_tray (GtkWidget *tray,
+			  gboolean visiblity)
+{
+	GtkWidget *state;
+
+	g_return_if_fail (tray != NULL);
+
+	state = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (tray), "state"));
+
+	if (visiblity)
+	{
+		gtk_widget_show (tray);
+		gtk_widget_hide (state);
+	}
+	else
+	{
+		gtk_widget_hide (tray);
+		gtk_widget_show (state);
+	}
+
+	if (tray->parent && GTK_IS_WIDGET (tray->parent)) 
+	{
+		gtk_widget_queue_resize (tray->parent);
+	}
+}
+
+/**
+ * smartbookmark_setup_tray: setup the history combo according to the
+ * @history list.
+ */
+static void
+smartbookmark_setup_history (GtkCombo *combo,
+			     GSList *history)
+{
+	if (NULL != history)
+	{
+		GSList *item;
+		GList *strings = NULL;
+
+		for (item = history; item; item = item->next)
+			strings = g_list_append (strings, item->data);
+
+		gtk_combo_set_popdown_strings (combo, strings);
+
+		g_list_free (strings);
+	}
+	else
+		gtk_list_clear_items (GTK_LIST (combo->list), 0, -1);
+}
+
+/**
+ * smartbookmark_add_to_history: add a value to the smartbookmarks history
+ */
+static void smartbookmark_add_to_history (GtkWidget *widget,
+					  char *text)
+{
+	GtkCombo *combo;
+	GtkWidget *tray;
+	char *key, *str;
+	GSList *history, *item;
+
+	if (0 == strlen (text)) return;
+
+	combo = GTK_COMBO (widget->parent);
+	tray = GTK_WIDGET (combo)->parent;
+	key = gtk_object_get_data (GTK_OBJECT (tray), "key");
+	str = CONF_STATE_SMARTBOOKMARK_HISTORY (key);
+
+	history = eel_gconf_get_string_list (str);
+	item = g_slist_find_custom (history, text, (GCompareFunc) strcmp);
+
+	if (item)
+	{
+		history = g_slist_remove_link (history, item);
+		history = g_slist_concat (item, history);
+	}
+	else
+	{
+		history = g_slist_prepend (history, strdup (text));
+	}
+
+	eel_gconf_set_string_list (str, history);
+
+	for (item = history; item; item = item->next) g_free (item->data);
+	g_slist_free (history);
+
+	g_free (str);
+}
+
+/**
+ * smartbookmark_visibility_changed: modification monitor for the
+ * smartbookmark's tray visiblity.
+ */
+static void
+smartbookmark_visibility_changed (GConfClient *client,
+                                  guint cnxn_id,
+                                  GConfEntry *entry,
+                                  gpointer user_data)
+{
+	smartbookmark_setup_tray (GTK_WIDGET (user_data),
+				  !gconf_value_get_bool (entry->value));
+}
+
+/**
+ * smartbookmark_history_changed: modification monitor for the
+ * smartbookmark's history.
+ */
+static void
+smartbookmark_history_changed (GConfClient *client,
+                               guint cnxn_id,
+                               GConfEntry *entry,
+                               gpointer user_data)
+{
+	GSList *history, *item;
+
+	history = eel_gconf_get_string_list (entry->key);
+
+	smartbookmark_setup_history (GTK_COMBO (user_data), history);
+
+	for (item = history; item; item = item->next) g_free (item->data);
+	g_slist_free (history);
+}
+
+/**
+ * smartbookmark_remove_notifier: kill the associated notifier
+ */
+static void
+smartbookmark_remove_notifier (GtkWidget *widget,
+			       guint noid)
+{
+	eel_gconf_notification_remove (noid);
+}
