@@ -25,6 +25,7 @@
 #include "mozilla.h"
 #include "downloader.h"
 #include "context.h"
+#include "prefs.h"
 
 #include <string.h>
 #include <gtkmozembed.h>
@@ -38,6 +39,8 @@
 #include <libgnomeui/gtkpixmapmenuitem.h>
 #include <libgnomeui/gnome-preferences.h>
 
+#include <libgnomevfs/gnome-vfs.h>
+
 /* defines */
 #define CONTEXT_MENU_ITEM(label, stock_id, action) \
  { GNOME_APP_UI_ITEM, label, NULL, NULL, GINT_TO_POINTER(action), NULL, \
@@ -58,14 +61,12 @@ static char OPEN_NEW_TAB_STR[]  = N_("Open in a new tab");
 static char DOWNLOAD_FILE_STR[] = N_("Download link");
 static char COPY_LINK_LOC_STR[] = N_("Copy link location");
 static char ADD_BM_STR[]        = N_("Add bookmark");
-static char OPEN_IMG_STR[]      = N_("Open image");
 static char OPEN_IMG_WIN_STR[]  = N_("Open image in a new window");
 static char OPEN_IMG_TAB_STR[]  = N_("Open image in a new tab");
 static char SAVE_IMG_STR[]      = N_("Save image");
 static char SAVE_IMG_AS_STR[]   = N_("Save image as...");
 static char USE_IMG_AS_BG_STR[] = N_("Use image as background");
 static char COPY_IMG_LOC_STR[]  = N_("Copy image location");
-static char BLOCK_IMG_SRC_STR[] = N_("Block image source");
 static char OPEN_FRAME_WIN_STR[]     = N_("Open frame in window");
 static char OPEN_FRAME_NEW_WIN_STR[] = N_("Open frame in new window");
 static char OPEN_FRAME_NEW_TAB_STR[] = N_("Open frame in new tab");
@@ -87,15 +88,19 @@ static char FULLSCREEN_VIEW_STR[]    = N_("Full Screen View");
   CONTEXT_MENU_ITEM(DOWNLOAD_FILE_STR, GNOME_STOCK_MENU_SAVE,  GALEON_CONTEXT_DOWNLOAD), \
   CONTEXT_MENU_ITEM(COPY_LINK_LOC_STR, GNOME_STOCK_MENU_COPY,  GALEON_CONTEXT_COPYLOC),    \
   CONTEXT_MENU_ITEM(ADD_BM_STR,        GNOME_STOCK_MENU_BLANK, GALEON_CONTEXT_ADDTEMPBMLINK),
+/* 
+ * The first item is "Open Image (%s)", the last is "Block images from %s"
+ * These are defined as NULL here and added in fill_in_image_context_menu
+ */
 #define IMAGE_CONTEXT_MENU \
-  CONTEXT_MENU_ITEM(OPEN_IMG_STR,      GNOME_STOCK_MENU_OPEN,    GALEON_CONTEXT_OPENIMAGE),       \
+  CONTEXT_MENU_ITEM(NULL,              GNOME_STOCK_MENU_OPEN,    GALEON_CONTEXT_OPENIMAGE),       \
   CONTEXT_MENU_ITEM(OPEN_IMG_WIN_STR,  GNOME_STOCK_MENU_NEW,     GALEON_CONTEXT_OPENIMAGEWINDOW), \
   CONTEXT_MENU_ITEM(OPEN_IMG_TAB_STR,  GNOME_STOCK_MENU_NEW,     GALEON_CONTEXT_OPENIMAGETAB),    \
   CONTEXT_MENU_ITEM(SAVE_IMG_STR,      GNOME_STOCK_MENU_SAVE,    GALEON_CONTEXT_SAVEIMAGE),       \
   CONTEXT_MENU_ITEM(SAVE_IMG_AS_STR,   GNOME_STOCK_MENU_SAVE_AS, GALEON_CONTEXT_SAVEIMAGEAS),     \
   CONTEXT_MENU_ITEM(USE_IMG_AS_BG_STR, GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_USEIMAGEASBG),    \
   CONTEXT_MENU_ITEM(COPY_IMG_LOC_STR,  GNOME_STOCK_MENU_COPY,    GALEON_CONTEXT_COPYIMAGELOC),    \
-  CONTEXT_MENU_ITEM(BLOCK_IMG_SRC_STR, GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_BLOCKIMAGESOURCE),
+  CONTEXT_MENU_ITEM(NULL,              GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_BLOCKIMAGESOURCE),
 #define FRAME_CONTEXT_MENU \
   CONTEXT_MENU_ITEM(OPEN_FRAME_WIN_STR,     GNOME_STOCK_MENU_OPEN,    GALEON_CONTEXT_OPENFRAME),       \
   CONTEXT_MENU_ITEM(OPEN_FRAME_NEW_WIN_STR, GNOME_STOCK_MENU_OPEN,    GALEON_CONTEXT_OPENFRAMEWINDOW), \
@@ -104,6 +109,7 @@ static char FULLSCREEN_VIEW_STR[]    = N_("Full Screen View");
   CONTEXT_MENU_ITEM(COPY_FRAME_LOC_STR,     GNOME_STOCK_MENU_COPY,    GALEON_CONTEXT_COPYFRAMELOC),    \
   CONTEXT_MENU_ITEM(ADD_BM_FRAME_STR,       GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_ADDTEMPBMFRAME),
 #define SHORT_CONTEXT_MENU \
+  CONTEXT_MENU_ITEM(COPY_LINK_LOC_STR, GNOME_STOCK_MENU_COPY,  GALEON_CONTEXT_COPYDOCLOC),    \
   CONTEXT_MENU_ITEM(SAVE_AS_STR,               GNOME_STOCK_MENU_SAVE_AS, GALEON_CONTEXT_SAVEAS),     \
   CONTEXT_MENU_ITEM(VIEW_SOURCE_STR,           GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_VIEWSOURCE),     \
   CONTEXT_MENU_TOGGLEITEM(FULLSCREEN_VIEW_STR, GNOME_STOCK_MENU_BLANK,   GALEON_CONTEXT_FULLSCREEN), \
@@ -124,6 +130,7 @@ typedef enum
 	GALEON_CONTEXT_OPENTAB,
 	GALEON_CONTEXT_DOWNLOAD,
 	GALEON_CONTEXT_COPYLOC,
+	GALEON_CONTEXT_COPYDOCLOC,
 	GALEON_CONTEXT_ADDTEMPBMLINK,
 	GALEON_CONTEXT_OPENIMAGE,
 	GALEON_CONTEXT_OPENIMAGEWINDOW,
@@ -173,6 +180,9 @@ static void context_menu_create_fake_bookmarks_root_recursively
 static void fill_in_image_context_menu (gchar *imagename, 
 	gint pos, GnomeUIInfo *uiinfo);
 
+/* local variables */
+static gboolean found_bookmarks = FALSE;
+
 /**
  * context_set_full_screen_check: searches for a check button and sets it
  **/
@@ -217,9 +227,11 @@ static void context_set_navigation_buttons (GaleonEmbed *embed,
 	gint go_back_pos = 0, go_forward_pos = 1, go_up_pos = 2;
 	
 	show_history_popup = 
-	      gnome_config_get_bool ("/galeon/Mouse/history_popup=FALSE");
-	can_go_back = gtk_moz_embed_can_go_back (GTK_MOZ_EMBED(embed->mozEmbed));
-	can_go_forward = gtk_moz_embed_can_go_forward (GTK_MOZ_EMBED(embed->mozEmbed));
+	      eel_gconf_get_boolean (CONF_MOUSE_HISTORY_POPUP);
+	can_go_back = gtk_moz_embed_can_go_back 
+		(GTK_MOZ_EMBED(embed->mozEmbed));
+	can_go_forward = gtk_moz_embed_can_go_forward 
+		(GTK_MOZ_EMBED(embed->mozEmbed));
 	can_go_up = embed_can_go_up (embed);
 	
 	for (i = 0; uiinfo[i].type != GNOME_APP_UI_ENDOFINFO; i++)
@@ -417,7 +429,7 @@ static void context_do_action (GaleonContextAction action,
 {
 	gchar *url;
 
-	gboolean tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
+	gboolean tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED);
 
 	switch (action)
 	{
@@ -457,6 +469,9 @@ static void context_do_action (GaleonContextAction action,
 	case GALEON_CONTEXT_COPYLOC:
 		copy_to_clipboard (info->link, embed);
 		break;
+	case GALEON_CONTEXT_COPYDOCLOC:
+		copy_to_clipboard (embed->site_location, embed);
+		break;
 	case GALEON_CONTEXT_ADDTEMPBMLINK:
 		add_temp_link_bookmark (info->linktext, info->link, NULL);
 		break;
@@ -585,10 +600,8 @@ context_show_appearance_menu (GaleonWindow *window, GtkMenu *menu,
 {
 	GtkMenu *appearance_menu;
 	GtkWidget *appearance_menu_item;
-	GSList *icons_radiogroup = NULL;
 	GSList *radiogroup = NULL;
 	ToolbarStyle style;
-	ToolbarIcons icons;
 
 	if (in_submenu)
 	{
@@ -641,7 +654,7 @@ context_show_appearance_menu (GaleonWindow *window, GtkMenu *menu,
 	{
 		/* we're on the main toolbar */
 
-		style = gnome_config_get_int (CONF_APPEARANCE_TOOLBAR_STYLE);
+		style = eel_gconf_get_integer (CONF_TOOLBAR_STYLE);
 		context_menu_add_radio_item 
 			(appearance_menu, _("Text _beside icons"),
 			 context_main_toolbar_appearance_menu_cb, NULL,
@@ -661,22 +674,6 @@ context_show_appearance_menu (GaleonWindow *window, GtkMenu *menu,
 
 		context_menu_add_seperator (appearance_menu);
 
-		icons = gnome_config_get_int (CONF_APPEARANCE_TOOLBAR_ICONS);
-		context_menu_add_radio_item 
-			(appearance_menu, _("_Default icons"),
-			 context_main_toolbar_icons_menu_cb, NULL,
-			 (icons == TOOLBAR_ICONS_NORMAL), &icons_radiogroup);
-		context_menu_add_radio_item 
-			(appearance_menu, _("_Large icons"),
-			 context_main_toolbar_icons_menu_cb, NULL,
-			 (icons == TOOLBAR_ICONS_LARGE), &icons_radiogroup);
-		context_menu_add_radio_item 
-			(appearance_menu, _("_Nautilus icons"),
-			 context_main_toolbar_icons_menu_cb, NULL,
-			 (icons == TOOLBAR_ICONS_NAUTILUS), &icons_radiogroup);
-
-		context_menu_add_seperator (appearance_menu);
-
 		context_menu_add_item (appearance_menu, _("_Edit toolbar..."),
 				       context_main_toolbar_edit_toolbar_cb,
 				       window, GNOME_STOCK_MENU_PROP, TRUE);
@@ -703,6 +700,12 @@ context_menu_add_bookmarks (GtkMenu *menu, GaleonEmbed *embed)
 	new_item = gtk_menu_item_new ();
 	gtk_menu_append (menu, new_item);
 
+	if (found_bookmarks)
+	{
+		context_menu_add_seperator (menu);
+		found_bookmarks = FALSE;
+	}
+	
 	bookmarks_menu_create_recursively (fake_bookmarks_root, 
 					   menu, NULL, tips, FALSE, TRUE);
 	
@@ -718,13 +721,14 @@ context_menu_add_bookmarks (GtkMenu *menu, GaleonEmbed *embed)
  * fill_in_image_context_menu: fill in the image part
  * of the context menu with the image name
  */
-/* FIXME LEAAAAAAKKKK ! */
 static void
 fill_in_image_context_menu (gchar *imagename, gint pos,
 			GnomeUIInfo *uiinfo)
 {
 	gchar *name, *s;
+	const gchar *host;
 	gint length;
+	GnomeVFSURI *vfs_uri = gnome_vfs_uri_new (imagename);
 
 	g_assert (imagename != NULL);
 	
@@ -740,11 +744,27 @@ fill_in_image_context_menu (gchar *imagename, gint pos,
 	}
 	
 	name = escape_uline_accel (s);
+	g_free(uiinfo[pos].label);
 	uiinfo[pos].label = 
 		g_strdup_printf (_("Open image (%s)"), name);
 
 	g_free (s);
 	g_free (name);
+
+	g_free(uiinfo[pos+7].label);
+	host = gnome_vfs_uri_get_host_name(vfs_uri);
+	if (host)
+	{
+		uiinfo[pos+7].label = 
+			g_strdup_printf (_("Block images from %s"), host);
+	}
+	else
+	{
+		uiinfo[pos+7].label = 
+			g_strdup (_("Block image source"));
+	}
+
+	gnome_vfs_uri_unref(vfs_uri);
 }
 
 /**
@@ -766,8 +786,11 @@ context_menu_create_fake_bookmarks_root_recursively (BookmarkItem *b,
 						     GList **l)
 {
 	GList *li;
-	if (b->create_context_menu) 
+	if (b->create_context_menu)
+	{
+		found_bookmarks = TRUE;
 		*l = g_list_append (*l, b);
+	}
 	if (!b->alias_of)
 		for (li = b->list; li; li = li->next)
 			context_menu_create_fake_bookmarks_root_recursively
