@@ -17,6 +17,7 @@
 #include "window.h"
 #include "misc.h"
 #include "dialog.h"
+#include "prefs.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -151,9 +152,8 @@ dialog_set_parent (GtkWidget *dialog, GtkWidget *parent_window)
 	/* fullscreen mode flag */
 	extern gboolean fullscreen_active;
 
-	/* check */
-	g_assert (GTK_IS_WINDOW (dialog));
-	g_assert (parent_window == NULL || GTK_IS_WINDOW (parent_window));
+	/* Won't get far if this is true... */
+	if (dialog == NULL) return;
 
 	/* if it's not a toplevel app (like e.g. the bm editor) */
 	if (!(GNOME_IS_APP (dialog)) && parent_window != NULL)
@@ -165,7 +165,7 @@ dialog_set_parent (GtkWidget *dialog, GtkWidget *parent_window)
 
 	/* check if we're in fullscreen mode and stacked above dock */
 	if (fullscreen_active && 
-	    gnome_config_get_bool (CONF_APPEARANCE_FULLSCREEN_STACKING))
+	    eel_gconf_get_boolean (CONF_WINDOWS_FULLSCREEN_STACKING))
 	{
 		/* get to the top (must be shown first) */
 		gtk_widget_show (dialog);
