@@ -16,14 +16,21 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+#include <glade.h>
+
 #include <gtk/gtkwindow.h>
 
-void
-dialog_ask_file_name (const gchar *title, const gchar *label_title, 
-		      const gchar *def,
-		      FileCallbackFunc callback_ok, 
-		      FileCallbackFunc callback_cancel, 
-		      gboolean must_exist, gboolean warn_overwrite,
-		      gpointer user_data);
-gboolean 
-dialog_ask_question (const gchar *question, GtkWindow *parent);
+gboolean dialog_ask_question (const gchar *question, GtkWidget *parent);
+gboolean dialog_check_overwrite_file (const gchar *fname, GtkWidget *parent);
+gboolean dialog_confirm_check (GtkWidget *parent, const gchar *dialogTitle,
+			       const gchar *text, const gchar *checkMsg,
+			       gboolean *checkValue);
+void dialog_set_parent (GtkWidget *dialog, GtkWidget *parent_window);
+
+#ifdef __cplusplus
+}
+#endif
