@@ -31,6 +31,7 @@
 #include "MozRegisterComponents.h"
 #include "filepicker.h"
 #include "dialog.h"
+#include "mozilla_notifiers.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -52,9 +53,12 @@ static GtkWidget * new_num_accel_menu_item (gint num, gchar *origtext,
 					    gboolean lettersok, 
 					    GtkWidget *menu);
 static void save_url (GaleonEmbed *embed, char *target, 
-		      char* default_dir_pref, gboolean ask);
-static void save_url_message (GaleonEmbed *embed, char *path);
+		      char* default_dir_pref, gboolean ask,
+		      gboolean main, gboolean ask_content);
+static void save_url_message (GaleonEmbed *embed, char *path,
+			      gboolean result);
 static void embed_select_new_page (GaleonEmbed *embed);
+static gboolean save_with_content (GaleonEmbed *embed, char *path, gboolean main);
 
 /** The global list of all GaleonEmbed structures */
 GList *all_embeds = NULL;
@@ -73,7 +77,6 @@ gboolean pushed_startup = FALSE;
  * with the order of these unless you know what you're doing! -- MattA */
 const GtkTargetEntry embed_drop_types[] =
 {
-	{ "GALEON_EMBED",    0, DND_TARGET_GALEON_EMBED    },
 	{ "GALEON_URL",      0, DND_TARGET_GALEON_URL      },
 	{ "text/uri-list",   0, DND_TARGET_TEXT_URI_LIST   },
 	{ "_NETSCAPE_URL",   0, DND_TARGET_NETSCAPE_URL    },
@@ -182,7 +185,7 @@ embed_create_default (GaleonEmbed *previous_embed, gboolean new_window)
 	embed = embed_create_from_url (previous_embed, url, TRUE, new_window);
 
 	/* copy across session history (the wrapper must be initialized) */
-	start_page = gnome_config_get_int (CONF_GENERAL_NEWPAGE_TYPE);
+	start_page = eel_gconf_get_integer (CONF_GENERAL_NEWPAGE_TYPE);
 	if (previous_embed != NULL && start_page == STARTPAGE_LAST)
 	{
 		mozilla_copy_session_history (previous_embed, embed);
@@ -240,6 +243,8 @@ embed_create_no_window (void)
 	embed->site_title = g_strdup (_("Untitled"));
 	embed->site_title_utf8 = locale_to_utf8 (embed->site_title);
 
+	embed->notebook_close_button = NULL;
+
 	/* initial zoom level */
 	embed->zoom = 100;
 	
@@ -283,18 +288,19 @@ embed_get_default_url (GaleonEmbed *previous_embed)
 	/* find out where we're supposed to start */
 	if (previous_embed == NULL)
 	{
-		page_type = gnome_config_get_int (CONF_GENERAL_STARTPAGE_TYPE);
+		page_type = eel_gconf_get_integer 
+			(CONF_GENERAL_STARTPAGE_TYPE);
 	}
 	else
 	{
-		page_type = gnome_config_get_int (CONF_GENERAL_NEWPAGE_TYPE);
+		page_type = eel_gconf_get_integer (CONF_GENERAL_NEWPAGE_TYPE);
 	}
 
        	/* return the appropriate page */
 	if (page_type == STARTPAGE_HOME)
 	{
 		/* get location of home page */
-		home_page_url = gnome_config_get_string(CONF_GENERAL_HOMEPAGE);
+		home_page_url = eel_gconf_get_string(CONF_GENERAL_HOMEPAGE);
 
 		/* load home page, if set */
 		if (home_page_url != NULL)
@@ -346,12 +352,12 @@ embed_set_visibility (GaleonEmbed *embed, gboolean visibility)
 	/* test for showing */
 	if (visibility && !embed->is_visible)
 	{
-		window_show_one_embed (window);
-
 		/* show the widget */
-		/* don't ask me why this has to be done after showing
+		/* don't ask me why this has to be done BEFORE showing
 		 * the main window, it just does... */
 		gtk_widget_show (embed->mozEmbed);
+
+		window_show_one_embed (window);
 	}
 
 	/* new status */
@@ -455,17 +461,13 @@ embed_activate_link (GaleonEmbed *previous, GaleonEmbed **new_embed,
 	{
 	case 1:
 		/* left button (or no button) */
-		if (!previous)
-		{
+		if (!previous) {
 			new = previous = embed_create_from_url 
 				(NULL, "about:blank", FALSE, TRUE);
 		}
-		if (shifted)
-		{
+		if (shifted) {
 			downloader_save_link (previous, url);
-		}
-		else
-		{
+		} else {
 			embed_load_url (previous, url);
 		}
 		handled = TRUE;
@@ -473,26 +475,19 @@ embed_activate_link (GaleonEmbed *previous, GaleonEmbed **new_embed,
 
 	case 2:
 		/* middle button */
-		if (link_state == LINKSTATE_NEWTAB)
-		{
+		if (link_state == LINKSTATE_NEWTAB) {
 			tabbed_mode = TRUE;
 		}
-		else if (link_state == LINKSTATE_NEWWIN)
-		{
+		else if (link_state == LINKSTATE_NEWWIN) {
 			tabbed_mode = FALSE;
+		} else {
+			tabbed_mode = eel_gconf_get_boolean 
+				(CONF_TABS_TABBED);
 		}
-		else
-		{
-			tabbed_mode = gnome_config_get_bool 
-				(CONF_APPEARANCE_TABBED);
-		}
-		if (shifted && !(alted || ctrled))
-		{
+		if (shifted && !(alted || ctrled)) {
 			new = embed_create_from_url (previous, url,
 						     FALSE, tabbed_mode);
-		}
-		else
-		{
+		} else {
 			new = embed_create_from_url (previous, url,
 						     FALSE, !tabbed_mode);
 		}
@@ -511,8 +506,7 @@ embed_activate_link (GaleonEmbed *previous, GaleonEmbed **new_embed,
 	}
 
 	/* store return value (NULL if not handled) */
-	if (new_embed != NULL)
-	{
+	if (new_embed != NULL) {
 		*new_embed = new;
 	}
 
@@ -543,21 +537,23 @@ embed_switch_to_page (GaleonEmbed *embed)
 }
 
 /**
- * embed_startup_init: initialitation that need to occur only one time
+ * embed_startup_init: initialisation that needs to occur only one time
  * before any gtkmozembed object is created
  */
 void
-embed_startup_init (void)
+embed_startup_init ()
 {
 	/* general mozilla stuff that needs doing */
 	embed_single_init ();
 	mozilla_register_components ();
 	mozilla_load_prefs ();
-	
+
+	/* installs notifiers for some mozilla prefs */
+	mozilla_notifiers_init();
+
 	/* only register the external ftp handler if it isn't galeon */
-	if (gnome_config_get_int(CONF_HANDLER_EXTERNAL_FTP) == 1 &&
-	    !(preferences_galeon_used_by_gnome_for_ftp ()))
-	{
+	if (eel_gconf_get_integer(CONF_DOWNLOADING_EXTERNAL_FTP) == 1 &&
+		!(preferences_galeon_used_by_gnome_for_ftp ()))	{
 		mozilla_register_FtpProtocolHandler ();
 	}
 }
@@ -660,8 +656,7 @@ embed_single_init (void)
 void 
 embed_wrapper_init (GaleonEmbed *embed)
 {
-	gboolean event_listener = gnome_config_get_bool(CONF_MOUSE_LINKS_DRAG);
-	embed->wrapper = mozilla_wrapper_init (embed, event_listener);
+	embed->wrapper = mozilla_wrapper_init (embed);
 }
 
 /**
@@ -682,7 +677,8 @@ embed_progress_clear(GaleonEmbed *embed)
 void 
 embed_save_image (GaleonEmbed *embed, gchar *url, gboolean ask)
 {
-	save_url (embed, url, CONF_STATE_SAVE_IMAGE_DIR, ask);
+	save_url (embed, url, CONF_STATE_SAVE_IMAGE_DIR, 
+		  ask, FALSE, FALSE);
 }
 
 /**
@@ -713,7 +709,7 @@ void embed_set_image_as_background (GaleonEmbed *embed, gchar *url)
 void embed_save_document (GaleonEmbed *embed, gboolean main)
 {
 	gchar *target = NULL;
-
+		
 	/* get either frame or main document */
 	if (main)
 	{
@@ -724,8 +720,15 @@ void embed_save_document (GaleonEmbed *embed, gboolean main)
 		target = mozilla_get_document_url (embed);
 	}
 
+	if (!target)
+	{
+		gnome_error_dialog (_("No document is loaded"));
+		return;
+	}
+
 	/* save it */
-	save_url (embed, target,  CONF_STATE_SAVE_DIR, TRUE);
+	save_url (embed, target,  CONF_STATE_SAVE_DIR, 
+		  TRUE, TRUE, main);
 
 	/* free */
 	g_free (target);
@@ -733,10 +736,12 @@ void embed_save_document (GaleonEmbed *embed, gboolean main)
 
 /**
  * save_url: save a given URL
+ * FIXME pass main here is weird
  */
 static void 
 save_url (GaleonEmbed *embed, gchar *target, 
-	  gchar *default_dir_pref, gboolean ask)
+	  gchar *default_dir_pref, gboolean ask,
+	  gboolean ask_content, gboolean main)
 {
 	GnomeVFSURI *uri;
 	gchar *retPath = NULL;
@@ -744,6 +749,7 @@ save_url (GaleonEmbed *embed, gchar *target,
 	gchar *dirName;
 	gchar *retDir;
 	gboolean ret;
+	gboolean content;
 
 	/* Get a filename from the target url */
 	uri = gnome_vfs_uri_new (target);
@@ -753,16 +759,23 @@ save_url (GaleonEmbed *embed, gchar *target,
 		gnome_vfs_uri_unref (uri);
 	}
 
-	dirName = gnome_config_get_string (default_dir_pref);
+	dirName = eel_gconf_get_string (default_dir_pref);
+
+	if (!dirName || strcmp (dirName,"~")==0)
+	{
+		g_free (dirName);
+		dirName = g_strdup(g_get_home_dir ());
+	}
 
 	if (ask)
 	{
-		ret = show_file_picker (NULL, N_("Select the file to save"),
-					dirName, fileName, modeSave, &retPath);                
+		ret = show_file_picker (NULL, _("Select the file to save"),
+					dirName, fileName, modeSave, &retPath,
+					ask_content ? &content : NULL);
 	}
 	else
 	{
-		retPath = g_strconcat (dirName, fileName, NULL);
+		retPath = g_concat_dir_and_file (dirName, fileName);
 		ret = TRUE;
 	}
 
@@ -772,14 +785,21 @@ save_url (GaleonEmbed *embed, gchar *target,
 		if (uri)
 		{
 			retDir = gnome_vfs_uri_extract_dirname (uri);
-			mozilla_save_url (embed, target, 
-					  retPath, ACTION_NONE);
-			save_url_message (embed,retPath);
+
+			if (ask_content && content)
+			{
+				ret = save_with_content (embed, retPath, main);
+			}
+			else
+			{
+				ret = mozilla_save_url (embed, target, 
+							retPath, ACTION_NONE);
+			}
+			save_url_message (embed, retPath, ret);
 
 			/* set default save dir */
-			gnome_config_set_string (default_dir_pref, 
+			eel_gconf_set_string (default_dir_pref, 
 						 retDir);
-			gnome_config_sync ();
 
 			g_free (retDir);
 			gnome_vfs_uri_unref (uri);
@@ -791,11 +811,44 @@ save_url (GaleonEmbed *embed, gchar *target,
 	g_free (retPath);
 }
 
+static
+gboolean save_with_content (GaleonEmbed *embed, char *path, gboolean main)
+{
+	gchar *data_dir;
+	gboolean ret;
+
+	return_val_if_not_embed (embed, FALSE);
+
+	data_dir = g_strconcat (path, "content", NULL);
+
+	if (mkdir (data_dir, 0755) != -1)
+	{
+		ret = mozilla_save_document (embed, path, data_dir,
+					     ACTION_NONE, main); 
+	}
+	else 
+	{
+		ret = FALSE;
+	}
+
+	g_free (data_dir);
+
+	return ret;
+}
+
 static 
-void save_url_message (GaleonEmbed *embed, char *path)
+void save_url_message (GaleonEmbed *embed, char *path, gboolean result)
 {
 	char *temp_message;
-	temp_message = g_strdup_printf(_("Saved as %s"), path);
+	if (result)
+	{
+		temp_message = g_strdup_printf(_("Saved as %s"), path);
+	}
+	else
+	{
+		temp_message = g_strdup_printf(_("Save as %s failed"), path);
+	}
+
 	window_update_temp_message (embed->parent_window,
 				    temp_message);
 	g_free (temp_message);
@@ -811,12 +864,18 @@ void embed_view_source (GaleonEmbed *embed, gboolean main,
 	
 	if (main) {
 		url = mozilla_get_main_document_url (embed);
-	} else {
+	} 
+	else {
 		url = mozilla_get_document_url (embed);
 	}
-	g_return_if_fail (url);
 
-	if (gnome_config_get_bool (CONF_HANDLERS_USE_EXTERNAL_SOURCE_VIEWER))
+	if (!url)
+	{
+		gnome_error_dialog (_("No document is loaded"));
+		return;
+	}
+
+	if (eel_gconf_get_boolean (CONF_PROGRAMS_USE_EXTERNAL_SOURCE_VIEWER))
 	{
 		embed_view_source_external (embed, url);
 	}
@@ -832,11 +891,12 @@ void embed_view_source (GaleonEmbed *embed, gboolean main,
  */
 static void embed_view_source_external (GaleonEmbed *embed, const gchar *url)
 {
-	/* FIXME is use /tmp ok ? */
-	char *filename = g_strdup("/tmp/galeon-viewsource-XXXXXX");
+	char *filename;
 	char *htmlfile;
 	int result;
 
+	filename = g_strconcat(g_get_tmp_dir(),"/galeon-viewsource-XXXXXX", NULL);
+
 	/* get a name for the temporary file */
 	result = mkstemp (filename);
 
@@ -870,6 +930,12 @@ embed_close (GaleonEmbed *embed)
 		return;
 	}
 	embed->is_closing = TRUE;
+	
+	/* remove the close timeout, if its active */
+	if (embed->close_timeout_id)
+	{
+		gtk_timeout_remove (embed->close_timeout_id);
+	}
 
 	/* check args */
 	window = embed->parent_window;
@@ -957,7 +1023,7 @@ embed_update_page_location (GaleonEmbed *embed)
 		window_set_url_entry (window, new_location);
 	
 		/* update the drag location pixmap */
-		if (gnome_config_get_bool (CONF_GENERAL_FAVICONS_ENABLED) &&
+		if (eel_gconf_get_boolean (CONF_BOOKMARKS_FAVICONS_ENABLED) &&
 		    embed->parent_window->toolBarOn && 
 		    window->drag_pixmap != NULL)
 		{
@@ -981,7 +1047,7 @@ embed_update_page_title (GaleonEmbed *embed)
 	GaleonWindow *window;
 
 	/* set notebook label (although this might not be visible) */
-	embed_set_notebook_label (embed);
+	embed_update_tab_title (embed);
 
 	/* if this page isn't being viewed, get out now */
 	if (!embed->is_active)
@@ -994,7 +1060,8 @@ embed_update_page_title (GaleonEmbed *embed)
 	return_if_not_window (window);
 
 	/* get the format string */
-	title_string = gnome_config_get_string (CONF_APPEARANCE_WINDOWS_TITLE);
+	title_string = eel_gconf_get_string (CONF_WINDOWS_TITLE);
+	if (!title_string) return;
 
 	/* format the full title */
 	full_title = g_strdup_printf (title_string, embed->site_title);
@@ -1008,11 +1075,10 @@ embed_update_page_title (GaleonEmbed *embed)
 }
 
 /**
- * embed_set_notebook_label: sets the notebook tab label belonging to embed
- * to the string contained in text.
+ * embed_update_tab_title: updates the tab to contain the page title
  */
 void
-embed_set_notebook_label (GaleonEmbed *embed)
+embed_update_tab_title (GaleonEmbed *embed)
 {
 	GaleonWindow *window;
 	gchar *shortened;
@@ -1026,22 +1092,22 @@ embed_set_notebook_label (GaleonEmbed *embed)
 	return_if_not_window (window);
 
         /* shorten notebook's label */
-	tab = embed->notebook_event_box;
-	style = gnome_config_get_int (CONF_APPEARANCE_TABBED_SHORTEN_STYLE);
+	tab = embed->notebook_hbox;
+	style = eel_gconf_get_integer (CONF_TABS_TABBED_SHORTEN_STYLE);
         switch (style)
         {
         case 0:
 		/* abbreviate the text label */
                 gtk_widget_set_usize (GTK_WIDGET (tab), -1, -1);
-                length = gnome_config_get_int 
-			(CONF_APPEARANCE_TABBED_SHORTEN_CHARS);   
+                length = eel_gconf_get_integer (
+			CONF_TABS_TABBED_SHORTEN_CHARS);   
                 shortened = shorten_name (embed->site_title, length);
                 break;
 
         case 1: 
 		/* set fixed size of the label widget */
-		length = gnome_config_get_int 
-			(CONF_APPEARANCE_TABBED_SHORTEN_POINTS);
+		length = eel_gconf_get_integer 
+			(CONF_TABS_TABBED_SHORTEN_POINTS);
                 gtk_widget_set_usize (GTK_WIDGET (tab), length, -2);
                	shortened = embed->site_title;
                 break;
@@ -1072,11 +1138,13 @@ embed_set_notebook_label (GaleonEmbed *embed)
 }
 
 /**
- * embed_set_notebook_label_status: sets the status for the notebook tab
- * label belonging to embed.
+ * embed_update_tab_status: sets the status for the notebook tab
+ * label belonging to embed.  this means changing the color of the tab
+ * title and changing the closebutton between the active/inactive states if
+ * necessary
  */
 void
-embed_set_notebook_label_status (GaleonEmbed *embed)
+embed_update_tab_status (GaleonEmbed *embed)
 {
 	GtkWidget *label;
 
@@ -1090,48 +1158,50 @@ embed_set_notebook_label_status (GaleonEmbed *embed)
 	if (embed->load_started > 0)
 	{
 		/* loading... */
-		if (embed->is_active)
-		{
-			/* active embed */
-			gtk_widget_set_style (label,
-				active_loading_text_style);
-		}
-		else
-		{
-			/* inactive */
-			gtk_widget_set_style (label,
-				inactive_loading_text_style);
-		}
+		gtk_widget_set_style (label, loading_text_style);
 	}
 	else if (!(embed->has_been_viewed))
 	{
 		/* loaded, with new content */
-		gtk_widget_set_style (label, inactive_new_text_style);
+		gtk_widget_set_style (label, new_text_style);
 	}
 	else if (embed->is_active)
 	{
 		/* loaded, viewed */
 		gtk_widget_set_rc_style (label);
 	}
-	else
+
+	/* update closebutton status */
+	if (embed->notebook_close_button != NULL)
 	{
-		/* loaded, seen, but currently not at the front */
-		gtk_widget_set_style (label, inactive_text_style);
+		gboolean make_close_insensitive;
+
+		make_close_insensitive = eel_gconf_get_boolean (
+			CONF_TABS_TABBED_CLOSEBUTTON_INSENSITIVE);
+
+		if (embed->is_active || !make_close_insensitive)
+		{
+			gtk_widget_set_sensitive (embed->notebook_close_button,
+						  TRUE);
+		}
+		else if (make_close_insensitive)
+		{
+			gtk_widget_set_sensitive (embed->notebook_close_button,
+						  FALSE);
+		}
 	}
 }
 
 /**
- * embed_notebook_tab_update_closebutton (internal function): tab, the
- * notebook tab belonging to embed, is updated (if necessary) to reflect
- * the user's preference in the display of a close button.  if tab is
- * null, it will be found using embed.
+ * embed_notebook_tab_update_close_button: the notebook tab belonging to
+ * embed, is updated (if necessary) to reflect the user's preference in the
+ * display of a close button.
  */
 void
-embed_update_tab_closebutton (GaleonEmbed *embed)
+embed_update_tab_close_button (GaleonEmbed *embed)
 {
 	GaleonWindow *window;
-	GtkWidget *button;
-	GtkWidget *pixmap;
+	GtkWidget *button, *pixmap;
 	gboolean show;
 
 	/* check args */
@@ -1140,7 +1210,7 @@ embed_update_tab_closebutton (GaleonEmbed *embed)
 	return_if_not_window (window);
 
 	/* check prefs */
-	show = gnome_config_get_bool (CONF_APPEARANCE_TABBED_CLOSEBUTTON);
+	show = eel_gconf_get_boolean (CONF_TABS_TABBED_CLOSEBUTTON);
 
 	/* check for case when this is being destroyed */
 	if (!show && embed->notebook_close_button != NULL)
@@ -1165,22 +1235,25 @@ embed_update_tab_closebutton (GaleonEmbed *embed)
 			}
 		}
 
-		/* build and setup the button */
+		/* build button */
 		button = embed->notebook_close_button = gtk_button_new ();
 		gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-		
-		/* put the close pixmap into the button */
 		pixmap = gtk_pixmap_new (close_pix->pixmap, close_pix->mask);
+		gtk_widget_show (pixmap);
 		gtk_container_add (GTK_CONTAINER (button), pixmap);
 
 		/* pack button into tab */
 		gtk_box_pack_start (GTK_BOX (embed->notebook_hbox), button,
 				    FALSE, FALSE, 0);
-		gtk_widget_show_all (button);
 
-		/* when the close button is clicked, close the embed */
-		gtk_signal_connect (GTK_OBJECT (button), "clicked", 
-				    embed_tab_close_clicked_cb, embed);
+		/* connect click handler */
+		gtk_signal_connect (GTK_OBJECT (button), "clicked",
+				    GTK_SIGNAL_FUNC (
+				    	embed_tab_close_button_clicked_cb),
+				    embed);
+
+		/* show the button */
+		gtk_widget_show (button);
 	}
 }
 
@@ -1418,8 +1491,7 @@ embed_shift_tab (GaleonEmbed *embed, gint amount)
 
 /* embed_move_tab - moves an embed.  dest_embed can be an embed in the
  * same window or in another window, or NULL if embed should be moved to a
- * new window (not yet implemented, since i don't have a way to easily test
- * it) -Dan */
+ * new window */
 void
 embed_move_tab (GaleonEmbed *embed, GaleonEmbed *dest_embed)
 {
@@ -1516,15 +1588,11 @@ embed_move_tab (GaleonEmbed *embed, GaleonEmbed *dest_embed)
 		gtk_notebook_insert_page (dest_notebook, embed->mozEmbed, 
 					  embed->notebook_hbox, dest_page);
 
-		/* if the embed was dropped on top of the active embed,
-		 * switch to the dropped embed's page */
-		if (dest_embed != NULL && dest_embed->is_active)
-		{
-			gtk_notebook_set_page (dest_notebook, dest_page);
-		}
+		/* switch to the dropped embed's page */
+		gtk_notebook_set_page (dest_notebook, dest_page);
 
 		/* update tab state */
-		embed_set_notebook_label_status (embed);
+		embed_update_tab_status (embed);
 
 		/* and remove the tab's reference */
 		gtk_object_unref (GTK_OBJECT (embed->notebook_hbox));
@@ -1556,7 +1624,9 @@ new_num_accel_menu_item (gint num, gchar *origtext, gboolean lettersok,
 static void
 add_bf_menu (GtkWidget *menu, char* label, int index, GaleonEmbed *embed, int level)
 {
-	GtkWidget *item = new_num_accel_menu_item (level, label, TRUE, menu);
+	gchar *tmp = escape_uline_accel (label);
+	GtkWidget *item = new_num_accel_menu_item (level, tmp, TRUE, menu);
+	g_free (tmp);
 
 	gtk_widget_show (item);
 	gtk_object_set_user_data (GTK_OBJECT (item), embed);
