@@ -55,9 +55,9 @@ void embed_open_frame (GaleonEmbed *embed, gboolean same_embed,
 void embed_reload (GaleonEmbed *embed);
 void embed_update_page_location (GaleonEmbed *embed);
 void embed_update_page_title (GaleonEmbed *embed);
-void embed_set_notebook_label (GaleonEmbed *embed);
-void embed_set_notebook_label_status (GaleonEmbed *embed);
-void embed_update_tab_closebutton (GaleonEmbed *embed);
+void embed_update_tab_title (GaleonEmbed *embed);
+void embed_update_tab_status (GaleonEmbed *embed);
+void embed_update_tab_close_button (GaleonEmbed *embed);
 void embed_go_up (GaleonEmbed *embed, gint levels, LinkState state);
 gboolean embed_can_go_up (GaleonEmbed *embed);
 void embed_set_zoom (GaleonEmbed *embed, gint zoom);
@@ -74,14 +74,5 @@ void embed_drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 			          gint x, gint y, 
 			          GtkSelectionData *selection_data,
 			          guint info, guint time, GaleonEmbed *embed);
-void embed_notebook_switch_page_cb (GtkNotebook *notebook, 
-				    GtkNotebookPage *page, guint page_num);
-void embed_notebook_add_remove_page_cb (GtkContainer *container,
-					GtkWidget *widget, 
-					GaleonWindow *window);
-void embed_tab_close_clicked_cb (GtkWidget *button, GaleonEmbed *embed);
-void embed_tab_drag_data_get_cb (GtkWidget *widget, GdkDragContext *context,
-				 GtkSelectionData *selection_data, guint info,
-				 guint time, GaleonEmbed *embed);
-void embed_tab_button_press_event_cb (GtkWidget *widget, GdkEventButton *event,
-				      GaleonEmbed *embed);
+void embed_tab_close_button_clicked_cb (GtkWidget *button, GaleonEmbed *embed);
+void save_site_location (GaleonEmbed *embed);
