@@ -75,10 +75,9 @@ struct _GaleonEmbed
 	GtkWidget *mozEmbed;
 
 	/* tab widgets */
-	GtkWidget *notebook_event_box;           /* overall event box */
-	GtkWidget *notebook_hbox;                /* packing box       */
-	GtkWidget *notebook_label;               /* the label         */
-	GtkWidget *notebook_close_button;        /* the button        */
+	GtkWidget *notebook_hbox;                /* packing box           */
+	GtkWidget *notebook_label;               /* the label             */
+	GtkWidget *notebook_close_button;        /* active close button   */
 
 	/* C++ helper, a "GaleonWrapper" */
         gpointer wrapper;
@@ -123,6 +122,9 @@ struct _GaleonEmbed
 
 	/* how this embed was focused if applicable */
 	FocusType focus_type;
+
+	/* close timeout to ensure all mozembed callbacks have finished */
+	gint close_timeout_id;
 };
 
 /**
@@ -180,7 +182,6 @@ struct _GaleonWindow
 	GtkWidget *home_button;
 	GtkWidget *stop_button;
 	GtkWidget *go_button;
-	GtkWidget *find_button;
 	GtkWidget *zoom_spin;
 	GtkWidget *drag_event_box;
 	GtkWidget *drag_pixmap;
@@ -199,10 +200,12 @@ struct _GaleonWindow
 	GtkWidget *file_session_history_separator_sup;
 	GtkWidget *file_session_history_separator_inf;
 	GtkWidget *view_toolbar;
+	GtkWidget *view_bookmarks;
 	GtkWidget *view_statusbar;
 	GtkWidget *view_menubar;
 	GtkWidget *view_fullscreen;
 	GtkWidget *detach_tab;
+	GtkWidget *move_tab_to_window_menu;
 	GtkWidget *next_tab;
 	GtkWidget *prev_tab;
 	GtkWidget *move_tab_left;
@@ -253,6 +256,13 @@ struct _GaleonWindow
 
 	/* saved layout */
 	gchar *layout;
+
+	/* notifiers IDs to remove them when the window is destroyed */
+	GList *notifiers_id;
+
+	/* used for draggable tabs */
+	gint drag_motion_signal;
+	gint drag_release_signal;
 };
 
 /**
@@ -606,6 +616,19 @@ typedef struct
 	gboolean alwaysUse;
 } MimeItem;
 
+typedef struct
+{
+	gchar *serif;
+	gchar *sansserif;
+	gchar *cursive;
+	gchar *fantasy;
+	gchar *monospace;
+
+	int min_size;
+	
+	gboolean changed;
+} LanguageFontInfo;
+
 /**
  * PreferencesDialog: structure representing Preferences dialog box
  */
@@ -617,11 +640,12 @@ typedef struct
 	GtkWidget *themes_page;
 	GtkWidget *spinner_page;
 	GtkWidget *iconlist;
+	GtkWidget *themelist;
+	GtkWidget *theme_dir_entry;
 	GtkWidget *smart_bm_example_entry;
 	GtkWidget *lang_entry;/*FIXME*/
 	GtkWidget *toolbar_style_optionmenu;
 	GtkWidget *toolbar_share_main_toolbar;
-	GtkWidget *toolbar_use_default_icons;
 	GtkWidget *startpage_entry;
 	GtkWidget *startup_show_homepage;
 	GtkWidget *newpage_show_homepage;
@@ -632,11 +656,12 @@ typedef struct
 	GtkWidget *autobookmarks_enabled_check;
 	GtkWidget *autobookmarks_count_spin;
 	GtkWidget *autobookmarks_shorten_spin;
-	GtkWidget *bookmarks_backup_enabled_check;
-	GtkWidget *bookmarks_backup_count_spin;
 	GtkWidget *smart_bm_entry_width_spin;
+	GtkWidget *smart_bm_goto_button_checkbutton;
+	GtkWidget *smart_bm_history_checkbutton;
 	GtkWidget *java_checkbutton;
 	GtkWidget *javascript_checkbutton;
+	GtkWidget *user_agent_entry;
 
 	/* toolbar editor */
 	GtkWidget *add_button;
@@ -677,6 +702,7 @@ typedef struct
 	GtkWidget *tabbed_insert_new_tabs_check;
 	GtkWidget *tabbed_always_show_check;
 	GtkWidget *tabbed_closebutton_check;
+	GtkWidget *tabbed_closebutton_insensitive_check;
 	GtkWidget *tabbed_shorten_spin;
 	GtkWidget *tabbed_shorten_spin2;
 	GtkWidget *tabbed_position_optionmenu;
@@ -684,6 +710,8 @@ typedef struct
 	GtkWidget *tabbed_separator_check;
 	GtkWidget *tabbed_vowel_check;
 	GtkWidget *tabbed_prefix_check;
+	GtkWidget *tabbed_loading_color;
+	GtkWidget *tabbed_new_color;
 
 	/* ... */
 	GtkWidget *autocompletion_enabled_checkbutton;
@@ -698,7 +726,6 @@ typedef struct
 	GtkWidget *no_proxies_for_entry;
 	GtkWidget *disk_cache_entry;
 	GtkWidget *mem_cache_entry;
-	GtkWidget *lang_encode;
 	GtkWidget *serif_font_fpicker;
 	GtkWidget *sansserif_font_fpicker;
 	GtkWidget *cursive_font_fpicker;
@@ -710,6 +737,7 @@ typedef struct
 	GtkWidget *use_own_fonts_checkbutton;
 	GtkWidget *images_always_load;
 	GtkWidget *show_toolbars_in_fullscreen_check;
+	GtkWidget *show_bookmarks_in_fullscreen_check;
 	GtkWidget *show_menubar_in_fullscreen_check;
 	GtkWidget *show_statusbar_in_fullscreen_check;
 	GtkWidget *use_external_source_viewer_check;
@@ -755,6 +783,12 @@ typedef struct
 	GtkWidget *crash_behaviour_optionmenu;
 	GtkWidget *links_drag_checkbutton;
 	GtkWidget *cache_compare_optionmenu;
+	GtkWidget *lang_encoding_optionmenu;
+	GtkWidget *autozoom_checkbutton;
+
+	/* keep the fonts changed in the prefs dialog */
+	LanguageFontInfo *fonts;
+	gboolean last_language;
 } PreferencesDialog;
 
 /**
@@ -825,7 +859,7 @@ struct _HistoryItem
  */
 typedef enum
 {
-	TOOLBAR_ITEM_NEW,
+	TOOLBAR_ITEM_NEW = 0,
 	TOOLBAR_ITEM_BACK,
 	TOOLBAR_ITEM_UP,
 	TOOLBAR_ITEM_FORWARD,
@@ -855,7 +889,7 @@ typedef struct
 	const gchar *label;
 	const gchar *tooltip;
 	const gchar *stock_icon;
-	const gchar *nautilus_icon;
+	const gchar *theme_icon;
 	gboolean priority; /* i.e. show the text in horizontal mode */
 	gboolean togglebutton;
 	gint order;        /* order value for default insertion place */
@@ -906,16 +940,6 @@ typedef enum
 	TOOLBAR_STYLE_ICONS_ONLY,       /* not used for bookmark toolbars */
 } ToolbarStyle;
 
-/**
- * ToolbarIcons: the different main toolbar types
- */
-typedef enum
-{
-	TOOLBAR_ICONS_NORMAL,
-	TOOLBAR_ICONS_LARGE,
-	TOOLBAR_ICONS_NAUTILUS
-} ToolbarIcons;
-
 /**
  * Forward declaration
  */
@@ -1074,7 +1098,6 @@ typedef enum
 	DND_TARGET_NETSCAPE_URL, 
 	DND_TARGET_TEXT_URI_LIST,
 	DND_TARGET_STRING,
-	DND_TARGET_GALEON_EMBED
 } DnDType;
 
 /**
@@ -1174,7 +1197,8 @@ typedef enum
 	PREFS_PAGE_ID_ADVANCED_NETWORK,
 	PREFS_PAGE_ID_ADVANCED_FILTERING,
 	PREFS_PAGE_ID_ADVANCED_PERSISTENT_DATA,
-	PREFS_PAGE_ID_ADVANCED_CRASH_RECOVERY
+	PREFS_PAGE_ID_ADVANCED_CRASH_RECOVERY,
+	PREFS_PAGE_COUNT /* must be last */
 } PreferencesPageId;
 
 /**
