@@ -24,8 +24,10 @@
 #include "autocompletion.h"
 #include "misc.h"
 #include "state.h"
+#include "prefs.h"
 #include "glade.h"
 #include "dialog.h"
+#include "eel-gconf-extensions.h"
 
 #include <string.h>
 #include <gtk/gtkoptionmenu.h>
@@ -65,6 +67,8 @@ static gboolean history_dirty = FALSE;
  */
 static GList *history_views = NULL;
 
+static GList *history_notifiers_id;
+
 /* sorting functions for each history column */
 static const GCompareFunc history_compare[5] =
 {
@@ -140,6 +144,7 @@ static void history_drag_data_get_cb (GtkWidget *widget,
 				      HistoryView *hv);
 void 
 window_menu_recent_activate_cb (GtkMenuItem *menuitem, gpointer user_data);
+static void history_notifiers_init(HistoryView *hv);
 
 /**
  * history_init: initialise the history subsystem
@@ -233,9 +238,12 @@ history_show_dialog (GaleonWindow *window)
 	/* build and show the history dialog */
 	hv = history_build_view (window, "history_dialog");
 
-	state_load_window_state (hv->view, "history", 530, 450);
+	state_load_window_state (hv->view, "history");
 	state_load_column_widths (hv->ctree, "history");
 
+	/* initialize notifiers */
+	history_notifiers_init (hv);
+
 	/* show the widget */
 	dialog_set_parent (hv->view, window->WMain);
 }
@@ -248,7 +256,10 @@ history_hide_dialog (HistoryView *hv)
 {
 	state_save_window_state (hv->view->window, "history");
 	state_save_column_widths (hv->ctree, "history");
-	
+
+	/* remove notifiers */
+	history_notifiers_remove ();
+
 	gtk_widget_destroy (GTK_WIDGET (hv->view));
 	history_destroy_view (hv);
 }
@@ -270,6 +281,9 @@ history_show_dock (GaleonWindow *window)
 	/* build and pack the history dock */
 	hv = history_build_view (window, "history_dock");
 
+	/* initialize notifiers */
+	history_notifiers_init (hv);
+
 	/* add as a dock */
 	window_dock (window, hv->view, CONF_STATE_HISTORY_DOCK_WIDTH);
 
@@ -450,6 +464,10 @@ history_set_page_title (const gchar *url,
 	if (strcmp (title_utf8, hi->title_utf8) == 0)
 	{
 		/* identical */
+		/* add to the Go menu of each window (FIXME?) */
+		g_list_foreach (all_windows, (GFunc)history_add_recent_menu, 
+				hi);
+		
 		g_free (title_locale);
 		g_free (title_utf8);
 		return;
@@ -526,13 +544,16 @@ history_build_view (GaleonWindow *window, const char *xml_root)
 	}
 
 	/* get the last state */
-	hv->group = gnome_config_get_bool (CONF_HISTORY_HOST_GROUP);
-	hv->search = gnome_config_get_string (CONF_HISTORY_SEARCH_TEXT);
-	hv->time = gnome_config_get_int (CONF_HISTORY_SEARCH_TIME);
+	hv->group = eel_gconf_get_boolean (CONF_HISTORY_HOST_GROUP);
+	hv->search = eel_gconf_get_string (CONF_HISTORY_SEARCH_TEXT);
+	if (hv->search == NULL)
+		hv->search = g_strdup("");
+	hv->time = eel_gconf_get_integer (CONF_HISTORY_SEARCH_TIME);
 
 	/* set in the dialog */
 	GTK_TOGGLE_BUTTON (hv->host_checkbutton)->active = hv->group;
 	gtk_entry_set_text (GTK_ENTRY (hv->search_entry), hv->search);
+
 	gtk_option_menu_set_history (GTK_OPTION_MENU (hv->time_optionmenu),
 				     hv->time);
 
@@ -971,9 +992,11 @@ history_clear_recent_menu (void)
 		GaleonWindow *window = (GaleonWindow *)(w->data);
 		GtkWidget *menu;
 		GList *children;
+		GList *tmplist;
 
 		menu = window->go_menu;
 		children = gtk_container_children (GTK_CONTAINER(menu));
+		tmplist = children;
 		children = g_list_nth (children, go_recent_pos);
 		
 		while (children != NULL)
@@ -984,7 +1007,7 @@ history_clear_recent_menu (void)
 			children = g_list_next (children);
 		}
 		
-		g_list_free(g_list_first(children));
+		g_list_free (tmplist);
 	}
 }
 
@@ -1162,7 +1185,7 @@ history_add_recent_menu (GaleonWindow *window, HistoryItem *hi)
 		else 
 			count++;
 	}
-        g_list_free(l);
+        g_list_free (l);
         menuitem = history_create_recent_menu_item (hi, 0, menu);
         gtk_menu_insert (GTK_MENU (menu), menuitem, go_recent_pos);
         gtk_widget_show (menuitem);
@@ -1176,17 +1199,32 @@ history_create_recent_menu_item (HistoryItem *hi, gint num, GtkWidget *menu)
 	GtkWidget *pixmap = gtk_pixmap_new (pixmap_data->pixmap,
 		pixmap_data->mask);
 	GtkWidget *menuitem = gtk_pixmap_menu_item_new ();
-	gchar *title_locale;
+	gchar *title_locale, *s, *tmp = NULL;
 	GtkWidget *l;
+	gint length;
 
-	title_locale = (hi->title_locale != NULL ? 
-			hi->title_locale : hi->title_utf8);
+	s = (hi->title_locale != NULL ? 
+	     hi->title_locale : hi->title_utf8);
+	length = strlen (s);
+	
+	/* shorten if needed */
+	if (length > 40)
+	{
+		tmp = g_strdup_printf ("%.29s...%s", s, s + length - 8);
+	}
+	
+	title_locale = escape_uline_accel (tmp ? tmp : s);
+	
 	l = new_num_accel_label (num, title_locale, FALSE, menu, menuitem);
+
+	if (tmp) g_free (tmp);
+	g_free (title_locale);
 	
 	gtk_box_pack_start (GTK_BOX (hb), l, FALSE, FALSE, 2);
 	gtk_container_add (GTK_CONTAINER(menuitem), hb);
 	gtk_pixmap_menu_item_set_pixmap (GTK_PIXMAP_MENU_ITEM(menuitem),
 		pixmap);
+	gtk_widget_lock_accelerators (menuitem);
 	
 	gtk_widget_show_all (hb);
 	if (gnome_preferences_get_menus_have_icons ())
@@ -1349,8 +1387,8 @@ tree_sort (GtkCTree *ctree, GtkCTreeNode *node, gpointer data)
 	clist = GTK_CLIST (ctree);
 
 	/* find out the sort column and ordering */
-	sort_column = gnome_config_get_int (CONF_HISTORY_SORT_COLUMN);
-	sort_order = gnome_config_get_int (CONF_HISTORY_SORT_ORDER);	
+	sort_column = eel_gconf_get_integer (CONF_HISTORY_SORT_COLUMN);
+	sort_order = eel_gconf_get_integer (CONF_HISTORY_SORT_ORDER);
 
 	/* find starting node */
 	if (node != NULL)
@@ -1449,7 +1487,7 @@ history_load (void)
 	history_hosts = g_hash_table_new (g_str_hash, g_str_equal);
 
 	/* find out how many days to keep before expiring elements */
-	expire_days = gnome_config_get_int (CONF_HISTORY_EXPIRE);
+	expire_days = eel_gconf_get_integer (CONF_HISTORY_EXPIRE);
 
 	/* load the file if we can access it */
 	if (access (histfile, F_OK) != -1)
@@ -1831,3 +1869,39 @@ history_drag_data_get_cb (GtkWidget *widget, GdkDragContext *context,
 		break;
 	}
 }
+
+static const struct 
+{
+	char *gconf_key;
+	GConfClientNotifyFunc func;
+}
+notifiers [] =
+{
+	{ CONF_HISTORY_SEARCH_TEXT, history_properties_notifier },
+	{ CONF_HISTORY_SEARCH_TIME, history_properties_notifier },
+	{ CONF_HISTORY_HOST_GROUP, history_properties_notifier },
+	{ CONF_HISTORY_SORT_COLUMN, history_sort_notifier },
+	{ CONF_HISTORY_SORT_ORDER, history_sort_notifier },
+	{ NULL, NULL}
+};
+
+static
+void history_notifiers_init(HistoryView *hv) 
+{
+	int i;
+
+	for (i = 0; notifiers[i].gconf_key != NULL; i++)
+	{
+		galeon_notification_add(
+			notifiers[i].gconf_key,
+			(GConfClientNotifyFunc)notifiers[i].func,
+			(gpointer)hv,
+			&history_notifiers_id);
+	}
+}
+
+
+void history_notifiers_remove(void) 
+{
+	galeon_notification_remove(&history_notifiers_id);
+}
