@@ -16,6 +16,8 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <gconf/gconf-client.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -44,6 +46,7 @@ void history_resort_view (HistoryView *hv);
 void history_refresh_view (HistoryView *hv);
 void history_destroy_view (HistoryView *hv);
 void history_clear_recent_menu (void);
+void history_notifiers_remove (void);
 
 /* history_callbacks.c */
 gint history_popup_selection_done_cb (GtkWidget *menushell,
@@ -56,6 +59,16 @@ gint history_compare_last_cb (gconstpointer a, gconstpointer b);
 gint history_compare_first_cb (gconstpointer a, gconstpointer b);
 gint history_compare_visits_cb (gconstpointer a, gconstpointer b);
 gboolean history_periodic_save_cb (gpointer data);
+void
+history_properties_notifier (GConfClient *client,
+			     guint cnxn_id,
+			     GConfEntry *entry,
+			     gpointer user_data);
+void
+history_sort_notifier (GConfClient *client,
+		       guint cnxn_id,
+		       GConfEntry *entry,
+		       gpointer user_data);
 
 #ifdef __cplusplus
 }
