@@ -23,6 +23,7 @@
 #include "history.h"
 #include "embed.h"
 #include "session.h"
+#include "mozilla_prefs.h"
 #include "misc.h"
 #include "bookmarks.h"
 #include "window.h"
@@ -32,8 +33,6 @@
 #include <glade/glade.h>
 
 /* GNOME includes */
-#include <gconf/gconf.h>
-#include <gconf/gconf-client.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-i18n.h>
@@ -45,6 +44,9 @@
 #ifdef ENABLE_APPLET
 #include <applet-widget.h>
 #endif
+#ifdef ENABLE_GNOME_FILE_SELECTOR
+#include <bonobo/bonobo-main.h>
+#endif /* !ENABLE_GNOME_FILE_SELECTOR */
 
 #include <errno.h>
 
@@ -68,7 +70,6 @@ Galeon_Browser impl_Galeon_Browser__create (PortableServer_POA poa,
 /* global variables */
 gboolean galeon_server_mode = FALSE;
 gboolean galeon_panel_mode = FALSE;
-GConfClient *gconf_client = NULL;
 
 /* file local variables */
 static CORBA_Environment corba_env;          /* corba environment for errors */
@@ -102,7 +103,7 @@ static struct poptOption popt_options[] =
 	  N_("Attempt to load URL in existing Galeon window"),
 	  NULL },
 #ifdef ENABLE_APPLET
-	{ "panel", 'p', POPT_ARG_NONE, &galeon_server_mode, 0,
+	{ "panel", 'p', POPT_ARG_NONE, &galeon_panel_mode, 0,
 	  N_("Open as a panel applet"),
 	  NULL },		  
 #endif
@@ -179,6 +180,13 @@ main (int argc, char *argv[])
 					    popt_options, 0, &context);
 	}
 
+	if (galeon_panel_mode && galeon_server_mode)
+	{
+		galeon_server_mode = FALSE;
+		g_message (_("Panel mode. "
+			     "Ignoring server mode option."));
+	}
+
 	/* load arguments that aren't regular options (urls to load) */
 	n_urls = translate_url_arguments (context, &url);
 
@@ -186,11 +194,14 @@ main (int argc, char *argv[])
 	orb = corba_init (&argc, argv);
 	
 	/* check if galeon is already running and use that instead */
-	if (new_view_on_running_shell (n_urls, url))
+	if (!galeon_panel_mode && !galeon_server_mode)
 	{
-		/* done */
-		return 0;
-	} 
+		if (new_view_on_running_shell (n_urls, url))
+		{
+			/* done */
+			return 0;
+		} 
+	}
 
 	if (close_option || quit_option)
 	{
@@ -215,7 +226,7 @@ main (int argc, char *argv[])
 	no_default = galeon_init (argc, argv);
 
 	/* if we have command-line supplied URLs */
-	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED) ? 1 : 0;
+	tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED) ? 1 : 0;
 	for (i = 0; i < n_urls; i++)
 	{
 		last_embed = embed_create_from_url (last_embed, url[i],
@@ -243,6 +254,9 @@ main (int argc, char *argv[])
 		gtk_main ();
 	}
 
+	eel_gconf_monitor_remove ("/apps/galeon");
+	eel_gconf_monitor_remove ("/apps/nautilus/preferences");
+
 	/* exit cleanly */
 	return 0;
 }
@@ -329,8 +343,8 @@ check_mozilla_version (void)
 
 		verMatchStr = g_strdup_printf ("%xv%x", MOZILLA_VERSION,
 						mozilla_version);
-		rememberStr = gnome_config_get_string 
-				("/galeon/General/moz_mismatch_remember=");
+		rememberStr = eel_gconf_get_string 
+				(CONF_MOZ_MISMATCH);
 
 		if (rememberStr != NULL)
 		{
@@ -339,8 +353,8 @@ check_mozilla_version (void)
 			if (ignore == 0)
 			{
 				g_free (verMatchStr);
-				return gnome_config_get_int
-				    ("/galeon/General/moz_mismatch_action=1");
+				return eel_gconf_get_integer
+				    (CONF_MOZ_MISMATCH_ACTION);
 			}
 		}
 
@@ -361,13 +375,12 @@ check_mozilla_version (void)
 		g_free (questionStr);
 		if (rememberv)
 		{
-			gnome_config_set_string
-				("/galeon/General/moz_mismatch_remember",
+			eel_gconf_set_string
+				(CONF_MOZ_MISMATCH,
 				verMatchStr);
-			gnome_config_set_int
-				("/galeon/General/moz_mismatch_action",
+			eel_gconf_set_integer
+				(CONF_MOZ_MISMATCH_ACTION,
 				 !continuev);
-			gnome_config_sync ();
 		}
 		g_free (verMatchStr);
 		return !continuev;
@@ -444,6 +457,12 @@ corba_init (int *argc, char *argv[])
 	/* initialise OAF */
 	orb = oaf_init (*argc, argv);
 
+#ifdef ENABLE_GNOME_FILE_SELECTOR
+        if ( !bonobo_init (orb, NULL, NULL)) {
+                g_warning ("Cannot initialize Bonobo component framework !");
+        }
+#endif /* !ENABLE_GNOME_FILE_SELECTOR */
+        
 	/* return handle */
 	return orb;
 }
@@ -553,7 +572,7 @@ new_shell (CORBA_ORB orb)
 	}
 
 	/* register server with oaf */
-	reg_id = oaf_make_registration_id ("OAFIID:GNOME:galeon:20010802",
+	reg_id = oaf_make_registration_id ("OAFIID:GNOME_Galeon_Automation",
 					   g_getenv ("DISPLAY"));
 	oaf_active_server_register (reg_id, client);
 	g_free (reg_id);
@@ -574,16 +593,14 @@ galeon_init (int argc, char *argv[])
 	extern gboolean pushed_startup;
 	gboolean moz_version_checked = FALSE;
 	GdkAtom message_type;
+	gchar *teststr;
+	gchar *mozprefs;
+	gboolean set_mozilla_prefs = FALSE;
 
 	/* initialise GNOME VFS */
 	vfs_success = gnome_vfs_init ();
 	g_assert (vfs_success);
 
-	/* initialise gconf */
-	gconf_init (argc, argv, NULL);
-	gconf_client = gconf_client_get_default ();
-	g_assert (gconf_client != NULL);
-
 	/* initialise GLADE */
 	glade_gnome_init ();
 
@@ -597,6 +614,29 @@ galeon_init (int argc, char *argv[])
 		moz_version_checked = TRUE;
 	}
 
+	/* preload the prefs */
+	/* it also enables notifiers support */
+	eel_gconf_monitor_add ("/apps/galeon");
+	eel_gconf_monitor_add ("/apps/nautilus/preferences");
+
+	/* check if gconf schemas are working */
+	teststr = gconf_client_get_string (eel_gconf_client_get_global(),
+					  "/apps/galeon/gconf_test", NULL);
+	if (!teststr)
+	{
+		GtkWidget *dialog;
+		dialog = gnome_error_dialog ("Cannot find a schema for galeon preferences. \nCheck your gconf setup, look at galeon FAQ for more info");
+		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+		exit (0);
+	}
+	else
+	{
+		g_free (teststr);
+	}
+
+	/* check if this is the first time this user has run Galeon */
+	new_user = newuser_check ();
+
 	/* initialise Galeon itself */
 	initialise_colours ();
 	preferences_load ();
@@ -606,8 +646,18 @@ galeon_init (int argc, char *argv[])
 	/* init mozilla home */
 	gtk_moz_embed_set_comp_path (g_getenv ("MOZILLA_FIVE_HOME"));
 
-	/* set the profile dir for gtkmozembed */
+	/* set a path for the profile */
 	profile = g_concat_dir_and_file (g_get_home_dir (), ".galeon/mozilla");
+
+	/* no mozilla prefs ? */
+	mozprefs = g_concat_dir_and_file (profile, "/galeon/prefs.js"); 
+	if (!g_file_exists(mozprefs))
+	{
+		set_mozilla_prefs = TRUE;
+	}
+	g_free (mozprefs);
+
+	/* initialize profile */
 	gtk_moz_embed_set_profile_path (profile, "galeon");
 	g_free (profile);
 
@@ -616,6 +666,13 @@ galeon_init (int argc, char *argv[])
 	{
 		embed_startup_init ();
 	}
+
+	/* set mozilla prefs if necessary */
+	if (set_mozilla_prefs)
+	{
+		mozilla_prefs_set ();
+	}
+
 	/* if we couldn't check the first time, try now since initializing moz
 	 * will set the MOZILLA_FIVE_HOME in some configurations */
 	if (!moz_version_checked)
@@ -626,9 +683,6 @@ galeon_init (int argc, char *argv[])
 		}
 	}
 
-	/* check if this is the first time this user has run Galeon */
-	new_user = newuser_check ();
-
 	/* resume a crashed session, if any */
 	session_recovery = session_autoresume ();
 
@@ -651,8 +705,7 @@ galeon_init (int argc, char *argv[])
 	gdk_error_trap_push ();
 
 	/* if this flag doesn't get cleared, then Galeon crashed */
-	gnome_config_set_int ("/galeon/General/crashed", TRUE);
-	gnome_config_sync ();
+	eel_gconf_set_integer (CONF_CRASH_CRASHED, TRUE);
 
 	/* return saying whether or not to override default window creation */
 	return (new_user || session_recovery);
@@ -680,9 +733,6 @@ create_default_browser (void)
 	/* create an empty window */
 	window = window_create (GTK_MOZ_EMBED_FLAG_ALLCHROME);
 	
-	/* make sure the user can see it */
-	set_settings_menu_window (window);
-	
 	/* show as fullscreen? */
 	/* THE WINDOW NEEDS TO BE VISIBLE HERE */
 	if (open_fullscreen)
@@ -750,7 +800,7 @@ theme_changed_cb (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 	/* set in all tabs */
 	for (e = all_embeds; e != NULL; e = g_list_next (e))
 	{
-		embed_set_notebook_label_status ((GaleonEmbed *)(e->data));
+		embed_update_tab_status ((GaleonEmbed *)(e->data));
 	}
 	
 	/* process other GDK events */
