@@ -31,6 +31,7 @@
 #include "xlink.h"
 #include "dialog.h"
 #include "filepicker.h"
+#include "mozilla.h"
 
 #include <string.h>
 #include <gtkmozembed.h>
@@ -63,7 +64,7 @@ window_menu_bookmarks_edit_activate_cb (GtkMenuItem *menuitem,
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
 
-	if (gnome_config_get_bool (CONF_STATE_BOOKMARKS_DOCK_ENABLE))
+	if (eel_gconf_get_boolean (CONF_STATE_BOOKMARKS_DOCK_ENABLE))
 	{
 		if (window->dock_type != DOCK_BOOKMARKS)
 		{
@@ -278,7 +279,7 @@ window_menu_view_source_activate_cb (GtkMenuItem *menuitem,
 
 	return_if_not_window (window);
 
-	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
+	tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED);
 	embed_view_source (window->active_embed, TRUE, !tabbed_mode);
 }
 
@@ -289,8 +290,8 @@ void window_menu_history_activate_cb (GtkMenuItem *menuitem,
 				      GaleonWindow *window)
 {
 	return_if_not_window (window);
-	
-	if (gnome_config_get_bool (CONF_STATE_HISTORY_DOCK_ENABLE))
+
+	if (eel_gconf_get_boolean (CONF_STATE_HISTORY_DOCK_ENABLE))
 	{
 		if (window->dock_type != DOCK_HISTORY)
 		{
@@ -503,9 +504,10 @@ window_menu_exit_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 	   to the main loop first.  This is because we might have been
 	   activated from within the mozembed_key_press callback, and if the
 	   widget is destroyed before returning from the callback, very bad
-	   things happen.  --Josh */
-	gtk_timeout_add (50, (GtkFunction) session_quit_cb,
-			 GINT_TO_POINTER (FALSE));
+	   things happen.  --Josh 
+	   Lets take care the timer is not triggered again - yaneti */
+	quit_timeout_id = gtk_timeout_add (50, (GtkFunction) session_quit_cb,
+					   GINT_TO_POINTER (FALSE));
 }
 
 /** 
@@ -516,7 +518,8 @@ window_menu_exit_with_session_activate_cb (GtkMenuItem *menuitem,
 					   GaleonWindow *window)
 {
 	return_if_not_window (window);
-	session_quit (TRUE);
+	quit_timeout_id = gtk_timeout_add (50, (GtkFunction) session_quit_cb,
+					   GINT_TO_POINTER (TRUE));
 }
 
 /** 
@@ -538,7 +541,9 @@ window_menu_close_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	embed_close (window->active_embed);
+	window->active_embed->close_timeout_id = 
+		gtk_timeout_add (50, (GtkFunction) embed_close,
+				 (gpointer)window->active_embed);
 }
 
 /** 
@@ -697,6 +702,21 @@ void window_menu_toolbar_activate_cb (GtkCheckMenuItem *menuitem,
 	}
 }
 
+/** 
+ * window_menu_bookmarks_toolbars_activate_cb: hide/show the bookmarks toolbars
+ */
+void window_menu_bookmarks_toolbars_activate_cb (GtkCheckMenuItem *menuitem,
+						 GaleonWindow *window)
+{
+	return_if_not_window (window);
+
+	bookmarks_toolbars_set_visibility (window, menuitem->active);
+	if (window->hbox)
+	{
+		gtk_widget_queue_resize (GTK_WIDGET (window->hbox));
+	}
+}
+
 /** 
  * window_menu_menubar_activate_cb: hide/show the menubar
  */
@@ -759,7 +779,8 @@ window_menu_own_colors_activate_cb (GtkCheckMenuItem *menuitem,
 				    GaleonWindow *window)
 {
 	return_if_not_window (window);
-	preferences_set_use_own_colors (menuitem->active);
+	eel_gconf_set_boolean(CONF_RENDERING_USE_OWN_COLORS, 
+			      menuitem->active);
 }
 
 void
@@ -767,7 +788,8 @@ window_menu_enable_javascript_activate_cb (GtkCheckMenuItem *menuitem,
 					   GaleonWindow *window)
 {
 	return_if_not_window (window);
-	preferences_set_enable_javascript (menuitem->active);
+	eel_gconf_set_boolean(CONF_FILTERING_JAVASCRIPT_ENABLED, 
+			      menuitem->active);
 }
 
 void
@@ -775,7 +797,7 @@ window_menu_enable_proxy_activate_cb (GtkCheckMenuItem *menuitem,
                                            GaleonWindow *window)
 {
         return_if_not_window (window);
-        preferences_set_enable_proxy (menuitem->active);
+	eel_gconf_set_boolean(CONF_NETWORK_PROXY_ENABLED, menuitem->active);
 }
 
 
@@ -784,7 +806,8 @@ window_menu_autojump_activate_cb (GtkCheckMenuItem *menuitem,
 					   GaleonWindow *window)
 {
 	return_if_not_window (window);
-	preferences_set_autojump(menuitem->active);
+	eel_gconf_set_boolean(CONF_TABS_TABBED_AUTOJUMP, 
+			      menuitem->active);
 }
 
 void
@@ -792,7 +815,7 @@ window_menu_own_fonts_activate_cb (GtkCheckMenuItem *menuitem,
 				   GaleonWindow *window)
 {
 	return_if_not_window (window);
-	preferences_set_use_own_fonts (menuitem->active);
+	eel_gconf_set_boolean(CONF_RENDERING_USE_OWN_FONTS, menuitem->active);
 }
 
 void
@@ -800,7 +823,7 @@ window_menu_enable_java_activate_cb (GtkCheckMenuItem *menuitem,
 				     GaleonWindow *window)
 {
 	return_if_not_window (window);
-	preferences_set_enable_java (menuitem->active);
+	eel_gconf_set_boolean(CONF_FILTERING_JAVA_ENABLED, menuitem->active);
 }
 
 void
@@ -810,7 +833,8 @@ window_menu_images_never_activate_cb (GtkCheckMenuItem *menuitem,
 	return_if_not_window (window);
 	if (menuitem->active) 
 	{
-		preferences_set_image_loading (IMAGE_LOADING_NEVER);
+		eel_gconf_set_integer(CONF_FILTERING_IMAGE_LOADING_TYPE,
+				      IMAGE_LOADING_NEVER);
 	}
 }
 
@@ -821,8 +845,8 @@ window_menu_images_current_server_activate_cb (GtkCheckMenuItem *menuitem,
 	return_if_not_window (window);
 	if (menuitem->active) 
 	{
-		preferences_set_image_loading
-			(IMAGE_LOADING_FROM_CURRENT_SERVER_ONLY);
+		eel_gconf_set_integer(CONF_FILTERING_IMAGE_LOADING_TYPE,
+				      IMAGE_LOADING_FROM_CURRENT_SERVER_ONLY);
 	}
 }
 
@@ -833,7 +857,8 @@ window_menu_images_always_activate_cb (GtkCheckMenuItem *menuitem,
 	return_if_not_window (window);
 	if (menuitem->active) 
 	{
-		preferences_set_image_loading (IMAGE_LOADING_ALWAYS);
+		eel_gconf_set_integer(CONF_FILTERING_IMAGE_LOADING_TYPE, 
+				      IMAGE_LOADING_ALWAYS);
 	}
 }
 
@@ -872,7 +897,8 @@ window_menu_animate_always_activate_cb (GtkCheckMenuItem *menuitem,
 	return_if_not_window (window);
 	if (menuitem->active) 
 	{
-		preferences_set_animate (ANIMATE_ALWAYS);
+		eel_gconf_set_integer(CONF_FILTERING_ANIMATE_TYPE, 
+				      ANIMATE_ALWAYS);
 	}
 }
 
@@ -883,7 +909,8 @@ window_menu_animate_once_through_activate_cb (GtkCheckMenuItem *menuitem,
 	return_if_not_window (window);
 	if (menuitem->active) 
 	{
-		preferences_set_animate (ANIMATE_ONCE_THROUGH);
+		eel_gconf_set_integer(CONF_FILTERING_ANIMATE_TYPE,
+				      ANIMATE_ONCE_THROUGH);
 	}
 }
 
@@ -894,7 +921,8 @@ window_menu_animate_never_activate_cb (GtkCheckMenuItem *menuitem,
 	return_if_not_window (window);
 	if (menuitem->active) 
 	{
-		preferences_set_animate (ANIMATE_NEVER);
+		eel_gconf_set_integer(CONF_FILTERING_ANIMATE_TYPE,
+				      ANIMATE_NEVER);
 	}
 }
 
@@ -938,26 +966,72 @@ void
 window_menu_zoom_in_activate_cb (GtkCheckMenuItem *menuitem,
 				 GaleonWindow *window)
 {
+	GaleonEmbed *embed;
+
 	return_if_not_window (window);
-	gtk_spin_button_spin (GTK_SPIN_BUTTON (window->zoom_spin),
-			      GTK_SPIN_STEP_FORWARD, 10); 
+	embed = window->active_embed;
+	return_if_not_embed (embed);
+
+	if (window->zoom_spin)
+	{
+		gtk_spin_button_spin (GTK_SPIN_BUTTON (window->zoom_spin),
+				      GTK_SPIN_STEP_FORWARD, 10); 
+	}
+	else
+	{
+		float zoom;
+		mozilla_get_zoom (embed, &zoom);
+		zoom = zoom*100+10;
+		embed_set_zoom (embed, zoom);
+		history_set_zoom (embed->site_location, zoom);
+	}
 }
 
 void
 window_menu_zoom_100_activate_cb (GtkCheckMenuItem *menuitem,
 				 GaleonWindow *window)
 {
+	GaleonEmbed *embed;
+
 	return_if_not_window (window);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (window->zoom_spin), 100);
+	embed = window->active_embed;
+	return_if_not_embed (embed);
+
+	if (window->zoom_spin)
+	{
+		gtk_spin_button_set_value 
+			(GTK_SPIN_BUTTON (window->zoom_spin), 100);
+	}
+	else
+	{
+		embed_set_zoom (embed, 100);
+		history_set_zoom (embed->site_location, 100);
+	}
 }
 
 void
 window_menu_zoom_out_activate_cb (GtkCheckMenuItem *menuitem,
 				  GaleonWindow *window)
 {
+	GaleonEmbed *embed;
+
 	return_if_not_window (window);
-	gtk_spin_button_spin (GTK_SPIN_BUTTON (window->zoom_spin),
-			      GTK_SPIN_STEP_BACKWARD, 10); 
+	embed = window->active_embed;
+	return_if_not_embed (embed);
+
+	if (window->zoom_spin)
+	{
+		gtk_spin_button_spin (GTK_SPIN_BUTTON (window->zoom_spin),
+				      GTK_SPIN_STEP_BACKWARD, 10); 
+	}
+	else
+	{
+		float zoom;
+		mozilla_get_zoom (embed, &zoom);
+		zoom = zoom*100-10;
+		embed_set_zoom (embed, zoom);
+		history_set_zoom (embed->site_location, zoom);
+	}
 }
 
 void 
@@ -1044,7 +1118,7 @@ window_menu_save_session_as_activate_cb (GtkWidget *menuitem,
 						     "/.galeon/sessions");
 	if (show_file_picker (window->WMain, _("Save session as..."),
 			      sessions_dir, NULL, modeSave,
-			      &file))
+			      &file, NULL))
 	{
 		if (dialog_check_overwrite_file (file, NULL)) 
 		{
@@ -1066,7 +1140,7 @@ window_menu_open_session_activate_cb (GtkWidget *menuitem,
 						     "/.galeon/sessions");
 	if (show_file_picker (window->WMain, _("Open session..."),
 			      sessions_dir, NULL, modeOpen,
-			      &file)) 
+			      &file, NULL)) 
 	{
 		session_load_from (file);
 		session_history_add (file);
@@ -1075,3 +1149,17 @@ window_menu_open_session_activate_cb (GtkWidget *menuitem,
 	g_free (file);
 	g_free (sessions_dir);
 }
+
+void
+window_menu_show_java_console_cb (GtkWidget *meniutem,
+				  GaleonWindow *window)
+{
+	mozilla_show_java_console ();
+}
+
+void
+window_menu_show_javascript_console_cb (GtkWidget *meniutem,
+					GaleonWindow *window)
+{
+	mozilla_show_javascript_console ();
+}
