@@ -54,7 +54,7 @@ GnomeUIInfo file_menu_uiinfo[] =
 	{ GNOME_APP_UI_ITEM, N_("New _tab"), NULL,
 	  window_menu_new_tab_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW,
-	  GDK_b, GDK_CONTROL_MASK, NULL },
+	  GDK_T, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_MENU_OPEN_ITEM (window_menu_open_activate_cb, NULL),
 	{ GNOME_APP_UI_ITEM, N_("Open _URL..."), N_("Open a new URL"),
 	  window_menu_open_url_activate_cb, NULL, NULL,
@@ -79,7 +79,7 @@ GnomeUIInfo file_menu_uiinfo[] =
 	  N_("Save the location of each open window to a file"),
 	  window_menu_save_session_as_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Open session..."),
+	{ GNOME_APP_UI_ITEM, N_("Op_en session..."),
 	  N_("Restore a previous browsing session from a file"),
 	  window_menu_open_session_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
@@ -135,11 +135,15 @@ GnomeUIInfo view_menu_uiinfo[] =
 	{ GNOME_APP_UI_TOGGLEITEM, N_("_Menubar"), NULL,
 	  window_menu_menubar_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_M, GDK_CONTROL_MASK, NULL },
+	  GDK_M, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
 	{ GNOME_APP_UI_TOGGLEITEM, N_("_Toolbar"), NULL,
 	  window_menu_toolbar_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
-	  GDK_T, GDK_CONTROL_MASK, NULL },
+	  GDK_T, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
+	{ GNOME_APP_UI_TOGGLEITEM, N_("_Bookmarks toolbars"), NULL,
+	  window_menu_bookmarks_toolbars_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  GDK_B, GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
 	{ GNOME_APP_UI_TOGGLEITEM, N_("_Statusbar"), NULL,
 	  window_menu_statusbar_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
@@ -171,6 +175,10 @@ GnomeUIInfo tab_menu_uiinfo[] =
 	  window_menu_detach_tab_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_I, GDK_CONTROL_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Move tab to another window"), NULL,
+	  NULL, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	{ GNOME_APP_UI_ITEM, N_("_Previous tab"), NULL,
 	  window_menu_prev_tab_activate_cb, NULL, NULL,
@@ -184,11 +192,11 @@ GnomeUIInfo tab_menu_uiinfo[] =
 	{ GNOME_APP_UI_ITEM, N_("Move tab _left / up"), NULL,
 	  window_menu_shift_tab_left_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BACK, GDK_less,
-	  GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
+	  GDK_CONTROL_MASK, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Move tab _right / down"), NULL,
 	  window_menu_shift_tab_right_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_FORWARD, GDK_greater,
-	  GDK_CONTROL_MASK | GDK_SHIFT_MASK, NULL },
+	  GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_END
 };
 
@@ -322,7 +330,7 @@ GnomeUIInfo settings_menu_uiinfo[] =
           window_menu_enable_proxy_activate_cb, NULL, NULL,
           GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
-	{ GNOME_APP_UI_TOGGLEITEM, N_("Auto-jump to new tabs"), NULL,
+	{ GNOME_APP_UI_TOGGLEITEM, N_("Auto-jump to new _tabs"), NULL,
 	  window_menu_autojump_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
           0, 0, NULL },
@@ -333,18 +341,18 @@ GnomeUIInfo settings_menu_uiinfo[] =
 
 GnomeUIInfo bookmarks_menu_uiinfo[] =
 {
-	{ GNOME_APP_UI_ITEM, N_("_Add bookmark to"), NULL,
-	  NULL, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_INDEX,
-	  0, 0, NULL },
-	{ GNOME_APP_UI_ITEM, N_("_Edit..."), NULL,
-	  window_menu_bookmarks_edit_activate_cb, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN,
-	  GDK_E, GDK_CONTROL_MASK, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Add _bookmark"), NULL,
 	  window_menu_add_bookmark_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_D, GDK_CONTROL_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Add bookmark to"), NULL,
+	  NULL, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_INDEX,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Edit bookmarks..."), NULL,
+	  window_menu_bookmarks_edit_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN,
+	  GDK_B, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_END
 };
@@ -396,6 +404,17 @@ GnomeUIInfo tools_menu_uiinfo[] =
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_H, GDK_CONTROL_MASK, NULL },
 	GNOMEUIINFO_SEPARATOR, 
+	{ GNOME_APP_UI_ITEM, N_("_Java Console..."),
+	  N_("Show the Java Console"),
+	  window_menu_show_java_console_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("_Javascript Console..."),
+	  N_("Show the Javascript Console"),
+	  window_menu_show_javascript_console_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_NONE, NULL,
+	  0, 0, NULL },
+	GNOMEUIINFO_SEPARATOR, 
 	{ GNOME_APP_UI_ITEM, N_("_Rerun first time druid..."),
 	  N_("Lets run the first time druid again"),
 	  window_run_first_time_druid_activate_cb, NULL, NULL,
