@@ -33,6 +33,7 @@
 #include <dirent.h>
 #include <limits.h>
 
+#include <time.h>
 #include <stdlib.h>
 #include <string.h>
 #include <iconv.h>
@@ -51,16 +52,16 @@
 #include <libgnome/gnome-exec.h>
 #include <libgnome/gnome-config.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include "gtkmozembed.h"
 
 /* GNOME includes */
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-handle.h>
 
 /** Styles for tab labels */
-GtkStyle *active_loading_text_style;
-GtkStyle *inactive_loading_text_style;
-GtkStyle *inactive_new_text_style;
-GtkStyle *inactive_text_style;
+GtkStyle *loading_text_style = NULL;
+GtkStyle *new_text_style = NULL;
 
 /* Local prototypes */
 static gboolean is_punctuation (gchar c);
@@ -356,7 +357,7 @@ shorten_name(const gchar *input_name, gint target_length)
 	length = strlen (name);
 
 	/* look for some common prefixes -- should these be translated? */
-	if (gnome_config_get_bool (CONF_APPEARANCE_TABBED_PREFIX))
+	if (eel_gconf_get_boolean (CONF_TABS_TABBED_PREFIX))
 	{
 		/* prefixes that only come by themselves */
 		if (strncasecmp (name, "index of ", 9) == 0)
@@ -421,7 +422,7 @@ shorten_name(const gchar *input_name, gint target_length)
 
 	/* find in name the first occurence of one of
 	 * several common separators, and set it to '\0' */
-	if (gnome_config_get_bool (CONF_APPEARANCE_TABBED_SEPARATOR))
+	if (eel_gconf_get_boolean (CONF_TABS_TABBED_SEPARATOR))
 	{
 		gchar *first;
 		gchar *str;
@@ -460,7 +461,7 @@ shorten_name(const gchar *input_name, gint target_length)
 	/* getting tricky, try losing a few vowels */
 	/* yes, this is very anglocentric -- I don't know of any strategies
 	 * for other languages (particularly non-European) -- MattA */
-	if (gnome_config_get_bool (CONF_APPEARANCE_TABBED_VOWEL))
+	if (eel_gconf_get_boolean (CONF_TABS_TABBED_VOWEL))
 	{
 		for (i = length - 1, j = length - 1; i >= 0; i--)
 		{
@@ -520,18 +521,18 @@ shorten_name(const gchar *input_name, gint target_length)
 }
 
 /**
- * initialise_colours: initialise global colour styles
+ * initialise_colours: initialise (or update) global colour styles
  */
 void
 initialise_colours (void)
 {
 	GtkWidget *label;
 	GtkStyle  *rcstyle;
+	char *tmp;
 
 	/* create a dummy label so we can fetch the associated rc style */
 	label = gtk_label_new ("");
 	rcstyle = gtk_rc_get_style (label);
-	gtk_widget_destroy (label);
 
 	/* this is needed for some (broken?) themes */
 	if (rcstyle == NULL)
@@ -542,25 +543,26 @@ initialise_colours (void)
 	{
 		gtk_style_ref (rcstyle);
 	}
-		
+
+	/* free previous styles (if any) */
+	if (loading_text_style) gtk_style_unref (loading_text_style);
+	if (new_text_style) gtk_style_unref (new_text_style);
+	
 	/* make styles */
-	active_loading_text_style = gtk_style_copy (rcstyle);
-	inactive_loading_text_style = gtk_style_copy (rcstyle);
-	inactive_new_text_style = gtk_style_copy (rcstyle);
-	inactive_text_style = gtk_style_copy (rcstyle);
+	loading_text_style = gtk_style_copy (rcstyle);
+	new_text_style = gtk_style_copy (rcstyle);
 
 	/* load colours */
-	gdk_color_parse ("#ff0000", &(active_loading_text_style->fg[0]));
+	tmp = eel_gconf_get_string (CONF_TABS_TABBED_LOADING_COLOR);
+	gdk_color_parse (tmp, &(loading_text_style->fg[0]));
+	g_free (tmp);
 
-	inactive_loading_text_style->fg[0] = rcstyle->fg[4];
-	inactive_loading_text_style->fg[0].red = 65535;
-
-	inactive_new_text_style->fg[0] = rcstyle->fg[4];
-	inactive_new_text_style->fg[0].blue = 65535;
-
-	inactive_text_style->fg[0] = rcstyle->fg[4];
+	tmp = eel_gconf_get_string (CONF_TABS_TABBED_NEW_COLOR);
+	gdk_color_parse (tmp, &(new_text_style->fg[0]));
+	g_free (tmp);
 
 	/* free */
+	gtk_widget_destroy (label);
 	gtk_style_unref (rcstyle);
 }
 
@@ -676,16 +678,14 @@ launch_external_viewer (const gchar *filename)
 	mozilla_delete_temp_file_on_exit (filename);
 
 	/* get the program to execute */
-	prog = gnome_config_get_string (CONF_HANDLERS_EXTERNAL_SOURCE_VIEWER);
+	prog = eel_gconf_get_string (CONF_PROGRAMS_EXTERNAL_SOURCE_VIEWER);
+	if (!prog) return;
+
 	command = g_strconcat (prog," ", filename, NULL);
 
 	/* execute the viewer */
 	pid = gnome_execute_shell (NULL, command);
 
-	/* FIXME: need to detect child termination and remove the temporary
-	 * file -- this used to be done with "sleep", but that's a horrible
-	 * solution -- MattA */
-
 	/* free allocated string */
 	g_free (command);
 	g_free (prog);
@@ -1103,7 +1103,7 @@ pixbuf_threshold_alpha (GdkPixbuf *pixbuf)
 	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 	
 	/* get target colour to blend with based on style background */
-	bg = active_loading_text_style->bg[0];
+	bg = loading_text_style->bg[0];
 	tr = bg.red   >> 8;
 	tg = bg.green >> 8;
 	tb = bg.blue  >> 8;
@@ -1125,7 +1125,7 @@ pixbuf_threshold_alpha (GdkPixbuf *pixbuf)
 			b = ((a * b) + ((255 ^ a) * tb)) >> 8;
 
 			/* threshold alpha */
-			if (a > 16)
+			if (a > 12)
 			{
 				a = 255;
 			}
@@ -1224,12 +1224,11 @@ xmlGetIntProp (xmlNodePtr node, const gchar *attribute)
 }
 
 gchar *
-misc_parse_nautilus_uri (const gchar *uri)
+misc_parse_uri (const gchar *uri)
 {
 	const gchar *filename;
 	const gchar *mime_type;
-	xmlDocPtr doc;
-	gchar *url;
+	
 
 	/* check it's a file uri */
 	if (strncmp (uri, "file:", 5) != 0)
@@ -1246,39 +1245,95 @@ misc_parse_nautilus_uri (const gchar *uri)
 
 	/* get the mime type and check its a nautilus link */
 	mime_type = gnome_vfs_get_file_mime_type (filename, NULL, FALSE);
-	if (strcmp (mime_type, "application/x-nautilus-link") != 0)
-	{
-		return NULL;
-	}
-
-	/* attempt to parse it */
-	doc = xmlParseFile (filename);
-
-	/* check the document */
-	if (doc == NULL)
-	{
-		return NULL;
-	}
-
-	/* check the structure */
-	if (doc->root == NULL || doc->root->name == NULL ||
-	    g_strcasecmp (doc->root->name, "nautilus_object") != 0)
+	/* old style */
+	if (strcmp (mime_type, "application/x-nautilus-link") == 0)
 	{
+		xmlDocPtr doc;
+		gchar *url;
+		
+		/* attempt to parse it */
+		doc = xmlParseFile (filename);
+		
+		/* check the document */
+		if (doc == NULL)
+		{
+			return NULL;
+		}
+		
+		/* check the structure */
+		if (doc->root == NULL || doc->root->name == NULL ||
+		    g_strcasecmp (doc->root->name, "nautilus_object") != 0)
+		{
+			xmlFreeDoc (doc);
+			return NULL;
+		}
+		
+		/* get the url */
+		url = xmlGetProp (doc->root, "link");
+		if (url == NULL || strlen (url) == 0)
+		{
+			xmlFreeDoc (doc);
+			return NULL;
+		}
+		
+		/* return the URL */
 		xmlFreeDoc (doc);
-		return NULL;
+		return url;
 	}
-
-	/* get the url */
-	url = xmlGetProp (doc->root, "link");
-	if (url == NULL || strlen (url) == 0)
+	/* new style */
+	if (strcmp (mime_type, "application/x-gnome-app-info") == 0)
 	{
-		xmlFreeDoc (doc);
-		return NULL;
-	}
+		/* following is a very hacky way to extracting the URL
+		   argument from a .desktop file, if you know a better way 
+		   please let me know - mattsm */
+		/* begin hacky code */
+		gchar buf[81];
+		gchar *data;
+		gchar *url = NULL;
+		gchar **array;
+		guint i;
+		GString *file;
+		GnomeVFSHandle *handle;
+		GnomeVFSFileSize size;
 
-	/* return the URL */
-	xmlFreeDoc (doc);
-	return url;
+		buf[80] = '\0';
+		file = g_string_new ("");
+		if ( gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ) 
+		     == GNOME_VFS_OK )
+		{
+			while (gnome_vfs_read (handle, &buf, 80, &size) != 
+			       GNOME_VFS_ERROR_EOF)
+			{
+				file = g_string_append (file, buf);
+
+				/* reset buf */
+				for (i = 0; buf[i] != '\0'; i++)
+					buf[i] = '\0';
+			}
+			gnome_vfs_close (handle);
+			gnome_vfs_handle_destroy (handle);
+			data = file->str;
+			g_string_free (file, FALSE);
+
+			array = g_strsplit ( data, "\n", 0);
+			
+			for ( i = 0; array[i] != NULL; i++)
+			{
+				if (g_strncasecmp (array[i], 
+						   "URL=", 4) == 0)
+				{
+					url = g_strdup (array[i] + 4);
+				}
+			}
+
+			g_free (data);
+			g_strfreev (array);
+
+			return url;			
+		}
+		/* end hacky code */
+	}
+	return NULL;
 }
 
 /**
@@ -1627,4 +1682,36 @@ gtk_ctree_unlink (GtkCTree     *ctree,
     }
 }
 
+void galeon_exit (void)
+{
+	extern GList *mozilla_notifiers;
+
+	eel_gconf_set_integer (CONF_CRASH_CRASHED, FALSE);
+
+	/* FIXME: uncatch GDK errors which happen on theme changes */
+	gdk_error_trap_pop ();
+
+	/* shut down galeon subsystems */
+	preferences_save ();
+	mime_db_shutdown ();
+	bookmarks_exit ();
+	history_exit ();
+	mozilla_save_prefs ();
+
+	/* check... */
+	g_assert (g_list_length (all_embeds) == 0);
+	g_assert (g_list_length (all_windows) == 0);
+
+	/* shut down GNOME subsystems */
+	gnome_vfs_shutdown ();
+
+	/* remove notifiers for mozilla prefs */
+	galeon_notification_remove (&mozilla_notifiers);
+
+	/* absolutely no more mozembeds */
+	gtk_moz_embed_pop_startup ();
+
+	gtk_main_quit();
+}
+
 /*************************************************************************/
