@@ -82,7 +82,7 @@ void gtk_ctree_link (GtkCTree *ctree, GtkCTreeNode *node, GtkCTreeNode *parent,
 void gtk_ctree_unlink (GtkCTree *ctree, GtkCTreeNode *node,
 		       gboolean update_focus_row);
 gchar *new_num_accel_str (gint num, gchar *text, gboolean lettersok);
-gchar *misc_parse_nautilus_uri (const gchar *uri);
+gchar *misc_parse_uri (const gchar *uri);
 
 /* misc_callbacks.c */
 void mozembed_destroy_cb (GtkObject *object, GaleonEmbed *embed);
@@ -108,6 +108,7 @@ void context_main_toolbar_icons_menu_cb (GtkRadioMenuItem *menuitem,
 					 gpointer user_data);
 void context_main_toolbar_edit_toolbar_cb (GtkMenuItem *menuitem, 
 					   GaleonWindow *window);
+void galeon_exit (void);
 #ifdef __cplusplus
 }
 #endif
