@@ -27,6 +27,7 @@
 #include "session.h"
 #include "mozilla.h"
 #include "context.h"
+#include "prefs.h"
 #include "downloader.h"
 
 #include <time.h>
@@ -73,7 +74,7 @@ mozembed_new_window_cb (GtkMozEmbed *dummy, GtkMozEmbed **retval,
 	g_assert (retval != NULL);
 
 	/* check config */
-	open_in_tab = gnome_config_get_bool (CONF_APPEARANCE_TABBED_POPUPS);
+	open_in_tab = eel_gconf_get_boolean (CONF_TABS_TABBED_POPUPS);
 
 	/* override if it's some sort of XUL (must be some sort of dialog) */
 	if ((chrome_mask & GTK_MOZ_EMBED_FLAG_OPENASCHROME) != 0)
@@ -228,7 +229,7 @@ mozembed_load_started_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	if (embed->load_started == 1)
 	{
 		spinner_start (window);
-		embed_set_notebook_label_status (embed);
+		embed_update_tab_status (embed);
 		embed_progress_clear (embed);
 		embed->when_started = time (NULL);
 	}
@@ -250,7 +251,6 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 {
 	GaleonWindow *window;
 	gint zoom;
-	extern gboolean is_newuser;
 
 	/* check callback args */
 	return_if_not_embed (embed);
@@ -264,16 +264,22 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	/* stop the window spinner */
 	if (embed->load_started == 0)
 	{
-		/* check zoom setting for the new site */
-		zoom = history_get_zoom (embed->site_location);
-		if (zoom == 0 && embed->zoom_auto_set)
+		/* check zoom setting for the new site, unless it has 
+		   a javascript URI */
+		if (eel_gconf_get_boolean (CONF_HISTORY_AUTOZOOM) &&
+		    embed->site_location 
+		    && strncmp ("javascript:", embed->site_location, 11))
 		{
-			embed_set_zoom (embed, 100);
-		}
-		else if (zoom != 0 && zoom != embed->zoom)
-		{
-			embed_set_zoom (embed, zoom);
-			embed->zoom_auto_set = TRUE;
+			zoom = history_get_zoom (embed->site_location);
+			if (zoom == 0 && embed->zoom_auto_set)
+			{
+				embed_set_zoom (embed, 100);
+			}
+			else if (zoom != 0 && zoom != embed->zoom)
+			{
+				embed_set_zoom (embed, zoom);
+				embed->zoom_auto_set = TRUE;
+			}
 		}
 
 		/* stop spinner (actually reduces start count) */
@@ -282,8 +288,9 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 		/* new content, so only viewed if this is already active */
 		embed->has_been_viewed = embed->is_active;
 
-		/* set tab properly */
-		embed_set_notebook_label_status (embed);
+		/* update tab title and status */
+		embed_update_tab_title (embed);
+		embed_update_tab_status (embed);
 	}
 
 	/* update the buttons view */
@@ -292,14 +299,6 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 		window_update_nav_controls (window);
 		window_update_status_bar (window);
 	}
-
-	if (is_newuser)
-	{
-		mozilla_prefs_set_fonts_defaults ();
-		/* FIXME this is very slow !!! */
-		mozilla_prefs_set_fonts ();
-		is_newuser = FALSE;
-	}
 }
 
 /**
@@ -310,7 +309,7 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 				    guint status, GaleonEmbed *embed) 
 {
 	GaleonWindow *window;
-	float f;
+/*	float f;*/
 
 	return_if_not_embed (embed);
 	window = embed->parent_window;
@@ -318,6 +317,10 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 
 	/* sync the zoom levels, this shouldn't be necessary but mozilla
 	 * ignore the zoom level setting until it's loaded its first doc */
+	/* FIXME is this really really necessary ? I dont like the idea to
+	   force a zoom level much, also I this is called too many times
+	   here ! */
+	/*
 	if (embed->wrapper != NULL)
 	{
 		mozilla_get_zoom(embed, &f);
@@ -325,7 +328,7 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 		{
 			mozilla_set_zoom (embed, (float)embed->zoom / 100.0);
 		}
-	}
+	}*/
 
 	/* clear temporary message, if any */
 	if (embed == window->active_embed)
@@ -486,7 +489,7 @@ mozembed_js_status_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	window = embed->parent_window;
 	return_if_not_window (window);
 
- 	if (gnome_config_get_bool (CONF_ADVANCED_STATUSBAR_REWRITE))
+ 	if (eel_gconf_get_boolean (CONF_FILTERING_STATUSBAR_REWRITE))
   	{
 		/* get the javascript message */
 		if (embed->is_active)
@@ -524,8 +527,8 @@ mozembed_dom_mouse_down_cb (GtkMozEmbed *dummy, gpointer dom_event,
 			    GaleonEmbed *embed)
 {
 	WrapperMouseEventInfo *info;
-	gint button_action = gnome_config_get_int
-				("/galeon/Mouse/right_button_action=0");
+	gint button_action = eel_gconf_get_integer
+				(CONF_MOUSE_RMB_ACTION);
 
 	return_val_if_not_embed (embed, FALSE);
 
@@ -619,7 +622,7 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	switch (info->button)
 	{
 	case 2:
-		if (gnome_config_get_int("/galeon/Mouse/right_button_action")
+		if (eel_gconf_get_integer(CONF_MOUSE_RMB_ACTION)
 		    == 1)
 		{
 			/* kill the motion listener, if it exists */
@@ -637,8 +640,9 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	case 1:
 		if (!(info->ctx.context & CONTEXT_LINK))
 		{
-			if (gnome_config_get_int(
-				"/galeon/Mouse/middle_button_action=1") == 0)
+			if (eel_gconf_get_integer(
+				CONF_MOUSE_MMB_ACTION) == 0)
+
 				/* popup a bookmarks menu */
 				context_show_bookmark_menu (embed);
 			else {
@@ -1003,7 +1007,7 @@ mozembed_destroy_cb (GtkObject *object, GaleonEmbed *embed)
 	if (embed->is_active)
 	{
 		embed->is_active = FALSE;
-		embed->parent_window->active_embed = NULL;
+		window->active_embed = NULL;
 	}
 
 	/* free memory */
@@ -1020,6 +1024,12 @@ mozembed_destroy_cb (GtkObject *object, GaleonEmbed *embed)
 		embed->site_title_utf8 = NULL;
 	}
 
+	/* no longer visible */
+	if (embed->is_visible)
+	{
+		window->visible_embeds--;
+	}
+
 	/* destroy C++ wrapper */
 	if (embed->wrapper)
 	{
