@@ -50,8 +50,11 @@
 #include "nsILocalFile.h"
 #include "nsPIDOMWindow.h"
 #include "nsIDOMWindowInternal.h"
+#include "nsIJVMManager.h"
+#include "nsIJSConsoleService.h"
 
 static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
+static NS_DEFINE_CID(kJVMManagerCID, NS_JVMMANAGER_CID);
 
 /* local function prototypes */
 static char  *convert_ns_string_to_c_string (const nsString & ns_string);
@@ -146,17 +149,69 @@ mozilla_preference_set_int (const char *preference_name, int new_int_value)
 	return FALSE;
 }
 
+/**
+ * mozilla_preference_get_int: get an integer mozilla preference
+ */
+extern "C" gint
+mozilla_preference_get_int (const char *preference_name)
+{
+	int value = -1;
+
+	g_return_val_if_fail (preference_name != NULL, FALSE);
+
+	nsCOMPtr<nsIPrefService> prefService = 
+				do_GetService (NS_PREFSERVICE_CONTRACTID);
+	nsCOMPtr<nsIPrefBranch> pref;
+	prefService->GetBranch ("", getter_AddRefs(pref));
+
+	if (pref)
+	{
+		pref->GetIntPref (preference_name, &value);
+	}
+
+	return value;
+}
+
+/**
+ * mozilla_preference_get: get a string mozilla preference
+ */
+extern "C" gchar *
+mozilla_preference_get(const char *preference_name)
+{
+	gchar *value = NULL;
+	gchar *result = NULL;
+
+	g_return_val_if_fail (preference_name != NULL, FALSE);
+	nsCOMPtr<nsIPrefService> prefService = 
+				do_GetService (NS_PREFSERVICE_CONTRACTID);
+	nsCOMPtr<nsIPrefBranch> pref;
+	prefService->GetBranch ("", getter_AddRefs(pref));
+
+	if (pref)
+	{
+		pref->GetCharPref (preference_name, &value);            
+	}
+
+	/* it's allocated by mozilla, so I could not g_free it */
+	if (value)
+	{
+		result = g_strdup (value);
+		nsMemory::Free (value);
+	}
+
+	return result;
+}
+
 /**
  * mozilla_wrapper_init: initialize the mozilla wrapper
  * @attach_listener: attach event listener. Currently used for
  * links drag and drop only.
  */
 extern "C" gpointer 
-mozilla_wrapper_init (GaleonEmbed *embed, gboolean attach_listener)
+mozilla_wrapper_init (GaleonEmbed *embed)
 {
 	nsresult result;
 	GaleonWrapper *wrapper = new GaleonWrapper ();
-	wrapper->attachListener = attach_listener;
 	result = wrapper->Init (embed);
 	if (NS_FAILED(result))
 		g_warning ("Wrapper initialization failed");
@@ -223,6 +278,21 @@ extern "C" gboolean mozilla_save_link (GaleonEmbed *embed, const char *url)
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
 }
 
+extern "C" gboolean mozilla_save_document (GaleonEmbed *embed, char *filename,
+					   const char *datapath, int action,
+					   gboolean mainDoc)
+{
+	nsresult rv;
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+
+	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
+
+	rv = wrapper->SaveDocument (filename, datapath, action,
+				    mainDoc ? TRUE : FALSE);
+
+	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+}
+
 extern "C" gboolean mozilla_reload (GaleonEmbed *embed)
 {
 	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
@@ -274,12 +344,12 @@ mozilla_get_zoom (GaleonEmbed *embed, float *f)
  * mozilla_print: print a document
  */
 extern "C" gboolean
-mozilla_print (GaleonEmbed *embed, PrintInfo *info)
+mozilla_print (GaleonEmbed *embed, PrintInfo *info, gboolean preview)
 {
 	nsresult result;
 	GaleonWrapper *wrapper = (GaleonWrapper *)(embed->wrapper);
 
-	NS_WITH_SERVICE(nsIPrintOptions, options, kPrintOptionsCID, &result);
+	nsCOMPtr<nsIPrintOptions> options(do_GetService(kPrintOptionsCID, &result));
 	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
 	if (info->pages)
@@ -304,7 +374,7 @@ mozilla_print (GaleonEmbed *embed, PrintInfo *info)
 
 	options->SetPrintSilent (PR_TRUE);
 
-	result = wrapper->Print(options);
+	result = wrapper->Print(options, preview);
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
 }
 
@@ -612,9 +682,9 @@ mozilla_get_permissions (int type)
 			PRBool cap;
 			nsPermission->GetCapability (&cap);
 			if (cap == PR_TRUE) 
-				b->type = g_strdup_printf ("Allowed");
+				b->type = g_strdup_printf (_("Allowed"));
 			else 
-				b->type = g_strdup_printf ("Blocked");
+				b->type = g_strdup_printf (_("Blocked"));
 
 			permissions = g_list_prepend (permissions, b);
 		}
@@ -764,6 +834,7 @@ mozilla_get_charsets (GHashTable **charsets, GList **sorted_charset_titles)
 		if (charset_title_str == NULL || 
 		    strlen (charset_title_str) == 0)
 		{
+			g_free (charset_title_str);
 			charset_title_str = g_strdup (charset_str);
 		}
 		
@@ -1107,11 +1178,16 @@ mozilla_find_gtk_parent (nsIDOMWindow *domWindow)
         		(do_QueryInterface(windowChrome, &result));
         if (NS_FAILED(result)) return nsnull;
 
-        GtkWidget *parentWindow;
-        result = window->GetSiteWindow ((void **)&parentWindow);
-        if (NS_FAILED(result)) return nsnull;
+	GtkWidget *mozembed;
+	result = window->GetSiteWindow ((void **)&mozembed);
+	if (NS_FAILED(result)) return nsnull;
 
-        return parentWindow;
+	GaleonEmbed *embed = (GaleonEmbed*)
+		gtk_object_get_data (GTK_OBJECT (mozembed), "GaleonEmbed");
+	g_assert (embed != NULL);
+	g_assert (embed->magic == GALEON_EMBED_MAGIC);
+
+	return embed->parent_window->WMain;
 }
 
 extern "C" gboolean
@@ -1216,64 +1292,6 @@ mozilla_paste (GaleonEmbed *embed)
 	wrapper->Paste ();
 }
 
-extern "C" gchar *
-mozilla_get_default_font (const char *language, 
-			  const char *type, 
-			  const char *default_font)
-{
-       nsresult result;
-       gchar *ret = NULL;
-       gchar *font_name = NULL;
-       PRBool moreElements;
- 
-       nsCOMPtr<nsIFontList> 
-               fontList = do_CreateInstance ("@mozilla.org/gfx/fontlist;1");
-       if (!fontList)
-               return NULL;
- 
-       nsCOMPtr<nsISimpleEnumerator> fontEnumerator;
-       result = fontList->AvailableFonts (mozilla_locale_to_unicode(language),
-                                          mozilla_locale_to_unicode(type),
-                                          getter_AddRefs(fontEnumerator));
-       if (NS_FAILED(result))
-               return NULL;
- 
-       nsCOMPtr<nsISupportsWString> font;
- 
-       for (fontEnumerator->HasMoreElements(&moreElements) ;
-            moreElements == PR_TRUE ;
-            fontEnumerator->HasMoreElements(&moreElements))
-       {
-               result = fontEnumerator->GetNext (getter_AddRefs(font));
-               if (NS_FAILED(result))
-                       return NULL;
- 
-               PRUnichar *fontName; 
-               result = font->ToString (&fontName);
-               font_name = mozilla_unicode_to_locale (fontName);
-               if (NS_FAILED(result))
-                       return NULL;
- 
-               if (!ret)
-               {
-                       ret = g_strdup (font_name);
-               }
-               else
-               {
-                       if (default_font &&
-			   strstr(font_name, default_font))
-		       {
-                               g_free (ret);
-                               ret = g_strdup (font_name);
-                       }
-               }
-               g_free (font_name);
-               nsMemory::Free (fontName);
-       }
-       
-       return ret;
-}
-
 extern "C" void
 mozilla_delete_temp_file_on_exit (const char *filename)
 {
@@ -1294,3 +1312,30 @@ mozilla_delete_temp_file_on_exit (const char *filename)
 	}
 #endif
 }
+
+extern "C" gboolean
+mozilla_show_java_console (void)
+{
+	nsresult rv;
+	nsCOMPtr<nsIJVMManager> JVMManager = do_GetService (kJVMManagerCID,
+							    &rv);
+	if (NS_FAILED(rv)) return FALSE;
+
+	rv = JVMManager->ShowJavaConsole ();
+
+	return rv ? TRUE : FALSE;
+}
+
+extern "C" gboolean
+mozilla_show_javascript_console (void)
+{
+	nsresult rv;
+	nsCOMPtr<nsIJSConsoleService> JSConsoleService =
+		do_GetService ("@mozilla.org/embedcomp/jsconsole-service;1",
+			       &rv);
+	if (NS_FAILED(rv)) return FALSE;
+
+	rv = JSConsoleService->Open (nsnull);
+
+	return rv ? TRUE : FALSE;
+}
