@@ -123,6 +123,7 @@ const CharsetInfo charset_trans_array[] = {
 	{"windows-936",                           "windows-936",           LG_OTHER}
 };
 
+/*
 const gchar *lang_encode_name[LANG_ENC_NUM] =
 {
 	N_("Western"),
@@ -141,6 +142,7 @@ const gchar *lang_encode_name[LANG_ENC_NUM] =
 	N_("Hebrew"),
 	N_("Arabic")
 };
+*/
 
 const gchar *lang_encode_item[LANG_ENC_NUM] =
 {
@@ -155,12 +157,20 @@ const gchar *lang_encode_item[LANG_ENC_NUM] =
 	"el",
 	"tr",
 	"x-unicode",
-	"x-user-def",
 	"th",
 	"he",
 	"ar"
 };
 
+const gchar *font_types[] =
+{
+        "serif",
+        "sans-serif",
+        "cursive",
+        "fantasy",
+        "monospace"
+};
+
 gint
 get_lgroups_count(void)
 {
