@@ -20,4 +20,5 @@ void mozilla_prefs_set (void);
 void mozilla_prefs_set_default_charset (void);
 void mozilla_prefs_set_languages (void);
 void mozilla_prefs_set_fonts_defaults (void);
-void mozilla_prefs_set_fonts (void);
+void mozilla_prefs_set_fonts (LanguageFontInfo *info, int lang_enc);
+void mozilla_prefs_get_fonts (LanguageFontInfo *info, int lang_enc);
