@@ -25,6 +25,7 @@
 #include "galeon.h"
 #include "bookmarks.h"
 #include "misc.h"
+#include "prefs.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -579,7 +580,7 @@ netscape_import_prefs(void)
 
 	/* now we set some of galeon prefs to whats in "ns_prefs *prefs" */
 
-	gnome_config_set_string("/galeon/General/startpage", prefs->homepage);
+	eel_gconf_set_string(CONF_GENERAL_HOMEPAGE, prefs->homepage);
 	switch (prefs->startup_page)
 	{
 		case 0: buffer = 2; break;
@@ -588,24 +589,31 @@ netscape_import_prefs(void)
 	}
 	if (prefs->proxy_type == 1)  /* if manual proxy (auto not yet supported) */
 	{
-		gnome_config_push_prefix("/galeon/Advanced/");
-		gnome_config_set_bool ("proxy_enabled", prefs->proxy_type > 0);
-		gnome_config_set_string("http_proxy", prefs->http_proxy);
-		gnome_config_set_string("ftp_proxy", prefs->ftp_proxy);
-		gnome_config_set_int("http_proxy_port", prefs->http_proxy_port);
-		gnome_config_set_int("ftp_proxy_port", prefs->ftp_proxy_port);
-		gnome_config_set_string("no_proxies_for", prefs->no_proxies);
-		gnome_config_pop_prefix();
+		eel_gconf_set_boolean(CONF_NETWORK_PROXY_ENABLED, 
+				      prefs->proxy_type > 0);
+		eel_gconf_set_string(CONF_NETWORK_HTTP_PROXY, 
+				     prefs->http_proxy);
+		eel_gconf_set_string(CONF_NETWORK_FTP_PROXY, 
+				     prefs->ftp_proxy);
+	        eel_gconf_set_integer(CONF_NETWORK_HTTP_PROXY_PORT, 
+				      prefs->http_proxy_port);
+		eel_gconf_set_integer(CONF_NETWORK_FTP_PROXY_PORT, 
+				      prefs->ftp_proxy_port);
+		eel_gconf_set_string(CONF_NETWORK_NO_PROXIES_FOR, 
+				     prefs->no_proxies);
 	}
-	gnome_config_set_int (CONF_GENERAL_STARTPAGE_TYPE, buffer);
-	gnome_config_set_int (CONF_APPEARANCE_TOOLBAR_STYLE, 
-			      prefs->toolbar_style);
-	gnome_config_set_int (CONF_APPEARANCE_WINWIDTH, prefs->win_width);
-	gnome_config_set_int (CONF_APPEARANCE_WINHEIGHT, prefs->win_height);
-	gnome_config_set_int (CONF_ADVANCED_JAVA_ENABLED, prefs->enable_java);
-	gnome_config_set_int (CONF_ADVANCED_JAVASCRIPT_ENABLED, 
-			      prefs->enable_javascript);	
-	gnome_config_sync();
+	eel_gconf_set_integer(CONF_GENERAL_STARTPAGE_TYPE, 
+			     buffer);
+	eel_gconf_set_integer(CONF_TOOLBAR_STYLE, 
+			     prefs->toolbar_style);
+	eel_gconf_set_integer(CONF_STATE_WINWIDTH, 
+			     prefs->win_width);
+	eel_gconf_set_integer(CONF_STATE_WINHEIGHT, 
+			     prefs->win_height);
+	eel_gconf_set_boolean(CONF_FILTERING_JAVA_ENABLED, 
+			     prefs->enable_java);
+	eel_gconf_set_boolean(CONF_FILTERING_JAVASCRIPT_ENABLED, 
+			     prefs->enable_javascript);	
 
 /* 	set_toolbar_style_global ();
 	mozilla_prefs_set ();	*/
