@@ -21,6 +21,7 @@
 #include "embed.h"
 #include "glade.h"
 #include "bookmarks.h"
+#include "prefs.h"
 
 #include <string.h>
 #include <gtk/gtktogglebutton.h>
@@ -37,7 +38,7 @@ gboolean new_user_druid_delete_event_cb (GtkWidget *widget, GdkEventAny *event,
 void new_user_druid_create_smart_bookmarks_check_button_toggled_cb 
 (GtkWidget *toggle);
 static void add_smart (BookmarkItem *parent, gchar *name, gchar *nick,
-		       gchar *url, gchar *pixmap_file);
+		       gchar *url, gchar *pixmap_file, gchar *notes);
 static void add_bookmarklets (BookmarkItem *parent);
 
 /**
@@ -206,13 +207,13 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 			 * the google homepage I think -- MattA) */
 			add_smart (toolbar, "Google", "google",
 				   _("http://www.google.com/search?q=%s"),
-				   SHARE_DIR "/google.png");
+				   SHARE_DIR "/google.png", NULL);
 		}
 		if (google_groups_checked)
 		{
 			add_smart (toolbar, "Usenet", "news",
 				   "http://groups.google.com/groups?q=%s"
-				   "&meta=site%3Dgroups", NULL);
+				   "&meta=site%3Dgroups", NULL, NULL);
 		}
 		if (dogpile_checked)
 		{
@@ -226,13 +227,13 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 				   "engines=lycos&engines=kanoodle&"
 				   "engines=opendir&engines=realnames&"
 				   "engines=altavista&engines=yahoo",
-				   NULL);
+				   NULL, NULL);
 		}
 		if (yahoo_checked)
 		{
 			add_smart (toolbar, "Yahoo!", "yahoo",
 				   "http://search.yahoo.com/bin/search?p=%s",
-				   NULL);
+				   NULL, NULL);
 		}
 		if (dictionary_checked)
 		{
@@ -242,27 +243,27 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 			add_smart (toolbar, _("Dictionary"), "dict",
 				   /* xgettext: no-c-format */
 				   _("http://www.m-w.com/cgi-bin/"
-				     "dictionary?va=%s"), NULL);
+				     "dictionary?va=%s"), NULL, NULL);
 		}
 		if (bugzilla_checked)
 		{
 			add_smart (toolbar, "Bug #", "bug",
 				   "http://bugzilla.gnome.org/"
 				   "show_bug.cgi?id=%s", 
-				   SHARE_DIR "/bug.png");
+				   SHARE_DIR "/bug.png", NULL);
 		}
 		if (freshmeat_checked)
 		{
 			add_smart (toolbar, "Freshmeat", "fm",
 				   "http://freshmeat.net/search?q=%s",
-				   SHARE_DIR "/freshmeat.png");
+				   SHARE_DIR "/freshmeat.png", NULL);
 		}
 		if (rpmfind_checked)
 		{
 			add_smart (toolbar, "RPMFind", "rpm",
 				   "http://rpmfind.net/linux/rpm2html/"
 				   "search.php?query=%s",
-				   SHARE_DIR "/rpm.png");
+				   SHARE_DIR "/rpm.png", NULL);
 		}
 		if (debian_packages_checked)
 		{
@@ -271,7 +272,7 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 				   "search_packages.pl?keywords=%s&"
 				   "searchon=names&subword=1&version=all&"
 				   "release=all", 
-				   SHARE_DIR "/debian.png");
+				   SHARE_DIR "/debian.png", NULL);
 		}
 		if (bookmarklets_checked)
 		{
@@ -289,11 +290,6 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	{
   	        /* FIXME: dont use this callback directly! */
 	        bookmarks_import_mozilla_activate_cb (NULL, NULL);
-		
-#ifdef ENABLE_NLS
-		/* FIXME: right thing in the right place? */
-		mozilla_prefs_import_language ();
-#endif
 	}
 	
 	/* set URL handlers as requested */
@@ -317,25 +313,29 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	gnome_config_pop_prefix ();
 	
 	/* setup proxies */
-	gnome_config_push_prefix("/galeon/Advanced/");
 	if (http_proxy_site != NULL && strlen (http_proxy_site) > 0)
 	{
-		gnome_config_set_bool ("proxy_enabled", TRUE);
-		gnome_config_set_string ("http_proxy", http_proxy_site);
-		gnome_config_set_string ("ssl_proxy",  ssl_proxy_site);
-		gnome_config_set_string ("ftp_proxy",  ftp_proxy_site);
-		gnome_config_set_int ("http_proxy_port", http_proxy_port);
-		gnome_config_set_int ("ssl_proxy_port",  ssl_proxy_port);
-		gnome_config_set_int ("ftp_proxy_port",  ftp_proxy_port);
+		eel_gconf_set_boolean(CONF_NETWORK_PROXY_ENABLED
+				      , TRUE);
+		eel_gconf_set_string (CONF_NETWORK_HTTP_PROXY
+				      , http_proxy_site);
+		eel_gconf_set_string (CONF_NETWORK_SSL_PROXY
+				      ,  ssl_proxy_site);
+		eel_gconf_set_string (CONF_NETWORK_FTP_PROXY
+				      ,  ftp_proxy_site);
+	        eel_gconf_set_integer(CONF_NETWORK_HTTP_PROXY_PORT
+				      , http_proxy_port);
+		eel_gconf_set_integer(CONF_NETWORK_SSL_PROXY_PORT
+				      ,  ssl_proxy_port);
+		eel_gconf_set_integer(CONF_NETWORK_FTP_PROXY_PORT
+				      ,  ftp_proxy_port);
 	}
-	gnome_config_pop_prefix ();
 	
 	/* kill the dialog */
 	dialog = glade_lookup_widget (widget, "new_user_druid");
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 	
 	/* save all the initial settings */
-	gnome_config_sync ();
 	bookmarks_updated ();
 	
 	/* create an initial window if needed */
@@ -411,12 +411,12 @@ new_user_druid_create_smart_bookmarks_check_button_toggled_cb
  */
 static void
 add_smart (BookmarkItem *parent, gchar *name, gchar *nick, 
-	   gchar *url, gchar *pixmap_file)
+	   gchar *url, gchar *pixmap_file, gchar * notes)
 {
 	BookmarkItem *site;
 	
 	/* create the bookmarks */
-	site = bookmarks_new_bookmark (BM_SITE, TRUE, name, url, nick, NULL, 
+	site = bookmarks_new_bookmark (BM_SITE, TRUE, name, url, nick, notes, 
 				       pixmap_file);
 	
 	/* add it to the toolbar */
