@@ -16,7 +16,11 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "eel-gconf-extensions.h"
+
 /* prefs.c */
+guint prefs_key_len (gchar const * key);
+gchar * prefs_key_new (gchar const * in);
 PreferencesDialog *preferences_open (GaleonWindow *window);
 void preferences_show_page (GaleonWindow *window, PreferencesPageId id);
 void preferences_close (PreferencesDialog *pd);
@@ -35,7 +39,10 @@ void preferences_set_animate (AnimateType value);
 void preferences_set_autojump (gboolean value);
 void preferences_set_handlers_enable_mime (gboolean value);
 gboolean preferences_galeon_used_by_gnome_for_ftp (void);
-
+void prefs_fonts_remember (PreferencesDialog *pd);
+void prefs_fonts_language_init (PreferencesDialog *pd);
+void prefs_encoding_activated_cb (GtkWidget *button, 
+				  PreferencesDialog *pd);
 /* FIXME: these shouldn't be exported */
 void set_config_from_editable(GtkWidget *editable, gchar *config_name);
 void set_config_from_radiobuttongroup (GtkWidget *radiobutton, 
