@@ -114,8 +114,9 @@ void prefs_download_use_command_toggled_cb (GtkWidget *button,
 					    PreferencesDialog *pd);
 void prefs_mime_list_sort_cb (GtkCList *clist, gint column,
 			      PreferencesDialog *pd);
-void prefs_mime_save_to_disk_toggled_cb (GtkWidget *button, 
-					 PreferencesDialog *pd);
+void prefs_mime_save_to_disk_toggled_cb (GtkWidget *button, PreferencesDialog *pd); 
+void prefs_theme_select_row_cb (GtkCList *list, gint row, gint column,
+				GdkEvent *event, PreferencesDialog *pd);
 
 /**
  * prefs_mime_list_sort_cb:
@@ -209,16 +210,16 @@ prefs_mime_add_button_clicked_cb (GtkButton *button, PreferencesDialog *pd)
 	gchar *text[5];
 	gint row;
 
-	newitem->name = g_strdup (N_("New mime item"));
+	newitem->name = g_strdup (_("New mime item"));
 	newitem->action = MIME_UNKNOWN;
-	newitem->helperApp = g_strdup (N_("None"));
+	newitem->helperApp = g_strdup (_("None"));
 	newitem->terminalHelper = FALSE;
 	newitem->urlHelper = FALSE;
 	newitem->alwaysUse = FALSE;
 	text[0] = newitem->name;
-	text[1] = N_("Unknown");
+	text[1] = _("Unknown");
 	text[2] = newitem->helperApp;
-	text[3] = N_("False");
+	text[3] = _("False");
 	text[4] = NULL;
 	row = gtk_clist_append (GTK_CLIST (clist), text);
 	gtk_clist_set_row_data (GTK_CLIST (clist), row, newitem);
@@ -374,25 +375,25 @@ prefs_mime_edit_cb (GtkWidget *w, PreferencesDialog *pd)
 	switch (pd->selected_mime->action)
 	{
 	case MIME_SAVE_TO_DISK:
-		text[1] = N_("Save to disk");
+		text[1] = _("Save to disk");
 		break;
 	case MIME_RUN_PROGRAM:  
-		text[1] = N_("Run with Helper App");
+		text[1] = _("Run with Helper App");
 		break;
 	case MIME_UNKNOWN:
-		text[1] = N_("Unknown");
+		text[1] = _("Unknown");
 		break;
 	case MIME_ASK_ACTION:
-		text[1] = N_("Ask the user");
+		text[1] = _("Ask the user");
 		break;
 	case MIME_IGNORE:
 		break;
 	}
 	text[2] = pd->selected_mime->helperApp;
 	if (pd->selected_mime->alwaysUse)
-		text[3] = N_("True");
+		text[3] = _("True");
 	else
-		text[3] = N_("False");
+		text[3] = _("False");
 
 	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 0, text[0]);
 	gtk_clist_set_text (GTK_CLIST (pd->mime_list), row, 1, text[1]);
@@ -558,6 +559,16 @@ prefs_spinner_select_icon_cb (GnomeIconList *gil, gint num,
 	gtk_entry_set_text (GTK_ENTRY (pd->spinner_dir_entry), path);
 }
 
+void
+prefs_theme_select_row_cb (GtkCList *list, gint row, gint column,
+			   GdkEvent *event, PreferencesDialog *pd)
+{
+	const gchar *path;
+
+	path = gtk_clist_get_row_data (list, row);
+	gtk_entry_set_text (GTK_ENTRY (pd->theme_dir_entry), path);
+}
+
 void 
 prefs_smart_bm_entry_width_spin_changed_cb (GtkEditable *editable,
 					    PreferencesDialog *pd)
@@ -666,3 +677,11 @@ prefs_mime_save_to_disk_toggled_cb (GtkWidget *button, PreferencesDialog *pd)
 	active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
 	gtk_widget_set_sensitive (pd->mime_helper_frame, !active);
 }
+
+void
+prefs_encoding_activated_cb (GtkWidget *button, 
+			     PreferencesDialog *pd)
+{
+	prefs_fonts_remember (pd);
+	prefs_fonts_language_init (pd);
+}
