@@ -26,6 +26,7 @@
 #include "prefs.h"
 #include "history.h"
 #include "mozilla.h"
+#include "eel-gconf-extensions.h"
 
 #include <gtkmozembed.h>
 #include <string.h>
@@ -72,7 +73,6 @@
 /* local function prototypes */
 static xmlDocPtr session_load (const gchar *filename);
 static void session_resume (xmlDocPtr doc, gboolean override_behaviour);
-static void galeon_exit (void);
 static void session_close_all_windows (void);
 
 /* local variables */
@@ -80,6 +80,10 @@ static GtkWidget *dialog = NULL;
 
 /** The global list of all GaleonWindow structures */
 GList *all_windows = NULL;
+gint window_count = 0;
+
+/* session quit timeout id */
+gint quit_timeout_id = 0;
 
 /**
  * session_autosave: save to the "crash recovery" session file
@@ -91,7 +95,7 @@ session_autosave (void)
 	gint autosave;
 
 	/* check we're configured to autosave */
-	autosave = gnome_config_get_int("/galeon/Advanced/crash_recovery=1");
+	autosave = eel_gconf_get_integer(CONF_CRASH_RECOVERY);
 	if (autosave == FALSE)
 		return;
 
@@ -118,17 +122,17 @@ session_autoresume (void)
 	gint crashed, saved, choice;
 
 	/* see if there's a crashed or saved session to resume */
-	crashed = gnome_config_get_int ("/galeon/General/crashed=0");
-	saved = gnome_config_get_int ("/galeon/General/session_saved=0");
-	gnome_config_set_int ("/galeon/General/session_saved", 0);
-	gnome_config_sync ();
+	crashed = eel_gconf_get_integer (CONF_CRASH_CRASHED);
+	saved = eel_gconf_get_integer (CONF_STATE_SESSION_SAVED);
+	eel_gconf_set_integer (CONF_STATE_SESSION_SAVED, 0);
+
 	if (crashed == FALSE && saved == FALSE)
 	{
 		return FALSE;
 	}
 
 	/* see if we're configured to autoresume */
-	autoresume = gnome_config_get_int("/galeon/Advanced/crash_recovery=1");
+	autoresume = eel_gconf_get_integer(CONF_CRASH_RECOVERY);
 	if (autoresume == RESUME_NEVER && !saved)
 	{
 		return FALSE;
@@ -372,8 +376,8 @@ session_resume (xmlDocPtr doc, gboolean override_behaviour)
 	else
 	{
 		/* decide whether to put into bookmarks or windows */
-		behaviour = gnome_config_get_int
-			(CONF_ADVANCED_CRASH_BEHAVIOUR);
+		behaviour = eel_gconf_get_integer 
+			(CONF_CRASH_BEHAVIOUR);
 	}
 	
 	if (behaviour == 1)
@@ -467,6 +471,9 @@ session_add_window (GaleonWindow *window)
 {
 	/* add it to the list of windows */
 	all_windows = g_list_prepend (all_windows, window);
+
+	/* Increment window count */
+	window_count++;
 }
 
 void 
@@ -475,9 +482,13 @@ session_remove_window (GaleonWindow *window)
 	/* remove window from list */
 	all_windows = g_list_remove (all_windows, window);
 
+	/* Decrement window count */
+	window_count--;
+
 	/* quit if this was the last window to go and
 	 * we're not in server mode */
-	if (g_list_length (all_windows) == 0 && !galeon_server_mode)
+	if (g_list_length (all_windows) == 0 && !galeon_server_mode
+	    && window_count ==0)
 	{
 		galeon_exit ();
 	}
@@ -496,8 +507,7 @@ session_save (void)
 	session_save_to (filename);
 	
 	/* set config marker */
-	gnome_config_set_int ("/galeon/General/session_saved", 1);
-	gnome_config_sync ();
+	eel_gconf_set_integer (CONF_STATE_SESSION_SAVED, 1);
 	
 	g_free (filename);
 }
@@ -505,12 +515,16 @@ session_save (void)
 void 
 session_quit (gboolean auto_save)
 {
+	/* if we were called as a quit timeout handler remove the timeout */
+	if (quit_timeout_id)
+	{
+		gtk_timeout_remove (quit_timeout_id);
+	}
+
 	/* save the session, if it was requested or if the user has the
 	 * "always save" option enabled */
 	if (auto_save ||
-	    gnome_config_get_bool (CONF_GENERAL_ALWAYS_SAVE_SESSION))
-
-	{
+	    eel_gconf_get_boolean (CONF_GENERAL_ALWAYS_SAVE_SESSION)) {
 		session_save ();
 	}
 
@@ -533,35 +547,6 @@ session_quit_cb (gpointer auto_save_ptr)
 	session_quit (GPOINTER_TO_INT (auto_save_ptr));
 }
 
-static
-void galeon_exit (void)
-{
-
-	gnome_config_set_int("/galeon/General/crashed", FALSE);
-
-	/* FIXME: uncatch GDK errors which happen on theme changes */
-	gdk_error_trap_pop ();
-
-	/* shut down galeon subsystems */
-	preferences_save ();
-	mime_db_shutdown ();
-	bookmarks_exit ();
-	history_exit ();
-	mozilla_save_prefs ();
-
-	/* check... */
-	g_assert (g_list_length (all_embeds) == 0);
-	g_assert (g_list_length (all_windows) == 0);
-
-	/* shut down GNOME subsystmes */
-	gnome_vfs_shutdown ();
-
-	/* absolutely no more mozembeds */
-	gtk_moz_embed_pop_startup ();
-
-	gtk_main_quit();
-}
-
 static void
 session_close_all_windows (void)
 {
@@ -646,17 +631,16 @@ session_history_get (void)
 {
 	GList *ret = NULL;
 	gint i, count;
-	gnome_config_push_prefix ("/galeon/SessionHistory/");
-	count = gnome_config_get_int ("count=0");
+
+	count = eel_gconf_get_integer (CONF_HISTORY_SESSION_COUNT);
 	for (i = count; i > 0; i--) 
 	{
-		gchar *key = g_strdup_printf ("File%d", i);
-		gchar *file = gnome_config_get_string (key);
+		gchar *key = g_strdup_printf (CONF_HISTORY_SESSION_PATH"/File%d", i);
+		gchar *file = eel_gconf_get_string (key);
 		if (file != NULL)
 			ret = g_list_prepend (ret, file);
 		g_free (key);
 	}
-	gnome_config_pop_prefix ();
 	return ret;
 }
 
@@ -682,21 +666,20 @@ session_history_add (const gchar *filename)
 	/* prepend the new item */
 	current_list = g_list_prepend (current_list, g_strdup (filename));
 	/* save the new list */
-	gnome_config_push_prefix ("/galeon/SessionHistory/");
-	max = gnome_config_get_int ("maxItems=4");
+	max = eel_gconf_get_integer (CONF_HISTORY_SESSION_MAXITEMS);
 	count = 0;
 	for (li = current_list; li != NULL; li = li->next) 
 	{
-		gchar *key = g_strdup_printf ("File%d", ++count);
+		gchar *key = g_strdup_printf (CONF_HISTORY_SESSION_PATH"/File%d", 
+					      ++count);
 		gchar *file = li->data;
-		gnome_config_set_string (key, file);
+		eel_gconf_set_string (key, file);
 		g_free (file);
 		g_free (key);
 		if (count == max) break;
 	}
-	gnome_config_set_int ("count", count);
-	gnome_config_pop_prefix ();
-	gnome_config_sync ();
+	eel_gconf_set_integer (CONF_HISTORY_SESSION_COUNT, count);
+
 	g_list_free (current_list);
 	/* update the GUI */
 	window_session_history_update_all ();
