@@ -28,3 +28,6 @@ void session_load_from (const gchar *filename);
 GList *session_history_get (void);
 void session_history_add (const gchar *filename);
 void session_history_load_index (gint index);
+
+/* quit timeout id */
+extern gint quit_timeout_id;
