@@ -22,8 +22,7 @@
 #include "galeon.h"
 
 void state_save_window_state (GdkWindow *window, const gchar *name);
-void state_load_window_state (GtkWidget *window, const gchar *name, 
-			      int defwidth, int defheight);
+void state_load_window_state (GtkWidget *window, const gchar *name);
 
 void state_save_column_widths (GtkWidget *w, const gchar *name);
 void state_load_column_widths (GtkWidget *w, const gchar *name);
