@@ -23,6 +23,7 @@
 #include "misc.h"
 #include "window.h"
 #include "bookmarks.h"
+#include "eel-gconf-extensions.h"
 
 #include <string.h>
 #include <gtk/gtkspinbutton.h>
@@ -36,20 +37,21 @@
 #include <libgnomeui/gnome-preferences.h>
 #include <libgnomeui/gnome-dock.h>
 #include <libgnomeui/gnome-entry.h>
-
-/* GNOME includes */
-#include <gconf/gconf.h>
-#include <gconf/gconf-client.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 /* local function prototypes */
 static void toolbar_update (GaleonWindow *window);
 static void toolbar_create_widgets (GaleonWindow *window, 
 				    GtkWidget *hbox, GtkWidget *toolbar,
 				    gboolean location_on_main_toolbar);
-static void toolbar_create_location_toolbar (GaleonWindow *window);
+static void toolbar_create_location_toolbar (GaleonWindow *window,
+					     gboolean show);
 static void toolbar_create_location_widgets (GaleonWindow *window, 
 					     GtkWidget *hbox, 
 					     GtkWidget *toolbar);
+static void toolbar_create_find_entry (GaleonWindow *window,
+				       GtkWidget *toolbar,
+				       ToolbarItemType type);
 static void toolbar_create_zoom_spin (GaleonWindow *window, 
 				      GtkWidget *toolbar);
 static void toolbar_create_drag_handle (GaleonWindow *window, 
@@ -60,17 +62,8 @@ static GtkWidget *toolbar_add_docked (GaleonWindow *window, GtkWidget *hbox,
 				      gchar *name, gint band_num);
 static void toolbar_set_style (GtkWidget *toolbar, GtkToolbarStyle final_style,
 			       GtkReliefStyle relief);
-static gchar *toolbar_get_theme_path (void);
-static GtkWidget *toolbar_get_theme_pixmap (const gchar *pixmap, 
-					    gboolean is_nautilus);
-static gboolean toolbar_theme_use_standard_icons (void);
-
-/* theme settings -- setup once per toolbar creation */
-static gchar *theme_path;
-static gchar *theme_name;
-
-/* path to Nautilus theme pref in config space */
-#define NAUTILUS_THEME_PREF "/apps/nautilus/preferences/theme"
+static GtkWidget *toolbar_get_theme_pixmap (const gchar *pixmap);
+static char *toolbar_get_nautilus_theme_path (void);
 
 /* array of available toolbar items */
 const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT] =
@@ -252,8 +245,8 @@ const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT] =
 		FALSE, /* priority? */
 		FALSE,
 		800,
-		window_find_button_clicked_cb,
-		window_generic_button_press_event_cb,
+		NULL,
+		NULL,
 		NULL,
 		NULL /* drag_data_received */
 	},
@@ -302,27 +295,14 @@ const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT] =
  * toolbar_create: create main toolbar
  */
 void
-toolbar_create (GaleonWindow *window)
+toolbar_create (GaleonWindow *window, gboolean show)
 {
 	gboolean location_on_main_toolbar;
-	ToolbarIcons toolbar_icons;
 	GtkWidget *hbox, *toolbar;
 
-	/* which icons to use */
-	toolbar_icons = gnome_config_get_int (CONF_APPEARANCE_TOOLBAR_ICONS);
-	if (toolbar_icons == TOOLBAR_ICONS_NAUTILUS)
-	{
-		/* get path or fail */
-		theme_path = toolbar_get_theme_path ();
-		if (theme_path == NULL || toolbar_theme_use_standard_icons ())
-		{
-			toolbar_icons = TOOLBAR_ICONS_NORMAL;
-		}
-	}
-
 	/* read prefs */
-	location_on_main_toolbar = gnome_config_get_bool 
-		(CONF_APPEARANCE_URL_LOCATION);
+	location_on_main_toolbar = eel_gconf_get_boolean
+		(CONF_TOOLBAR_URL_LOCATION);
 
 	/* build the overall box to pack the toolbar into */
 	hbox = gtk_hbox_new (FALSE, 2);
@@ -348,24 +328,18 @@ toolbar_create (GaleonWindow *window)
 				location_on_main_toolbar);
 
 	/* show the box and all its contents */
-	gtk_widget_show_all (window->main_dockitem);
+	gtk_widget_show_all (hbox);
+	if (show) gtk_widget_show (window->main_dockitem);
 
 	/* create location toolbar, if necessary */
 	if (!location_on_main_toolbar)
 	{
 		/* create it's own toolbar */
-		toolbar_create_location_toolbar (window);
+		toolbar_create_location_toolbar (window, show);
 	}
 
 	/* update the toolbar appearance */
 	toolbar_update (window);
-
-	/* free theme info */
-	if (toolbar_icons == TOOLBAR_ICONS_NAUTILUS)
-	{
-		g_free (theme_path);
-		g_free (theme_name);
-	}
 }
 
 /**
@@ -375,11 +349,13 @@ void
 toolbar_recreate (GaleonWindow *window)
 {
 	gchar *text = NULL;
+	gboolean show = FALSE;
 
 	/* get current text in url entry if any */
 	/* NB: this MUST be strdup'd here since it's only a reference
 	 * and we're about to destroy the toolbar (D'Oh! -- MattA) */
-	if (window->toolBarOn)
+	if (window->toolBarOn && window->location_gnomeentry && 
+	    window->location_entry)
 	{
 		text = g_strdup (gtk_entry_get_text 
 				 (GTK_ENTRY (window->location_entry)));
@@ -395,6 +371,7 @@ toolbar_recreate (GaleonWindow *window)
 	/* destroy main toolbar */
 	if (window->main_dockitem != NULL)
 	{
+		show = GTK_WIDGET_VISIBLE (window->main_dockitem);
 		gtk_widget_destroy (window->main_dockitem->parent);
 		window->main_dockitem = NULL;
 		window->main_toolbar = NULL;
@@ -419,7 +396,6 @@ toolbar_recreate (GaleonWindow *window)
 	window->home_button         = NULL;
 	window->stop_button         = NULL;
 	window->go_button           = NULL;
-	window->find_button         = NULL;
 	window->zoom_spin           = NULL;
 	window->drag_event_box      = NULL;
 	window->drag_pixmap         = NULL;
@@ -429,13 +405,14 @@ toolbar_recreate (GaleonWindow *window)
 	window->bookmarks_button    = NULL;
 
 	/* create new toolbar */
-	toolbar_create (window);
+	toolbar_create (window, show);
 	
 	/* set the controls properly for this window */
 	window_update_nav_controls (window);
 
 	/* restore location entry text and history */
-	if (window->toolBarOn)
+	if (window->toolBarOn && window->location_gnomeentry && 
+	    window->location_entry)
 	{
 		gnome_entry_load_history  
 			(GNOME_ENTRY (window->location_gnomeentry));
@@ -474,7 +451,7 @@ toolbar_update (GaleonWindow *window)
 	relief = (toolbar_button_relief ? GTK_RELIEF_NORMAL : GTK_RELIEF_NONE);
 
 	/* get real style */
-	style = gnome_config_get_int (CONF_APPEARANCE_TOOLBAR_STYLE);
+	style = eel_gconf_get_integer (CONF_TOOLBAR_STYLE);
 
 	/* convert to GTK+ style for setting in buttons */
 	final_style = GTK_TOOLBAR_BOTH; /* only to stop a warning */
@@ -509,47 +486,18 @@ toolbar_create_widgets (GaleonWindow *window,
 			gboolean location_on_main_toolbar)
 {
 	GtkWidget *spinner;
-	gint i, count;
-
-	/* create defaults when no configuration exists */
-	gnome_config_push_prefix ("/galeon/Toolbar/");
-	count = gnome_config_get_int ("count=-1");
-	if (count == -1)
-	{
-		/* default toolbar layout */
-		count = 9;
-		gnome_config_set_int ("count", count);
-		gnome_config_set_int ("item0", TOOLBAR_ITEM_BACK);
-		gnome_config_set_int ("item1", TOOLBAR_ITEM_FORWARD);
-		gnome_config_set_int ("item2", TOOLBAR_ITEM_REFRESH);
-		gnome_config_set_int ("item3", TOOLBAR_ITEM_START);
-		gnome_config_set_int ("item4", TOOLBAR_ITEM_STOP);
-		gnome_config_set_int ("item5", TOOLBAR_ITEM_ZOOM_SPIN);
-		gnome_config_set_int ("item6", TOOLBAR_ITEM_DRAG_HANDLE);
-		gnome_config_set_int ("item7", TOOLBAR_ITEM_LOCATION_ENTRY);
-		gnome_config_set_int ("item8", TOOLBAR_ITEM_SPINNER);
-		gnome_config_sync ();
-	}
-
-	/* create all toolbar items on the main toolbar */
-	for (i = 0; i < count; i++)
+	GSList *icon_list, *tmp;
+	
+	icon_list = eel_gconf_get_integer_list(CONF_TOOLBAR_ICON_LIST);
+	tmp = icon_list;
+	
+	while (tmp!=NULL) 
 	{
 		ToolbarItemType type;
-		gchar *key;
 
 		/* get toolbar item type */
-		key = g_strdup_printf ("item%d=-1", i);
-		type = gnome_config_get_int (key);
-		g_assert (type != -1);
-		g_free (key);
+		type = GPOINTER_TO_INT(tmp->data);
 	
-		/* check for illegal values */
-		if (i < 0 || i >= TOOLBAR_ITEM_COUNT)
-		{
-			g_warning (_("illegal toolbar item type\n"));
-			continue;
-		}
-
 		/* build the right thing */
 		switch (type)
 		{
@@ -581,9 +529,10 @@ toolbar_create_widgets (GaleonWindow *window,
 		case TOOLBAR_ITEM_SPINNER:
 			/* create the spinner */
 			spinner = spinner_create (window);
+			if (!spinner) break;
 			
 			/* if not the last item... */
-			if (i != count - 1)
+			if (g_slist_next(tmp)!=NULL)
 			{
 				/* pack it normally */
 				gtk_toolbar_append_element 
@@ -613,15 +562,20 @@ toolbar_create_widgets (GaleonWindow *window,
 			}
 			break;
 
+		case TOOLBAR_ITEM_FIND:
+			/* create find entry */
+			toolbar_create_find_entry (window, toolbar, type);
+			break;
+			
 		default:
 			/* create a normal button */
 			toolbar_create_button (window, toolbar, type);
 			break;
 		}
+		tmp = g_slist_next(tmp);
 	}
 
-	/* done parsing config */
-	gnome_config_pop_prefix ();
+	g_slist_free(icon_list);
 }
 
 /**
@@ -678,8 +632,8 @@ toolbar_create_location_widgets (GaleonWindow *window,
 	gboolean location_on_main_toolbar;
 
 	/* read prefs */
-	location_on_main_toolbar = gnome_config_get_bool 
-		(CONF_APPEARANCE_URL_LOCATION);
+	location_on_main_toolbar = eel_gconf_get_boolean
+		(CONF_TOOLBAR_URL_LOCATION);
 
 	/* create drag handle if on separate toolbar */
 	if (!location_on_main_toolbar)
@@ -740,7 +694,7 @@ toolbar_create_location_widgets (GaleonWindow *window,
  * toolbar_create_location_toolbar: create location toolbar
  */
 static void
-toolbar_create_location_toolbar (GaleonWindow *window)
+toolbar_create_location_toolbar (GaleonWindow *window, gboolean show)
 {
 	GtkWidget *hbox;
 	
@@ -769,7 +723,35 @@ toolbar_create_location_toolbar (GaleonWindow *window)
 					 window->location_toolbar);
 
 	/* show everything */
-	gtk_widget_show_all (window->location_dockitem);
+	gtk_widget_show_all (hbox);
+	if (show) gtk_widget_show (window->location_dockitem);
+}
+
+/**
+ * toolbar_create_find_entry: create find in page entry
+ */
+static void
+toolbar_create_find_entry (GaleonWindow *window, GtkWidget *toolbar,
+			   ToolbarItemType type)
+{
+	const ToolbarItem *item;
+	GtkWidget *entry;
+
+	/* get item from array */
+	item = &(toolbar_items[type]);
+
+	/* create the entry, pack it */
+	entry = gtk_entry_new ();
+	gtk_widget_set_usize (entry, 100, -2);	
+	gtk_toolbar_append_element (GTK_TOOLBAR (toolbar), 
+				    GTK_TOOLBAR_CHILD_WIDGET,
+				    GTK_WIDGET (entry), NULL, 
+				    _(item->tooltip), NULL, NULL, NULL, NULL);
+
+	/* connect signal (Return should search for inserted text) */
+	gtk_signal_connect (GTK_OBJECT (entry), "activate",
+			    GTK_SIGNAL_FUNC (window_find_entry_activate_cb), 
+			    window);
 }
 
 /**
@@ -814,38 +796,61 @@ static void
 toolbar_create_button (GaleonWindow *window, GtkWidget *toolbar, 
 		       ToolbarItemType type)
 {
-	ToolbarIcons toolbar_icons;
 	GtkWidget *button, *label, *box;
-	GtkWidget *pixmap = NULL;
+	GtkWidget *pixmap;
+	gchar *theme, *filename;
 	const ToolbarItem *item;
 
 	/* get item from array */
 	item = &(toolbar_items[type]);
 	
-	/* which icons to use */
-	toolbar_icons = gnome_config_get_int (CONF_APPEARANCE_TOOLBAR_ICONS);
+	pixmap = NULL;
+	
+	/* which theme to use */
+	theme = eel_gconf_get_string (CONF_TOOLBAR_THEME_DIR);
 
-	/* try getting large icon */
-	if (toolbar_icons == TOOLBAR_ICONS_LARGE)
+	/* get default nautilus theme when needed */
+	if (strcmp (theme, "NAUTILUS_THEME") == 0)
 	{
-		pixmap = toolbar_get_theme_pixmap (item->nautilus_icon, 
-						   FALSE);
+		g_free (theme);
+		theme = toolbar_get_nautilus_theme_path ();
 	}
 
-	/* try getting nautilus button */
-	if (toolbar_icons == TOOLBAR_ICONS_NAUTILUS)
+	if (theme && item->theme_icon)
 	{
-		pixmap = toolbar_get_theme_pixmap (item->nautilus_icon, 
-						   TRUE);
+		filename = g_strconcat (theme, "/", item->theme_icon, ".png", 
+					NULL);
+		pixmap = toolbar_get_theme_pixmap (filename);
+		g_free (filename);
 	}
 
 	/* use GNOME stock icon if not found or not requested */
 	if (pixmap == NULL)
 	{
-		if (toolbar_icons == TOOLBAR_ICONS_LARGE)
+		GdkPixbuf *pixbuf;
+		gint height = 24;
+
+		filename = NULL;
+
+		/* we take Refresh because this is the highest icon in 
+		 * nautilus themes... nautilus icons haven't got the same 
+		 * height. Also, we only take height because they aren't
+		 * square either :-/ */
+		if (theme)
+			filename = g_concat_dir_and_file (theme, "Refresh.png");
+		if (filename && g_file_exists (filename))
+		{
+			pixbuf = gdk_pixbuf_new_from_file (filename);
+			height = gdk_pixbuf_get_height (pixbuf);
+			gdk_pixbuf_unref (pixbuf);
+		}
+		if (filename) g_free (filename);
+		
+		if (height != 24)
 		{ 
-			pixmap = gnome_stock_pixmap_widget_at_size
-				(window->WMain, item->stock_icon, 36, 36);
+			pixmap = gnome_stock_pixmap_widget_at_size 
+				 (window->WMain, item->stock_icon, height, 
+				  height); /* GNOME stock icons are square */
 		}
 		else
 		{
@@ -854,8 +859,10 @@ toolbar_create_button (GaleonWindow *window, GtkWidget *toolbar,
 		}
 	}
 
+	if (theme) g_free (theme);
+
 	/* make the toolbar button and add it to the toolbar */
-	if (gnome_config_get_int (CONF_APPEARANCE_TOOLBAR_STYLE) ==
+	if (eel_gconf_get_integer (CONF_TOOLBAR_STYLE) ==
 	    TOOLBAR_STYLE_HORIZONTAL)
 	{
 		/* we create a horizontal toolbar style ourselves */
@@ -962,9 +969,6 @@ toolbar_create_button (GaleonWindow *window, GtkWidget *toolbar,
 	case TOOLBAR_ITEM_GO:
 		window->go_button = button;
 		break;
-	case TOOLBAR_ITEM_FIND:
-		window->find_button = button;
-		break;
 	case TOOLBAR_ITEM_HISTORY:
 		window->history_button = button;
 		break;
@@ -972,6 +976,7 @@ toolbar_create_button (GaleonWindow *window, GtkWidget *toolbar,
 		window->bookmarks_button = button;
 		break;
 	default:
+		break;
 	}
 }
 
@@ -992,8 +997,8 @@ toolbar_add_docked (GaleonWindow *window, GtkWidget *hbox,
 	toolbar_relief = gnome_preferences_get_toolbar_relief ();
 	relief = (toolbar_relief ? GTK_SHADOW_OUT : GTK_SHADOW_NONE);
 	toolbar_detachable = gnome_preferences_get_toolbar_detachable ();
-	location_on_main_toolbar = gnome_config_get_bool 
-		(CONF_APPEARANCE_URL_LOCATION);
+	location_on_main_toolbar = eel_gconf_get_boolean 
+		(CONF_TOOLBAR_URL_LOCATION);
 
 	/* default properties */
 	props = (GNOME_DOCK_ITEM_BEH_EXCLUSIVE |
@@ -1039,87 +1044,21 @@ toolbar_set_style (GtkWidget *toolbar, GtkToolbarStyle final_style,
 	gtk_toolbar_set_button_relief (GTK_TOOLBAR (toolbar), relief);
 }
 
-/**
- * toolbar_theme_use_standard_icons: FIXME
- */
-static gboolean 
-toolbar_theme_use_standard_icons (void)
-{
-	xmlDocPtr doc;
-	xmlNodePtr item;
-	char *filename;
-	gchar *icon_theme;
-	gboolean result = FALSE;
-
-	filename = g_strconcat (theme_path, theme_name, ".xml", NULL);
-	if (g_file_exists (filename))
-	{
-		doc = xmlParseFile (filename);
-		g_return_val_if_fail (doc != NULL, FALSE);
-		g_return_val_if_fail (doc->root != NULL, FALSE);
-		item = doc->root->childs;
-		while (item != NULL)
-		{
-			if (!strcmp (item->name, "toolbar"))
-			{
-				icon_theme = xmlGetProp (item, "icon_theme");
-				result = (strcmp (icon_theme, "standard")==0);
-				xmlFree (icon_theme);
-			}
-			item = item->next;
-		}
-		xmlFreeDoc (doc);
-	}
-
-	g_free (filename);
-	return result;
-}
-
 /**
  * toolbar_get_theme_pixmap: FIXME
  */
 static GtkWidget *
-toolbar_get_theme_pixmap (const gchar *pixmap, gboolean is_nautilus)
+toolbar_get_theme_pixmap (const gchar *pixmap)
 {
-	gchar *filename;
 	PixmapData *icon;
 	GtkWidget *widget;
 
-	if (is_nautilus)
+	if (!g_file_exists (pixmap))
 	{
-		filename = g_strconcat (theme_path, pixmap, ".png", NULL);
-	}
-	else
-	{
-		filename = g_strconcat (SHARE_DIR, "/", pixmap, ".png", NULL);
+		return NULL;
 	}
 
-	if (!g_file_exists (filename) || pixmap == NULL)
-	{
-		gchar *dir; 
-
-		g_free (filename);
-	
-		if (is_nautilus)
-		{
-			/* fall back to the default nautilus icon */
-			dir = gnome_datadir_file("pixmaps/nautilus/");
-			filename = g_strconcat (dir, pixmap, ".png", NULL);
-			g_free (dir);
-			if (!g_file_exists (filename))
-			{
-				g_free (filename);
-				return NULL;
-			}
-		}
-		else
-		{
-			return NULL;
-		}
-	}
-
-	icon = pixmap_data_new_from_file (filename);
-	g_free (filename);
+	icon = pixmap_data_new_from_file (pixmap);
 
 	if (icon != NULL)
 	{
@@ -1134,71 +1073,65 @@ toolbar_get_theme_pixmap (const gchar *pixmap, gboolean is_nautilus)
 }
 
 /**
- * toolbar_get_theme_path: FIXME
- */
+ * toolbar_get_nautilus_theme_path: get the path of the 
+ * current nautilus theme
+*/
 static char *
-toolbar_get_theme_path (void)
+toolbar_get_nautilus_theme_path (void)
 {
-	extern GConfClient *gconf_client;
-	char *aa_pixmap_path;
-	char *pixmap_path;
+	int i;
 	char *theme;
-	char *dir;
+	char *icon, *tmp;
+	char *result = NULL;
 
-	theme = gconf_client_get_string (gconf_client, 
-					 NAUTILUS_THEME_PREF, NULL);
-	if (!theme)
+	static gchar *path[] =
 	{
+		SHARE_DIR "/../pixmaps/nautilus/%s",
+		SHARE_DIR "/../pixmaps/nautilus/%s/fake-aa-navibar",
+		NULL,
+		NULL,
+		SHARE_DIR "/../pixmaps/nautilus/",
+		NULL
+	};
+
+	theme = eel_gconf_get_string(CONF_NAUTILUS_THEME);
+
+	if (!theme || 
+	    strcmp (theme, "default") == 0)
+	{
+		g_free (theme);
+		theme = g_strdup ("");
+	}
+	else
+	if (strcmp (theme, "gnome") == 0) /* use standard icons */
+	{
+		g_free (theme);
 		return NULL;
 	}
 
-	theme_name = theme;
+	path[2] = g_strconcat (g_get_home_dir (), 
+			       "/.nautilus/themes/%s", NULL);
+	path[3] = g_strconcat (g_get_home_dir (), 
+			       "/.nautilus/themes/fake-aa-navibar/%s",
+			       NULL);
 
-	dir = gnome_datadir_file ("pixmaps/nautilus/");
+	for (i = 0; path[i] != NULL && !result; i++)
+	{
+		tmp = g_strdup_printf (path[i], theme);
+		icon = g_strconcat (tmp, "/Refresh.png", NULL);
 
-	if (strcmp (theme_name, "default") == 0)
-	{
-		pixmap_path = g_strdup (dir);
-	} 
-	else
-	{
-		pixmap_path = g_strconcat (dir, theme, "/", NULL);
-	}
-	g_free (dir);
-
-	aa_pixmap_path = g_concat_dir_and_file (pixmap_path, 
-						"fake-aa-navibar/");
-	if (g_file_exists (aa_pixmap_path))
-	{
-		g_free (pixmap_path);
-		return aa_pixmap_path;
-	}
-	else
-	{
-		g_free (aa_pixmap_path);
-	}
-	
-	if (g_file_exists (pixmap_path))
-	{
-		return pixmap_path;
-	}
-	else
-	{
-		g_free (pixmap_path);
+		if (g_file_exists (icon))
+		{
+			result = g_strdup (tmp);
+		}
+		g_free (tmp);
+		g_free (icon);
 	}
 
-	/* try with the user directory */
-	/* FIXME: look for AA path here too */
-	pixmap_path = g_strconcat (g_get_home_dir (), "/.nautilus/themes/", 
-		theme, "/", NULL);
-	if (g_file_exists (pixmap_path))
-	{
-		return pixmap_path;
-	}
-	else
-	{
-		g_free (pixmap_path);
-	}
+	g_free (path [2]);
+	g_free (path [3]);
+	g_free (theme);
 
-	return NULL;
+	return result;
 }
+
