@@ -33,6 +33,7 @@
 #include "filepicker.h"
 #include "mozilla_i18n.h"
 #include "spinner.h"
+#include "window_notifiers.h"
 
 #include <string.h>
 #include <time.h>
@@ -56,6 +57,7 @@
 /* indicates a window is in fullscreen mode */
 gboolean fullscreen_active = FALSE;
 
+
 /* local function prototypes */
 static void window_init_data (GaleonWindow *window);
 static void window_set_menu_data (GaleonWindow *window);
@@ -74,7 +76,7 @@ static GtkWidget *window_notebook_tab_new (GaleonEmbed *embed,
 static GtkWidget *window_session_history_menuitem (gchar *file, gint index);
 static void window_session_history_menu_remove (GaleonWindow *window);
 
-/* clallbacks used here */
+/* callbacks used here */
 void file_open_ok_button_clicked_cb (const gchar *file, gpointer data);
 
 /* the types which can be dropped either into the GtkMozEmbed or onto
@@ -166,13 +168,6 @@ window_create (guint32 chrome_mask)
 	gchar *file;
 	gboolean ignore_layout;
 
-	const GtkTargetEntry window_drop_types[] =
-	{
-		{ "GALEON_EMBED", 0, DND_TARGET_GALEON_EMBED }
-	};
-	const gint window_drop_types_num_items = (sizeof (window_drop_types) /
-						  sizeof (GtkTargetEntry));
-
 	/* allocate and initialise the GaleonWindow structure */
 	window = g_new0 (GaleonWindow, 1);
 
@@ -207,6 +202,10 @@ window_create (guint32 chrome_mask)
 	window->dock = GNOME_APP (window->WMain)->dock;
 	window->dock_type = DOCK_NONE;
 
+	/* clear drag signals */
+	window->drag_motion_signal = 0;
+	window->drag_release_signal = 0;
+
 	/* save the toolbar and menubar layout */
 	if (!ignore_layout)
 	{
@@ -225,6 +224,9 @@ window_create (guint32 chrome_mask)
 		window_menubar_create (window);
 	}
 
+	/* add notifiers for the window */
+	window_init_notifiers(window);
+
 	/* restore the toolbar and menubar layout */
 	if (!ignore_layout)
 	{
@@ -248,14 +250,9 @@ window_create (guint32 chrome_mask)
 			   GTK_SIGNAL_FUNC(window_delete_cb),
 			   window);
 
-	/* set up DnD stuff */
-	gtk_drag_dest_set (window->WMain, GTK_DEST_DEFAULT_ALL,
-			   window_drop_types, window_drop_types_num_items,
-			   GDK_ACTION_COPY | GDK_ACTION_MOVE |
-			   GDK_ACTION_LINK | GDK_ACTION_ASK);
-	gtk_signal_connect (GTK_OBJECT (window->WMain), "drag_data_received",
-			    GTK_SIGNAL_FUNC(window_drag_data_received_cb),
-			    window);
+	gtk_signal_connect(GTK_OBJECT(window->WMain), "key-press-event",
+			   GTK_SIGNAL_FUNC(window_key_press_event),
+			   window);
 
 	/* NB: window isn't shown until we add an embed */
 	/* don't know the size at which to show it yet -- if we discover
@@ -299,7 +296,7 @@ window_init_menu_data (GaleonWindow *window)
 		GTK_MENU_ITEM (menubar_uiinfo[5].widget)->submenu;
 	window->go_menu = 
 		GTK_MENU_ITEM (menubar_uiinfo[6].widget)->submenu;
-
+		
 	/* file menu */
 	window->file_session_history_separator_sup = NULL; // not yet created
 	window->file_session_history_separator_inf =
@@ -313,16 +310,18 @@ window_init_menu_data (GaleonWindow *window)
 	/* view menu */
 	window->view_menubar = view_menu_uiinfo[0].widget;
 	window->view_toolbar = view_menu_uiinfo[1].widget;
-	window->view_statusbar = view_menu_uiinfo[2].widget;
-	window->view_fullscreen = view_menu_uiinfo[3].widget;
-	window->encoding = view_menu_uiinfo[6].widget;
+	window->view_bookmarks = view_menu_uiinfo[2].widget;
+	window->view_statusbar = view_menu_uiinfo[3].widget;
+	window->view_fullscreen = view_menu_uiinfo[4].widget;
+	window->encoding = view_menu_uiinfo[7].widget;
 
 	/* tab menu */
 	window->detach_tab = tab_menu_uiinfo[0].widget;
-	window->prev_tab = tab_menu_uiinfo[2].widget;
-	window->next_tab = tab_menu_uiinfo[3].widget;
-	window->move_tab_left = tab_menu_uiinfo[5].widget;
-	window->move_tab_right = tab_menu_uiinfo[6].widget;
+	window->move_tab_to_window_menu = tab_menu_uiinfo[1].widget;
+	window->prev_tab = tab_menu_uiinfo[3].widget;
+	window->next_tab = tab_menu_uiinfo[4].widget;
+	window->move_tab_left = tab_menu_uiinfo[6].widget;
+	window->move_tab_right = tab_menu_uiinfo[7].widget;
 
 	/* settings menu */
 	window->load_images_always = load_images_always_uiinfo[0].widget;
@@ -342,10 +341,19 @@ window_init_menu_data (GaleonWindow *window)
 	window->autojump = settings_menu_uiinfo[8].widget;
 
 	/* bookmarks menu */
-	window->add_bookmark = bookmarks_menu_uiinfo[0].widget;
+	window->add_bookmark = bookmarks_menu_uiinfo[1].widget;
 	window->bookmarks_separator = bookmarks_menu_uiinfo[3].widget;
 
 	window->open_url_entry = NULL;
+
+	/* connect signal for the move_tab_to_window_menu */
+	gtk_signal_connect (GTK_OBJECT (window->move_tab_to_window_menu),
+			    "activate", window_move_tab_to_window_menu_cb,
+			    window);
+	gtk_menu_item_set_submenu
+		(GTK_MENU_ITEM (window->move_tab_to_window_menu),
+		 gtk_menu_new ());
+
 }
 
 static void
@@ -376,6 +384,8 @@ window_menubar_create (GaleonWindow *window)
 		(GTK_CHECK_MENU_ITEM (window->view_menubar), TRUE);
 	gtk_check_menu_item_set_active 
 		(GTK_CHECK_MENU_ITEM (window->view_toolbar), TRUE);
+	gtk_check_menu_item_set_active 
+		(GTK_CHECK_MENU_ITEM (window->view_bookmarks), TRUE);
 	gtk_check_menu_item_set_active 
 		(GTK_CHECK_MENU_ITEM (window->view_statusbar), TRUE);
 
@@ -397,16 +407,19 @@ window_menubar_create (GaleonWindow *window)
 			    GTK_SIGNAL_FUNC (window_menu_edit_expose_event_cb),
 			    window);
 
-	/* setup view menu */
-	set_settings_menu_window (window);
+	/* setup view menu and install notifiers*/
+	window_menubar_init_notifiers (window);
 }
 
 static void
 window_toolbars_create (GaleonWindow *window)
 {
 	/* create the toolbars */
-	toolbar_create (window);
+	toolbar_create (window, TRUE);
 	bookmarks_toolbars_create (window);
+
+	/* add notifiers for the toolbars */
+	window_toolbars_init_notifiers(window);
 }
 
 void
@@ -473,28 +486,6 @@ window_restore_layout (GaleonWindow *window)
 	window->layout = NULL;
 }
 
-void
-window_toolbars_recreate (GaleonWindow *window)
-{
-	/* skip this window if no toolbars */
-	if (!(window->toolBarOn))
-	{
-		return;
-	}
-
-	/* save layout of window */
-	window_save_layout (window);
-	
-	/* recreate the main toolbar */
-	toolbar_recreate (window);
-
-	/* recreate the bookmarks toolbars for this window */
-	bookmarks_toolbars_recreate (window);
-
-	/* restore the layout */
-	window_restore_layout (window);
-}
-
 static void
 window_statusbar_create (GaleonWindow *window)
 {
@@ -537,13 +528,22 @@ window_create_embed_container (GaleonWindow *window)
 	gtk_notebook_popup_enable (GTK_NOTEBOOK (window->notebook));
 	gtk_notebook_set_scrollable (GTK_NOTEBOOK (window->notebook), TRUE);
 	gtk_signal_connect_after (GTK_OBJECT (window->notebook), "switch_page",
-				  embed_notebook_switch_page_cb, window);
+				  window_notebook_switch_page_cb, window);
 	gtk_signal_connect_after (GTK_OBJECT (window->notebook), "add",
-				  embed_notebook_add_remove_page_cb, window);
+				  window_notebook_add_remove_page_cb, window);
 	gtk_signal_connect_after (GTK_OBJECT (window->notebook), "remove",
-				  embed_notebook_add_remove_page_cb, window);
+				  window_notebook_add_remove_page_cb, window);
+
+	/* connect signal for tab drag events */
+	/* this causes crashes sometimes - Dan
+	gtk_signal_connect_after (GTK_OBJECT (window->notebook),
+				  "button_press_event", GTK_SIGNAL_FUNC
+				  (window_notebook_button_press_cb), window);
+				  */
+			    
 	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (window->notebook),
-		gnome_config_get_int (CONF_APPEARANCE_TABBED_POSITION));
+				  eel_gconf_get_integer 
+				  (CONF_TABS_TABBED_POSITION));
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (window->notebook), FALSE);
 	gtk_widget_show (GTK_WIDGET (window->notebook));
 
@@ -739,7 +739,21 @@ window_update_temp_message(GaleonWindow *window, const char *message)
 	/* fill the new temporary message */
 	if (message != NULL) 
 	{
+		char *tmp;
+
 		window->tempMessage = g_strdup (message);
+
+		/* remove \n or the statusbar
+		   height will grow */
+		tmp = window->tempMessage;
+		while (*tmp != '\0')
+		{
+			if (*tmp == '\n')
+			{
+				*tmp = ' ';
+			}
+			tmp ++;
+		}
 	}	
 	else
 	{	
@@ -786,7 +800,7 @@ window_update_tab_controls (GaleonWindow *window)
 	}
 
 	/* update tab display */
-	show_tabs = gnome_config_get_bool (CONF_APPEARANCE_TABBED_ALWAYS_SHOW);
+	show_tabs = eel_gconf_get_boolean (CONF_TABS_TABBED_ALWAYS_SHOW);
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->notebook),
 				    (show_tabs && !window->is_popup) ||
 				    (size > 1));
@@ -905,11 +919,11 @@ void window_show_open_dialog (GaleonWindow *window)
 	gchar *url;
 	GnomeVFSURI *uri;
 
-	dir = gnome_config_get_string (CONF_STATE_OPEN_DIR);
+	dir = eel_gconf_get_string (CONF_STATE_OPEN_DIR);
 
 	if (show_file_picker (window->WMain, _("Select the file to open:"),
 			      dir, NULL, modeOpen,
-			      &file))
+			      &file, NULL))
 	{
 		uri = gnome_vfs_uri_new (file);
 		if (uri)
@@ -922,9 +936,8 @@ void window_show_open_dialog (GaleonWindow *window)
 			g_free (url);
 
 			/* set default open dir */
-			gnome_config_set_string (CONF_STATE_OPEN_DIR, 
+			eel_gconf_set_string (CONF_STATE_OPEN_DIR, 
 						 retDir);
-			gnome_config_sync ();
 
 			g_free (retDir);
 			gnome_vfs_uri_unref (uri);
@@ -982,11 +995,10 @@ void window_toolbar_show (GaleonWindow *window)
 	}
 
 	gtk_widget_show (window->main_dockitem);	
-	if (!gnome_config_get_bool (CONF_APPEARANCE_URL_LOCATION))
+	if (!eel_gconf_get_boolean (CONF_TOOLBAR_URL_LOCATION))
 	{
 		gtk_widget_show (window->location_dockitem);
 	}
-	bookmarks_toolbars_set_visibility (window, TRUE);
 }
 
 /**
@@ -1001,7 +1013,6 @@ void window_toolbar_hide (GaleonWindow *window)
 		{
 			gtk_widget_hide (GTK_WIDGET (window->location_dockitem));
 		}
-		bookmarks_toolbars_set_visibility (window, FALSE);
 		gtk_widget_queue_resize (GTK_WIDGET (window->hbox));
 	}
 }
@@ -1061,6 +1072,9 @@ window_close (GaleonWindow *window)
 
 	return_if_not_window (window);
 
+	/* hide the window, this makes the app seem more responsive */
+	gtk_widget_hide (window->WMain);
+
 	/* close any dock (in order to store width of dock) */
 	window_undock (window);
 
@@ -1069,15 +1083,12 @@ window_close (GaleonWindow *window)
 	if (!window->is_popup &&
 	    !GTK_CHECK_MENU_ITEM (window->view_fullscreen)->active)
 	{	
-		gnome_config_set_int("/galeon/Appearance/winwidth", 
+		eel_gconf_set_integer(CONF_STATE_WINWIDTH, 
 				     window->WMain->allocation.width);
-		gnome_config_set_int("/galeon/Appearance/winheight", 
+		eel_gconf_set_integer(CONF_STATE_WINHEIGHT, 
 				     window->WMain->allocation.height);
 	}
 
-	/* hide the window, this makes the app seem more responsive */
-	gtk_widget_hide (window->WMain);
-
 	/* free any temporary message */
 	if (window->tempMessage != NULL)
 	{
@@ -1105,6 +1116,12 @@ window_close (GaleonWindow *window)
 		window->bookmarks_tooltips = NULL;
 	}
 
+	/* remove notifiers */
+	if (window->notifiers_id != NULL);
+	{
+		galeon_notification_remove(&window->notifiers_id);
+	}
+
 	/* any children? */
 	if (window->embed_list != NULL)
 	{
@@ -1127,12 +1144,11 @@ window_close (GaleonWindow *window)
 		/* remove window from session */
 		session_remove_window (window);
 
-		/* removed since this causes crash at exit... ??? -- MattA */
-#if 0
 		/* scrub and free the window structure */
-		memset (window, 0, sizeof (GaleonWindow));
-		g_free (window);
-#endif
+		/* FIXME doesnt seem to crash anymore, try to
+		   remove comments soon */
+/*		memset (window, 0, sizeof (GaleonWindow));
+		g_free (window);*/
 	}
 }
 
@@ -1144,8 +1160,8 @@ window_go_home (GaleonWindow *window, LinkState state)
 {
 	gchar *startpage;
 
-	startpage = gnome_config_get_string 
-		("/galeon/General/startpage=www.gnome.org");
+	startpage = eel_gconf_get_string 
+		(CONF_GENERAL_HOMEPAGE);
 
         if (startpage != NULL && strlen(startpage) != 0)
         {
@@ -1172,7 +1188,7 @@ window_set_layer (GaleonWindow *window)
 
 	/* check if we're in fullscreen mode and stacked above dock */
 	if (fullscreen_active && 
-	    gnome_config_get_bool (CONF_APPEARANCE_FULLSCREEN_STACKING))
+	    eel_gconf_get_boolean (CONF_WINDOWS_FULLSCREEN_STACKING))
 	{
 		/* get to the top */
 		gnome_win_hints_set_layer (window->WMain, 
@@ -1207,22 +1223,29 @@ window_set_fullscreen_mode (GaleonWindow *window, gboolean active)
 	{
 		GTK_CHECK_MENU_ITEM (window->view_fullscreen)->active = active;
 
-		if (!gnome_config_get_bool
-		    ("/galeon/Appearance/show_menubar_in_fullscreen")) 
+		if (!eel_gconf_get_boolean
+		    (CONF_WINDOWS_FS_SHOW_MENUBAR)) 
 		{
 			gtk_check_menu_item_set_active
 				(GTK_CHECK_MENU_ITEM (window->view_menubar),
 				 !active);
 		}
-		if (!gnome_config_get_bool
-		    ("/galeon/Appearance/show_toolbars_in_fullscreen")) 
+		if (!eel_gconf_get_boolean
+		    (CONF_WINDOWS_FS_SHOW_TOOLBARS)) 
 		{
 			gtk_check_menu_item_set_active
 				(GTK_CHECK_MENU_ITEM (window->view_toolbar),
 				 !active);
 		}
-		if (!gnome_config_get_bool
-		    ("/galeon/Appearance/show_statusbar_in_fullscreen"))
+		if (!eel_gconf_get_boolean
+		    (CONF_WINDOWS_FS_SHOW_BOOKMARKS)) 
+		{
+			gtk_check_menu_item_set_active
+				(GTK_CHECK_MENU_ITEM (window->view_bookmarks),
+				 !active);
+		}
+		if (!eel_gconf_get_boolean
+		    (CONF_WINDOWS_FS_SHOW_STATUSBAR))
 		{
 			gtk_check_menu_item_set_active
 				(GTK_CHECK_MENU_ITEM (window->view_statusbar),
@@ -1348,9 +1371,8 @@ window_undock (GaleonWindow *window)
 	if (window->docked_width_pref != NULL && 
 	    GTK_PANED (window->hpaned)->position_set)
 	{
-		gnome_config_set_int (window->docked_width_pref,
-				      GTK_PANED (window->hpaned)->child1_size);
-		gnome_config_sync ();
+	  eel_gconf_set_integer (window->docked_width_pref,
+				 GTK_PANED (window->hpaned)->child1_size);
 	}
 
 	/* hide the hpaned and notebook */
@@ -1405,7 +1427,7 @@ window_dock (GaleonWindow *window, GtkWidget *widget, const gchar *width_pref)
 	/* move pane to accomodate width */
 	if (width_pref != NULL)
 	{
-		width = gnome_config_get_int (width_pref);
+		width = eel_gconf_get_integer (width_pref);
 		if (width != -1)
 		{
 			gtk_paned_set_position (GTK_PANED (window->hpaned),
@@ -1507,7 +1529,7 @@ window_add_embed (GaleonWindow *window, GaleonEmbed *embed,
 	/* add as a tab into the notebook */
 	tab = window_notebook_tab_new (embed, embed->site_title);
 	if ((g_list_length (window->embed_list) > 1) &&
-	     gnome_config_get_bool (CONF_APPEARANCE_TABBED_INSERT_NEW_TABS))
+	     eel_gconf_get_boolean (CONF_TABS_TABBED_INSERT_NEW_TABS))
 	{
 		page = gtk_notebook_current_page 
 			(GTK_NOTEBOOK (window->notebook));
@@ -1531,7 +1553,7 @@ window_add_embed (GaleonWindow *window, GaleonEmbed *embed,
 	}
 
 	/* switch to page if necessary */
-	auto_jump = gnome_config_get_bool (CONF_APPEARANCE_TABBED_AUTOJUMP);
+	auto_jump = eel_gconf_get_boolean (CONF_TABS_TABBED_AUTOJUMP);
 	if (auto_jump || force_jump)
 	{
 		gtk_widget_show (embed->mozEmbed);
@@ -1566,7 +1588,7 @@ window_show_one_embed (GaleonWindow *window)
 {
 	gint width, height;
 	
-	/* keep count */
+	/* keepcount */
 	window->visible_embeds++;
 
 	/* if this isn't the first time this has been shown, skip the rest */
@@ -1579,8 +1601,8 @@ window_show_one_embed (GaleonWindow *window)
 	if (!window->set_size)
 	{
                 /* use system defaults */
-		width = gnome_config_get_int (CONF_APPEARANCE_WINWIDTH);
-		height = gnome_config_get_int (CONF_APPEARANCE_WINHEIGHT);
+		width = eel_gconf_get_integer (CONF_STATE_WINWIDTH);
+		height = eel_gconf_get_integer (CONF_STATE_WINHEIGHT);
 		gtk_window_set_default_size (GTK_WINDOW (window->WMain), 
 					     width, height);
 	}
@@ -1592,51 +1614,22 @@ window_show_one_embed (GaleonWindow *window)
 static GtkWidget *
 window_notebook_tab_new (GaleonEmbed *embed, const gchar *title)
 {
-	GtkWidget *event_box;
 	GtkWidget *label;
 	GtkWidget *hbox;
 
-	static const GtkTargetEntry tab_drag_types[] =
-	{
-		{ "GALEON_EMBED", 0, DND_TARGET_GALEON_EMBED }
-	};
-	static const gint tab_drag_types_num_items = (sizeof (tab_drag_types) /
-						      sizeof (GtkTargetEntry));
-
 	/* build the basic widgets */
 	embed->notebook_hbox = hbox = gtk_hbox_new (FALSE, 0);
-	embed->notebook_event_box = event_box = gtk_event_box_new ();
 	embed->notebook_label = label = gtk_label_new (NULL);
 
 	/* setup label */
 	gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.5);
 	gtk_misc_set_padding (GTK_MISC (label), 4, 0);
-	gtk_container_add (GTK_CONTAINER (event_box), label);
-
-	/* setup event box */
-	gtk_drag_source_set (event_box, GDK_BUTTON1_MASK, tab_drag_types,
-			     tab_drag_types_num_items, GDK_ACTION_COPY);
-	gtk_signal_connect (GTK_OBJECT (event_box), "drag_data_get",
-			    GTK_SIGNAL_FUNC (embed_tab_drag_data_get_cb),
-			    embed);
-	gtk_signal_connect (GTK_OBJECT (event_box), "button-press-event",
-			    GTK_SIGNAL_FUNC (embed_tab_button_press_event_cb),
-			    embed);
-	gtk_box_pack_start (GTK_BOX (hbox), event_box, TRUE, TRUE, 0);
-
-	/* setup hbox */
-	gtk_drag_dest_set (hbox, GTK_DEST_DEFAULT_ALL,
-			   embed_drop_types, embed_drop_types_num_items,
-			   (GDK_ACTION_COPY | GDK_ACTION_MOVE |
-			    GDK_ACTION_LINK | GDK_ACTION_ASK));
-	gtk_signal_connect (GTK_OBJECT (hbox), "drag_data_received",
-			    GTK_SIGNAL_FUNC(embed_drag_data_received_cb),
-			    embed);
+	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
 
 	/* set page title and add button */
-	embed_set_notebook_label (embed);
-	embed_set_notebook_label_status (embed);
-	embed_update_tab_closebutton (embed);
+	embed_update_tab_title (embed);
+	embed_update_tab_close_button (embed);
+	embed_update_tab_status (embed);
 
 	/* show everything */
 	gtk_widget_show_all (hbox);
@@ -1690,7 +1683,8 @@ window_session_history_menu_create (GaleonWindow *window)
 	g_list_free (l);
 }
 
-static void window_session_history_menu_remove (GaleonWindow *window)
+static void 
+window_session_history_menu_remove (GaleonWindow *window)
 {
 	GList *l;
 	GList *li;
