@@ -23,6 +23,7 @@ extern "C" {
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkeditable.h>
 #include <gdk/gdktypes.h>
+#include <gtk/gtknotebook.h>
 
 /* window.c */
 GaleonWindow *window_create (guint32 chrome_mask);
@@ -154,13 +155,6 @@ void window_location_gnomeentry_popwin_cb (GtkWidget *widget,
 gboolean window_generic_button_press_event_cb (GtkButton *button, 
 					       GdkEventButton *event,
 					       GaleonWindow *window);
-void window_drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
-				   gint x, gint y,
-				   GtkSelectionData *selection_data,
-				   guint info, guint time,
-				   GaleonWindow *window);
-void window_find_button_clicked_cb (GtkButton *button,
-				    GaleonWindow *window);
 void window_history_button_clicked_cb (GtkButton *button, 
 				       GaleonWindow *window);
 void window_bookmarks_button_clicked_cb (GtkButton *button, 
@@ -168,7 +162,24 @@ void window_bookmarks_button_clicked_cb (GtkButton *button,
 void window_print_button_clicked_cb (GtkButton *button, GaleonWindow *window);
 void window_session_history_menuitem_activate_cb (GtkMenuItem *menuitem, 
 						  gint index);
-
+void window_notebook_switch_page_cb (GtkNotebook *notebook, 
+				     GtkNotebookPage *page, guint page_num);
+void window_notebook_add_remove_page_cb (GtkContainer *container,
+					 GtkWidget *widget, 
+					 GaleonWindow *window);
+gint window_notebook_button_press_cb (GtkWidget *widget, GdkEventButton *e,
+				      GaleonWindow *window);
+gint window_notebook_motion_cb (GtkWidget *widget, GdkEventMotion *e,
+				GaleonEmbed *embed);
+gint window_notebook_button_release_cb (GtkWidget *widget, GdkEventButton *e,
+					GaleonEmbed *embed);
+gboolean window_key_press_event (GtkWidget *widget,
+				 GdkEventKey *event,
+				 GaleonWindow *window);
+void window_move_tab_to_window_menu_cb (GtkMenuItem *menu, 
+					GaleonWindow *window);
+gboolean window_find_entry_activate_cb (GtkWidget *widget, 
+				        GaleonWindow *window);
 #ifdef __cplusplus
 }
 #endif
