@@ -23,6 +23,7 @@
 #include "autocompletion.h"
 #include "history.h"
 #include "bookmarks.h"
+#include "prefs.h"
 #include "mozilla.h"
 #include "context.h"
 #include "toolbar.h"
@@ -48,10 +49,14 @@ void window_entry_changed_cb (GtkEditable *editable, gboolean *changed);
 void window_location_gnomeentry_popwin_cb (GtkWidget *widget, 
 					   GaleonWindow *window);
 static void window_handle_go_button (GaleonWindow *window, LinkState state);
+static void window_move_tab_to_window_menu_item_cb (GtkMenuItem *mi, 
+						   GaleonWindow *w);
 
 GTimer *zoom_timer = NULL;
 #define ZOOM_DELAY 0.20
 
+static GdkCursor *drag_cursor = NULL;
+
 /**
  * mozembed_selection_get_cb: get selection on copy link location
  * FIXME HYBRID: move this elsewhere
@@ -226,9 +231,8 @@ window_home_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 		if (action == 0 && embed->site_location != NULL &&
 		    strlen (embed->site_location) != 0)
 		{
-			gnome_config_set_string (CONF_GENERAL_HOMEPAGE,
+			eel_gconf_set_string (CONF_GENERAL_HOMEPAGE,
 						 embed->site_location);
-			gnome_config_sync ();
 		}
 
 		/* destroy the popup menu */
@@ -304,9 +308,9 @@ window_new_button_release_cb (GtkButton *button, GdkEventButton *event,
 	embed = window->active_embed;
 	return_val_if_not_embed (embed, FALSE);
 
-	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED) ? 1 : 0;
+	tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED) ? 1 : 0;
 	shift_modifier = (event->state & GDK_SHIFT_MASK) ? 0 : 1;
-	
+
 	switch (event->button)
 	{
 	case 1:
@@ -356,7 +360,7 @@ window_new_button_drag_data_received_cb (GtkWidget *widget,
 	return_if_not_embed (window->active_embed);
 
 	/* get state */
-	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
+	tabbed_mode = eel_gconf_get_boolean (CONF_TABS_TABBED);
 
 	/* check drop data */
 	switch (info)
@@ -626,6 +630,7 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 				    GaleonWindow *window)
 {
 	static gchar *before_completion = NULL;
+	static gboolean suggest = FALSE;
 	GtkEntry *entry = GTK_ENTRY (widget);
 	GtkEditable *editable = GTK_EDITABLE (widget);
 	gboolean url_dialog = FALSE;
@@ -639,6 +644,10 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 
 	auto_completion_reset();
 
+	/* only suggest heuristic completions if TAB is hit twice */
+	if (event->keyval != GDK_Tab)
+		suggest = FALSE;
+
 	/* don't process the standard GtkEntry editing shortcuts */
 	if (event->keyval > GDK_A && event->keyval < GDK_Z)
 		keyval = event->keyval + 0x20;
@@ -704,7 +713,9 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 
 		gtk_editable_delete_selection (editable);
 		text = gtk_editable_get_chars (editable, 0, -1);
-		common_prefix = auto_completion_complete_url_extended (text);
+		common_prefix = 
+			auto_completion_complete_url_extended (text, suggest);
+		suggest = FALSE;
 		if (common_prefix)
 		{
 			if (!before_completion) 
@@ -712,6 +723,11 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 
 			gtk_entry_set_text (entry, common_prefix);
 			auto_completion_display_alternatives (window, widget);
+			if (!strcmp (common_prefix, text))
+			{
+				/* really suggest something the next time */
+				suggest = TRUE; 
+			}
 			g_free (common_prefix);
 		}
 		return TRUE;
@@ -800,7 +816,6 @@ window_handle_go_button (GaleonWindow *window, LinkState state)
 		/* forcibly save contents of entry */
 		ge = GNOME_ENTRY (window->location_gnomeentry);
 		gnome_entry_save_history (ge);
-		gnome_config_sync ();
 
 		/* add item to location bars in all other
 		 * open browser windows */
@@ -853,7 +868,6 @@ window_go_button_release_cb (GtkButton *button, GdkEventButton *event,
 	/* open location in correct place */
 	window_handle_go_button (window,
 		mouse_state_to_link_state (event->button, event->state));
-
 	return TRUE;
 }
 
@@ -1157,43 +1171,6 @@ window_progress_action (GaleonWindow *window)
 	return TRUE;
 }
 
-/**
- * window_find_button_clicked_cb: Find button on toolar clicked; open find
- * dialog
- */
-void
-window_find_button_clicked_cb (GtkButton *button, GaleonWindow *window)
-{
-	find_show_dialog (window);
-}
-
-/**
- * window_drag_data_received_cb: handle dropped data
- */
-void
-window_drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
-			      gint x, gint y, GtkSelectionData *selection_data,
-			      guint info, guint time, GaleonWindow *window)
-{
-	switch (info)
-	{
-		case DND_TARGET_GALEON_EMBED:
-			{
-				GaleonEmbed *embed;
-
-				return_if_not_window (window);
-
-				/* get embed address */
-				memcpy (&embed, selection_data->data,
-					sizeof (GaleonEmbed *));
-				return_if_not_embed (embed);
-
-				embed_move_tab (embed, window->active_embed);
-				break;
-			}
-	}
-}
-
 /**
  * window_history_button_clicked_cb: history button clicked, show docked
  * history view unless already shown, in which case hide it.
@@ -1256,4 +1233,514 @@ window_session_history_menuitem_activate_cb (GtkMenuItem *menuitem,
 	session_history_load_index (index);
 }
 
+/**
+ * window_notebook_switch_page: called in tabbed mode when the user
+ * selects a different browser tab
+ */
+void
+window_notebook_switch_page_cb (GtkNotebook *notebook, 
+			        GtkNotebookPage *page, guint page_num)
+{
+	GtkMozEmbed *mozEmbed;
+	GaleonWindow *window;
+	GaleonEmbed *old_embed;
+	GaleonEmbed *embed;
 
+	g_return_if_fail (notebook != NULL);
+	g_return_if_fail (page != NULL);
+
+	/* check we REALLY REALLY have switched page */
+	if (GTK_NOTEBOOK (notebook)->cur_page != page)
+	{
+		return;
+	}
+
+	/* find the GtkMozEmbed from the page */
+	mozEmbed = (GtkMozEmbed *)page->child;
+	g_return_if_fail (GTK_IS_MOZ_EMBED (mozEmbed));
+
+	/* find the GaleonEmbed from the GtkMozEmbed */
+	embed = gtk_object_get_data (GTK_OBJECT (mozEmbed), "GaleonEmbed");
+	return_if_not_embed (embed);
+
+	/* find the GaleonWindow form the GaleonEmbed */
+	window = embed->parent_window;
+	return_if_not_window (window);
+
+	/* if not viewed this before we need to change state */
+	if (!(embed->has_been_viewed))
+	{
+		embed->has_been_viewed = TRUE;
+	}
+
+	/* set this as the active embed and save the content of the
+	 * location bar of the old embed */
+	old_embed = window->active_embed;
+	if (old_embed != NULL)
+	{
+		/* save edited site location information */
+		save_site_location (old_embed);
+
+		/* no longer the active window */
+		old_embed->is_active = FALSE;
+		embed_update_tab_status (old_embed);
+	}
+
+	/* this is now the active window */
+	embed->is_active = TRUE;
+	window->active_embed = embed;
+
+	/* move this embed to front of tab list to maintain stacking order */
+	window->embed_list = g_list_remove (window->embed_list, embed);
+	window->embed_list = g_list_prepend (window->embed_list, embed);
+
+	/* update tab label colour and closebutton status */
+	embed_update_tab_status (embed);
+
+	/* set the global title and location -- this is the lazy way! */
+	embed_update_page_title (embed);
+	embed_update_page_location (embed);
+
+	/* update toolbar/menu status */
+	window_update_nav_controls (window);
+	window_update_tab_controls (window);
+
+	/* update the status bar */
+	window_update_status_bar (window);
+
+	/* update the zoom control */
+	window_update_zoom (window);
+
+	/* set focus to the embed when clicking on a tab */
+	/* it's not grabbing the focus when opening a new tab
+	   from context menu, actually there is not a good way
+	   to do that */
+	if (embed->wrapper && window->visible_embeds > 1)
+	{
+		if (old_embed != NULL)
+		{
+			mozilla_deactivate (old_embed);
+		}
+		
+		mozilla_activate (embed);
+	}
+
+	/* we assume the user requested this focus by default */
+	embed->focus_type = FOCUS_ON_REQUEST;
+}
+
+/**
+ * window_notebook_add_remove_page_cb: called when a page is either added
+ * to or removed from the main notebook. Simply updates the tab controls
+ * on the menus.
+ */
+void
+window_notebook_add_remove_page_cb (GtkContainer *container,
+				    GtkWidget *widget, GaleonWindow *window)
+{
+	return_if_not_window (window);
+	window_update_tab_controls (window);
+}
+
+/**
+ * window_notebook_button_press_cb: initiates pseudo-drag events from the
+ * notebook tabs -- attaches motion notify and button release signals to
+ * the notebook */
+gint window_notebook_button_press_cb (GtkWidget *widget, GdkEventButton *e,
+				      GaleonWindow *window)
+{
+	if (e->button != 1 || e->type != GDK_BUTTON_PRESS) return FALSE;
+	
+	if (e->window != widget->window || !window ||
+	    window->magic != GALEON_WINDOW_MAGIC)
+	{
+		printf ("warning: window_notebook_button_press_cb: "
+			"sanity check failed.  e->window=%p, "
+			"widget->window=%p, window=%p\n", e->window,
+			widget->window, window);
+		return FALSE;
+	}
+
+	if (!(window->active_embed) ||
+	    window->active_embed->magic != GALEON_EMBED_MAGIC ||
+	    window->drag_motion_signal || window->drag_release_signal)
+	{
+		printf ("warning: window_notebook_button_press_cb: "
+			"sanity check failed.  window->active_embed=%p, "
+			"window->drag_motion_signal=%d, "
+			"window->drag_release_signal=%d\n",
+			window->active_embed, window->drag_motion_signal,
+			window->drag_release_signal);
+		return FALSE;
+	}
+
+	/* grab the mouse and change the cursor */
+	if (!drag_cursor) drag_cursor = gdk_cursor_new (GDK_FLEUR);
+
+	gtk_grab_add (window->WMain);
+	gdk_pointer_grab (window->WMain->window, FALSE,
+		GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+		NULL, drag_cursor, GDK_CURRENT_TIME);
+	window->drag_motion_signal = gtk_signal_connect (
+		GTK_OBJECT (window->WMain), "motion_notify_event",
+		GTK_SIGNAL_FUNC (window_notebook_motion_cb),
+		window->active_embed);
+	window->drag_release_signal = gtk_signal_connect (
+		GTK_OBJECT (window->WMain), "button_release_event",
+		GTK_SIGNAL_FUNC (window_notebook_button_release_cb),
+		window->active_embed);
+
+	return FALSE;
+}
+
+/**
+ * window_notebook_button_press_cb: handles "drag" events on the tabs.
+ * this function only handles moving the tab from one window to another;
+ * new window creation is handled on button release */
+gint window_notebook_motion_cb (GtkWidget *widget, GdkEventMotion *e,
+				GaleonEmbed *embed)
+{
+	GdkWindow *gdk_window;
+	GList *l;
+	gint x, y, x_rel, y_rel;
+	GaleonWindow *window, *dest_window = NULL;
+	GtkNotebook *dest_notebook;
+	GaleonEmbed *dest_embed = NULL;
+	GtkNotebookPage *page;
+	GtkWidget *tab, *mozembed;
+	gint page_num, dest_page_num = 0;
+
+	/* make sure the event is valid, and that we've received an embed */
+	if (e->window != widget->window || !embed ||
+	    embed->magic != GALEON_EMBED_MAGIC)
+	{
+		printf ("warning: window_notebook_motion_cb: "
+			"sanity check failed.  e->window=%p, "
+			"widget->window=%p, embed=%p\n",
+			e->window, widget->window, embed);
+		gdk_pointer_ungrab (GDK_CURRENT_TIME);
+		return FALSE;
+	}
+
+	/* get the source window */
+	window = embed->parent_window;
+	if (!window || window->magic != GALEON_WINDOW_MAGIC)
+	{
+		printf ("warning: window_notebook_motion_cb: "
+			"sanity check failed.  window=%p\n", window);
+		gdk_pointer_ungrab (GDK_CURRENT_TIME);
+		return FALSE;
+	}
+
+	/* get position of release */
+	x = e->x_root;
+	y = e->y_root;
+	x_rel = e->x;
+	y_rel = e->y;
+
+	/* find out which window (if any) the embed was dropped in */
+	gdk_window = gdk_window_at_pointer (&x, &y);
+	if (gdk_window) gdk_window = gdk_window_get_toplevel (gdk_window);
+	l = all_windows;
+	while (l)
+	{
+		if (gdk_window == ((GaleonWindow *) l->data)->WMain->window)
+		{
+			dest_window = l->data;
+			break;
+		}
+		l = l->next;
+	}
+
+	/* return if they haven't dragged into another galeon window */
+	if (!dest_window) return FALSE;
+			
+	if (dest_window->magic != GALEON_WINDOW_MAGIC)
+	{
+		printf ("warning: window_notebook_motion_cb: "
+			"sanity check failed.  dest_window=%p\n",
+			dest_window);
+		gdk_pointer_ungrab (GDK_CURRENT_TIME);
+		return FALSE;
+	}
+
+	/* determine which tab the cursor is over */
+	dest_notebook = GTK_NOTEBOOK (dest_window->notebook);
+	for (l = dest_notebook->children, page_num = 0;
+	     l; l = l->next, page_num++)
+	{
+		page = l->data;
+		tab = page->tab_label;
+
+		if (GTK_WIDGET_VISIBLE (tab))
+		{
+			/* handle top/bottom and left/right tabs
+			 * appropriately */
+			if (dest_notebook->tab_pos == GTK_POS_TOP ||
+			    dest_notebook->tab_pos == GTK_POS_BOTTOM)
+			{
+				if (tab->allocation.x <= x_rel)
+					dest_page_num = page_num;
+				else break;
+			}
+			else
+			{
+				if (tab->allocation.y <= y_rel)
+					dest_page_num = page_num;
+				else break;
+			}
+		}
+	}
+
+	/* find the embed associated with that tab */
+	mozembed = gtk_notebook_get_nth_page (dest_notebook,
+		dest_page_num);
+	if (mozembed)
+	{
+		dest_embed = gtk_object_get_data (
+			GTK_OBJECT (mozembed), "GaleonEmbed");
+	}
+
+	if (!dest_embed || dest_embed->magic != GALEON_EMBED_MAGIC)
+	{
+		printf ("warning: window_notebook_motion_cb: "
+			"sanity check failed.  dest_embed=%p\n",
+			dest_embed);
+		gdk_pointer_ungrab (GDK_CURRENT_TIME);
+		return FALSE;
+	}
+
+	/* do nothing if the dest embed is the same as the embed */
+	if (dest_embed == embed) return FALSE;
+
+	/* if we're moving the tab to another window... */
+	if (window != dest_window)
+	{
+		/* grab the pointer for the new notebook */
+		gtk_grab_add (dest_window->WMain);
+		gdk_pointer_grab (dest_window->WMain->window,
+			FALSE, GDK_POINTER_MOTION_MASK |
+			GDK_BUTTON_RELEASE_MASK,
+			NULL, drag_cursor, GDK_CURRENT_TIME);
+		dest_window->drag_motion_signal = gtk_signal_connect (
+			GTK_OBJECT (dest_window->WMain),
+			"motion_notify_event",
+			GTK_SIGNAL_FUNC (window_notebook_motion_cb),
+			embed);
+		dest_window->drag_release_signal = gtk_signal_connect (
+			GTK_OBJECT (dest_window->WMain),
+			"button_release_event",
+			GTK_SIGNAL_FUNC (window_notebook_button_release_cb),
+			embed);
+
+		/* ungrab the pointer for the old notebook */
+		gtk_signal_disconnect (GTK_OBJECT (widget),
+			window->drag_motion_signal);
+		gtk_signal_disconnect (GTK_OBJECT (widget),
+			window->drag_release_signal);
+		gtk_grab_remove (widget);
+		window->drag_motion_signal = 0;
+		window->drag_release_signal = 0;
+	}
+
+	/* move the embed */
+	embed_move_tab (embed, dest_embed);
+
+	return FALSE;
+}
+
+/**
+ * window_notebook_button_release_cb: ends pseudo-drag events from the
+ * notebook tabs, handling cases where the tab has been dropped outside of
+ * a galeon window */
+gint window_notebook_button_release_cb (GtkWidget *widget, GdkEventButton *e,
+					GaleonEmbed *embed)
+{
+	gint x, y;
+	GdkWindow *gdk_window;
+	GList *l;
+	GaleonWindow *window, *dest_window = NULL;
+
+	if (e->button != 1 || e->type != GDK_BUTTON_RELEASE) return FALSE;
+
+	/* make sure event is valid and we got an embed */
+	if (e->window != widget->window || !embed ||
+	    embed->magic != GALEON_EMBED_MAGIC)
+	{
+		printf ("warning: window_notebook_button_release_cb: "
+			"sanity check failed.  e->button=%d, e->window=%p "
+			"widget->window=%p, embed=%p\n", e->button,
+			e->window, widget->window, embed);
+		gdk_pointer_ungrab (GDK_CURRENT_TIME);
+		return FALSE;
+	}
+
+	/* get the source window, and make sure that it's valid and we're
+	 * already in a drag */
+	window = embed->parent_window;
+	if (!window || window->magic != GALEON_WINDOW_MAGIC ||
+	    !(window->drag_motion_signal) || !(window->drag_release_signal))
+	{
+		printf ("warning: window_notebook_button_release_cb: "
+			"sanity check failed.  window=%p\n", window);
+		gdk_pointer_ungrab (GDK_CURRENT_TIME);
+		return FALSE;
+	}
+
+	/* ungrab pointer */
+	gtk_signal_disconnect (GTK_OBJECT (widget),
+		window->drag_motion_signal);
+	gtk_signal_disconnect (GTK_OBJECT (widget),
+		window->drag_release_signal);
+	gdk_pointer_ungrab (GDK_CURRENT_TIME);
+	gtk_grab_remove (widget);
+	window->drag_motion_signal = 0;
+	window->drag_release_signal = 0;
+
+	/* get position of release */
+	x = e->x_root;
+	y = e->y_root;
+
+	/* find out which window (if any) the embed was dropped in */
+	gdk_window = gdk_window_at_pointer (&x, &y);
+	if (gdk_window) gdk_window = gdk_window_get_toplevel (gdk_window);
+	l = all_windows;
+	while (l)
+	{
+		if (gdk_window == ((GaleonWindow *) l->data)->WMain->window)
+		{
+			dest_window = l->data;
+			break;
+		}
+		l = l->next;
+	}
+
+	/* if we're NOT moving to a new window here, or we're the only
+	 * embed in our window, just return */
+	if (dest_window != NULL ||
+		g_list_length (embed->parent_window->embed_list) == 1)
+		return FALSE;
+
+	/* move the embed */
+	embed_move_tab (embed, NULL);
+
+	/* move the new window under the pointer */
+	gdk_window = GTK_WIDGET (embed->parent_window->WMain)->window;
+	gdk_window_move (gdk_window, e->x_root, e->y_root);
+
+	return FALSE;
+}
+
+gboolean window_key_press_event (GtkWidget *widget,
+				 GdkEventKey *event,
+				 GaleonWindow *window)
+{
+	int page;
+
+	if ((event->state & GDK_Alt_L) || (event->state & GDK_Alt_R))
+	{
+		page = event->keyval - GDK_0 -1;
+
+		if (page == -1) page = 9;
+
+		if (page>=-1 && page<=9)
+		{
+			gtk_notebook_set_page (GTK_NOTEBOOK (window->notebook),
+					       page == -1 ? -1 : page);
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
+void
+window_move_tab_to_window_menu_cb (GtkMenuItem *menuitem, 
+				   GaleonWindow *window)
+{
+	GtkMenu *menu;
+	GList *child_list;
+	GList *li;
+	if (menuitem->submenu == NULL) 
+	{
+		menu = GTK_MENU (gtk_menu_new ());
+		gtk_menu_item_set_submenu (menuitem, GTK_WIDGET (menu));
+	}
+	else
+	{
+		menu = GTK_MENU (menuitem->submenu);
+	}
+
+	/* remove old menu items */
+	child_list = gtk_container_children (GTK_CONTAINER (menu));
+	for (li = child_list; li != NULL; li = li->next)
+	{
+		gtk_widget_destroy (GTK_WIDGET (li->data));
+	}
+	g_list_free (child_list);
+
+	/* create menu items */
+	for (li = all_windows; li != NULL; li = li->next)
+	{
+		GaleonWindow *w;
+		GaleonEmbed *e;
+		gchar *title;
+		GtkWidget *item;
+		w = li->data;
+		if ((w == NULL) || (w->magic != GALEON_WINDOW_MAGIC))
+			continue;
+		e = w->active_embed;
+		if ((e == NULL) || (e->magic != GALEON_EMBED_MAGIC))
+			continue;
+		title = e->site_title;
+		if (title == NULL)
+			continue;
+		item = gtk_menu_item_new_with_label (title);
+		gtk_object_set_user_data (GTK_OBJECT (item), w);
+		gtk_widget_show (item);
+		gtk_menu_append (menu, item);
+		gtk_signal_connect (GTK_OBJECT (item), "activate", 
+				    window_move_tab_to_window_menu_item_cb,
+				    window);
+		if (w == window)
+			gtk_widget_set_sensitive (item, FALSE);
+	}
+	gtk_menu_reposition (menu);
+}
+
+static void
+window_move_tab_to_window_menu_item_cb (GtkMenuItem *mi, 
+					GaleonWindow *w)
+{
+	GaleonWindow *dest_w;
+	GaleonEmbed *dest_e;
+	GaleonWindow *source_w;
+	GaleonEmbed *source_e;
+	dest_w = gtk_object_get_user_data (GTK_OBJECT (mi));
+	return_if_not_window (dest_w);
+	dest_e = dest_w->active_embed;
+	return_if_not_embed (dest_e);
+	source_w = w;
+	return_if_not_window (source_w);
+	source_e = w->active_embed;
+	return_if_not_embed (source_e);
+	embed_move_tab (source_e, dest_e);
+}
+
+gboolean
+window_find_entry_activate_cb (GtkWidget *widget, GaleonWindow *window)
+{
+	gchar *text;
+	GaleonEmbed *embed;
+
+	return_val_if_not_window (window, FALSE);
+	embed = window->active_embed;
+	return_val_if_not_embed (embed, FALSE);
+	
+	/* get text and search */
+	text = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
+	find_next (embed, text);
+	g_free (text);
+
+	return TRUE;
+}
